// -*-C++-*-
/*!
 * @file  HiroGateRTCSVC_impl.cpp
 * @brief Service implementation code of HiroGateRTC.idl
 *
 */

#include "HiroGateRTCSVC_impl.h"
#include <cmath>
#include <fstream>
#include <string>
#include "../VSCommon/system/misc/misc.h"
#include "hgutil.h"


#if defined( _V0514_2 ) || defined( _V0514_5 ) || defined( _V0514_6 )
using namespace ikfast;
using namespace opencv;
using namespace std;
#endif

using namespace rtcutil;


/*
 * Example implementational code for IDL interface HiroGateRTC::HiroGateService
 */
HiroGateServiceSVC_impl::HiroGateServiceSVC_impl()
{
  // Please add extra constructor code here.
}

// -------- HiroGateService -------- //
HiroGateServiceSVC_impl::HiroGateServiceSVC_impl(
	RTC::CorbaConsumer<OpenHRP::RobotHardwareService> *rbhw_if,
	RTC::CorbaConsumer<OpenHRP::SequencePlayerService> *sqply_if,
	RTC::CorbaConsumer<OpenHRP::ArmControlService> *acsL_if,
	RTC::CorbaConsumer<OpenHRP::ArmControlService> *acsR_if,
	RTC::CorbaConsumer<OpenHRP::GrasperService> *gs_if )
	: svc_ptrs( rbhw_if, sqply_if, acsL_if, acsR_if, gs_if )
{
#	if defined( _V0514_2 ) || defined( _V0514_5 ) || defined( _V0514_6 )
	if( hiro::larm::getNumJoints() != IKBase::N_JOINTS )
		sys::exit( "HiroGateServiceSVC_impl: constructor: # joints of L arm not match." );
	if( hiro::rarm::getNumJoints() != IKBase::N_JOINTS )
		sys::exit( "HiroGateServiceSVC_impl: constructor: # joints of R arm not match." );
#	endif
}
bool HiroGateServiceSVC_impl::loadPoseDB(const char *file_path)
{
	std::ifstream ifs( file_path );
	if( !ifs ) {
		perror( "HiroGateServiceSVC_impl::loadPoseDB(): " );
		return false;
	}

	std::string line;
	
	std::getline(ifs, line);	// 擪s̓ǂݎ̂

	while( std::getline(ifs, line) ) {

		if( line == "" ) break;
		
		char *angles_buf = new char [line.length()+1];
		char *angles;

		/* id,name,,... */
		// id,
		int id;
		if( sscanf( line.c_str(), "%d,,", &id ) != 1 ) 
			return sys::returnByError<bool>( false, 
				"HiroGateServiceSVC_impl::loadPoseDB(): id field error\n" );

		pdb.length( id + 1 );

		strcpy( angles_buf, &(line.c_str()[index(line.c_str(), ',')+2]) );
			// line.c_str()[index(line.c_str(), ',')+1]
			//	0,,0,...
			//     ~

		angles = &angles_buf[0];

		/* ֐ߊeǂݍ */
		int part_idx;
		double temp[6];
		
		pdb[id].length(5);

		/* CY,NY,NP,, */
		part_idx = 0;
		if( sscanf( angles, "%lf,%lf,%lf,,",
			&temp[0], &temp[1], &temp[2] ) == 3 )
		{
			for( int i = 0 ; i < 3; ++ i ) temp[i] = M_PI*temp[i]/180.0;
			storeJointPartAngles( id, part_idx, 3, temp );
		}
		else
			storeJointPartAngles( id, part_idx, 0, NULL );

		// ',' ǂ݂Ԃ
		for( int i = 0; i <= 3; ++ i )	angles = &angles[index(angles, ',')+1];

		/* RSY,RSP,REP,RWY,RWP,RWR,, */
		part_idx = 1;
		if( sscanf( angles, "%lf,%lf,%lf,%lf,%lf,%lf,,",
			&temp[0], &temp[1], &temp[2], &temp[3], &temp[4], &temp[5] ) == 6 )
		{
			for( int i = 0 ; i < 6; ++ i ) temp[i] = M_PI*temp[i]/180.0;
			storeJointPartAngles( id, part_idx, 6, temp );
		}
		else
			storeJointPartAngles( id, part_idx, 0, NULL );

		// ',' ǂ݂Ԃ
		for( int i = 0; i <= 6; ++ i )	angles = &angles[index(angles, ',')+1];
		
		/* LSY,LSP,LEP,LWY,LWP,LWR,, */
		part_idx = 2;
		if( sscanf( angles, "%lf,%lf,%lf,%lf,%lf,%lf,,",
			&temp[0], &temp[1], &temp[2], &temp[3], &temp[4], &temp[5] ) == 6 )
		{
			for( int i = 0 ; i < 6; ++ i ) temp[i] = M_PI*temp[i]/180.0;
			storeJointPartAngles( id, part_idx, 6, temp );
		}
		else
			storeJointPartAngles( id, part_idx, 0, NULL );

		// ',' ǂ݂Ԃ
		for( int i = 0; i <= 6; ++ i )	angles = &angles[index(angles, ',')+1];

		/* RH00,RH01,RH10,RH11,, */
		part_idx = 3;
		if( sscanf( angles, "%lf,%lf,%lf,%lf,,",
			&temp[0], &temp[1], &temp[2], &temp[3] ) == 4 )
		{
			for( int i = 0 ; i < 4; ++ i ) temp[i] = M_PI*temp[i]/180.0;
			storeJointPartAngles( id, part_idx, 4, temp );
		}
		else
			storeJointPartAngles( id, part_idx, 0, NULL );

		// ',' ǂ݂Ԃ
		for( int i = 0; i <= 4; ++ i )	angles = &angles[index(angles, ',')+1];

		/* LH00,LH01,LH10,LH11 */
		part_idx = 4;
		if( sscanf( angles, "%lf,%lf,%lf,%lf,,",
			&temp[0], &temp[1], &temp[2], &temp[3] ) == 4 )
		{
			for( int i = 0 ; i < 4; ++ i ) temp[i] = M_PI*temp[i]/180.0;
			storeJointPartAngles( id, part_idx, 4, temp );
		}
		else
			storeJointPartAngles( id, part_idx, 0, NULL );


		delete[] angles_buf;

	}
	ifs.close();

	for( CORBA::ULong rec_i = 0; rec_i < pdb.length(); ++ rec_i )
		printf(
			"\tPoseDB[%2d]: %d:%d:%d:%d:%d\n",
			rec_i,
			pdb[rec_i][0].length(), pdb[rec_i][1].length(),
			pdb[rec_i][2].length(), pdb[rec_i][3].length(),
			pdb[rec_i][4].length() );

	return true;
}
// -------- HiroGateService -------- //

HiroGateServiceSVC_impl::~HiroGateServiceSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */



/*=========================================================
=========================================================*/
CORBA::Long HiroGateServiceSVC_impl::getJointAngles(
	HiroGateRTC::HiroGateService::JointAngles_out angles_rad)
{
  if( CORBA::is_nil( svc_ptrs.rbhw_if->getObject() ) )
		return 0x80000000;
	
	OpenHRP::RobotHardwareService::RobotState_var state;
	(*svc_ptrs.rbhw_if)->getStatus( state );

	//for( int i(0), n(state->angle.length()); i < n; ++ i ) {
	//	std::cout << "[" << i << "]: " << state->angle[i] << std::endl;
	//}

	//printf("A\n");

#	if 1

	// ܂m
	angles_rad = new HiroGateRTC::HiroGateService::JointAngles();

	// ʏ̃V[PX̊m
	angles_rad->length(5);
	(*angles_rad)[0].length(3);
	(*angles_rad)[1].length(6), (*angles_rad)[2].length(6);
	(*angles_rad)[3].length(4), (*angles_rad)[4].length(4);

	// 
	for( int i = 0, n = 3; i < n; ++ i ) (*angles_rad)[0][i] = state->angle[i+0];
	for( int i = 0, n = 6; i < n; ++ i ) (*angles_rad)[1][i] = state->angle[i+3];
	for( int i = 0, n = 6; i < n; ++ i ) (*angles_rad)[2][i] = state->angle[i+3+6];
	for( int i = 0, n = 4; i < n; ++ i ) (*angles_rad)[3][i] = state->angle[i+3+6+6];
	for( int i = 0, n = 4; i < n; ++ i ) (*angles_rad)[4][i] = state->angle[i+3+6+6+4];

#	else

	HiroGateRTC::HiroGateService::JointAngles temp;

	// ʏ̃V[PX̊m
	temp.length(5);
	temp[0].length(3);
	temp[1].length(6), temp[2].length(6);
	temp[3].length(4), temp[4].length(4);

	// 
	for( int i = 0, n = 3; i < n; ++ i ) temp[0][i] = state->angle[i+0];
	for( int i = 0, n = 6; i < n; ++ i ) temp[1][i] = state->angle[i+3];
	for( int i = 0, n = 6; i < n; ++ i ) temp[2][i] = state->angle[i+3+6];
	for( int i = 0, n = 4; i < n; ++ i ) temp[3][i] = state->angle[i+3+6+6];
	for( int i = 0, n = 4; i < n; ++ i ) temp[4][i] = state->angle[i+3+6+6+4];

	// Rs[̃q[v蓖
	angles_rad = new HiroGateRTC::HiroGateService::JointAngles(temp);

#endif

	// 1bŐ\bԌƂ냁[N͂ȂɌ

  return 0;
}


/*=========================================================
	쒆ǂ̃`FbN
=========================================================*/
CORBA::Boolean HiroGateServiceSVC_impl::isHandMovingL()
{	
#	if defined( _V0514_2 )
	return isJointMoving();
#	else
	if( CORBA::is_nil( svc_ptrs.acsL_if->getObject() ) )
		return true;
	return (*svc_ptrs.acsL_if)->checkStatus() == OpenHRP::ArmBusyState;
#endif
}
CORBA::Boolean HiroGateServiceSVC_impl::isHandMovingR()
{
#	if defined( _V0514_2 )
	return isJointMoving();
#	else
	if( CORBA::is_nil( svc_ptrs.acsR_if->getObject() ) )
		return true;
	return (*svc_ptrs.acsR_if)->checkStatus() == OpenHRP::ArmBusyState;
#	endif
}


CORBA::Boolean HiroGateServiceSVC_impl::isJointMoving()
{
	if( CORBA::is_nil( svc_ptrs.sqply_if->getObject() ) )
		return true;
	return (*svc_ptrs.sqply_if)->isEmpty( 0x1F, false ) ? false : true;
}
CORBA::Boolean HiroGateServiceSVC_impl::isFingerMovingL()
{
	if( CORBA::is_nil( svc_ptrs.gs_if->getObject() ) )
		return true;
	return (*svc_ptrs.gs_if)->isEmpty( 0x1F, false ) ? false : true;
}


/*=========================================================
	ی~
=========================================================*/
CORBA::Long HiroGateServiceSVC_impl::engageProtectiveStop()
{
	if( !isInOperation( *svc_ptrs.rbhw_if ) )
		return -1;
	
	if( (*svc_ptrs.rbhw_if)->engageProtectiveStop() )
		return -2;
	
	return 0;
}


/*=========================================================
	ی~
=========================================================*/
CORBA::Long HiroGateServiceSVC_impl::disengageProtectiveStop()
{
	if( !isInOperation( *svc_ptrs.rbhw_if ) )
		return -1;
	
	if( (*svc_ptrs.rbhw_if)->disengageProtectiveStop() )
		return -2;
	
	return 0;
}


/*=========================================================
	T[{ON/OFF
=========================================================*/
CORBA::Long HiroGateServiceSVC_impl::turnServo(
	CORBA::Boolean body_head_on, CORBA::Boolean armL_on, CORBA::Boolean armR_on, 
	CORBA::Boolean handL_on, CORBA::Boolean handR_on )
{
	if( !isInOperation( *svc_ptrs.rbhw_if ) )
		return -1;

	if( servo( "BODY", body_head_on, -1 ) )	return -2;
	if( servo( "RARM", armR_on, -1 ) )			return -3;
	if( servo( "LARM", armL_on, -1 ) )			return -4;
	if( servo( "RHAND", handR_on, -1 ) )		return -5;
	if( servo( "LHAND", handL_on, -1 ) )		return -6;

	return 0;
}
long HiroGateServiceSVC_impl::servo(
	const char *part_name, bool on, long rc_by_error )
{
	return (*svc_ptrs.rbhw_if)->servo(
		part_name, on ? 
		OpenHRP::RobotHardwareService::SwitchStatus::SWITCH_ON : 
		OpenHRP::RobotHardwareService::SwitchStatus::SWITCH_OFF ) ?
			0 : rc_by_error;
}


/*=========================================================
	pf[^x[X()
=========================================================*/
CORBA::Long HiroGateServiceSVC_impl::setPose(
	CORBA::Long pose_id, 
	CORBA::Double t, CORBA::Boolean wait_for_motion)
{
#	if 0
	if( CORBA::is_nil( svc_ptrs.sqply_if->getObject() ) )
		return -1;

	if( !(pose_id < pdb.length()) )
		return -2;

	/*
	for(int pi = 0; pi < pdb.length(); ++ pi ){
		for(int bi = 0; bi < pdb[pi].length(); ++ bi ){
			printf("pdb[%d][%d]:", pi, bi );
			for(int ji = 0; ji < pdb[pi][bi].length(); ++ ji ){
				printf("%.3f,", 180*pdb[pi][bi][ji] / 3.151492 );
			}
			printf("\n");
		}
	}
	printf("\n");
	*/

  return setJointAngles( pdb[pose_id], t, wait_for_motion );
#	else
	return 0;
#	endif
}

#if 0
CORBA::Long HiroGateServiceSVC_impl::outXperimental(
	HiroGateRTC::HiroGateService::Xperimental_out xperimental)
{
	// ܂m
	xperimental = new HiroGateRTC::HiroGateService::Xperimental();

	
	return 0;
}
#endif

// End of example implementational code



