// -*- C++ -*-
/*!
 * @file  HiroGate.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "HiroGate.h"
#include <iostream>

using std::cout;
using std::endl;

// Module specification
// <rtc-template block="module_spec">
static const char* hirogate_spec[] =
  {
    "implementation_id", "HiroGate",
    "type_name",         "HiroGate",
    "description",       "Hiro Gateway RTC",
#		if defined( _V0514_6 )
		// setHandPoseL/R = setTargetAngular
		"version",           "0.5.14.6",
#		elif defined( _V0514_5 )
		// setHandPoseL/R = IKmF+setTargetAngular
		"version",           "0.5.14.5",
#		elif defined( _V0514_2 )
		// setHandPoseL/R = IKmF+setJointAngles
    "version",           "0.5.14.2",
#		else
    "version",           "0.5.11.0",
#		endif
    "vendor",            "AISL.TUT",
    "category",          "Hiro",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.PoseDB", "pose-db.csv",
    // Widget
    "conf.__widget__.PoseDB", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
HiroGate::HiroGate(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_RobotHWSvcPort("RobotHWSvc"),
    m_SeqPlayerSvcPort("SeqPlayerSvc"),
    m_ArmCtrlSvcLPort("ArmCtrlSvcL"),
    m_ArmCtrlSvcRPort("ArmCtrlSvcR"),
    m_GrasperSvcPort("GrasperSvc"),
    m_HiroGateServicePort("HiroGateService")
    // </rtc-template>
		,
		// -------- HiroGateService -------- //
		m_hirogs_if( &m_rbhw_if, &m_sqply_if, &m_acsL_if, &m_acsR_if, &m_gs_if )
		// -------- HiroGateService -------- //
{
}

/*!
 * @brief destructor
 */
HiroGate::~HiroGate()
{
}



RTC::ReturnCode_t HiroGate::onInitialize()
{
	cout << ">> HiroGate::onInitialize()" << endl;
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  m_HiroGateServicePort.registerProvider("HiroGateSvcIF", "HiroGateRTC::HiroGateService", m_hirogs_if);
  
  // Set service consumers to Ports
  m_RobotHWSvcPort.registerConsumer("service0", "RobotHardwareService", m_rbhw_if);
  m_SeqPlayerSvcPort.registerConsumer("service0", "SequencePlayerService", m_sqply_if);
  m_ArmCtrlSvcLPort.registerConsumer("service0", "ArmControlService", m_acsL_if);
  m_ArmCtrlSvcRPort.registerConsumer("service0", "ArmControlService", m_acsR_if);
  m_GrasperSvcPort.registerConsumer("service0", "GrasperService", m_gs_if);
  
  // Set CORBA Service Ports
  addPort(m_RobotHWSvcPort);
  addPort(m_SeqPlayerSvcPort);
  addPort(m_ArmCtrlSvcLPort);
  addPort(m_ArmCtrlSvcRPort);
  addPort(m_GrasperSvcPort);
  addPort(m_HiroGateServicePort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("PoseDB", m_pdb_file_path, "pose-db.csv");
  //m_hirogs_if.loadPoseDB(m_pdb_file_path.c_str());
  // </rtc-template>
	cout << "<< HiroGate::onInitialize()" << endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t HiroGate::onFinalize()
{
	cout << ">> HiroGate::onFinalize()" << endl;
	cout << "<< HiroGate::onFinalize()" << endl;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HiroGate::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroGate::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t HiroGate::onActivated(RTC::UniqueId ec_id)
{
	cout << ">> HiroGate::onActivated()" << endl;
	cout << "<< HiroGate::onActivated()" << endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t HiroGate::onDeactivated(RTC::UniqueId ec_id)
{
	cout << ">> HiroGate::onDeactivated()" << endl;
	cout << "<< HiroGate::onDeactivated()" << endl;
  return RTC::RTC_OK;
}


double r2d( double rad )
{
	return rad*180.0/M_PI;
}


RTC::ReturnCode_t HiroGate::onExecute(RTC::UniqueId ec_id)
{
#	if 1
	printf(
		"\n"
		" RobotH/W %s\n"
		" SeqPlayer%s\n"
		"ArmCtrl(L)%s\n"
		"ArmCtrl(R)%s\n"
		"  Grasper %s\n"
		"\n",
		CORBA::is_nil(m_rbhw_if.getObject())  ? "" : "",
		CORBA::is_nil(m_sqply_if.getObject()) ? "" : "",
		CORBA::is_nil(m_acsL_if.getObject())  ? "" : "",
		CORBA::is_nil(m_acsR_if.getObject())  ? "" : "",
		CORBA::is_nil(m_gs_if.getObject())    ? "" : "" );

	HiroGateRTC::HiroGateService::JointAngles_var angles;
	if( ! m_hirogs_if.getJointAngles( angles.out() ) ) {
		printf( "[Body/Head]: %+3d, %+3d, %+3d\n", (int)r2d(angles[0][0]), (int)r2d(angles[0][1]), (int)r2d(angles[0][2]) );
		printf( "[ Arm   R ]: %+3d, %+3d, %+3d, %+3d, %+3d, %+3d\n", (int)r2d(angles[1][0]), (int)r2d(angles[1][1]), (int)r2d(angles[1][2]), (int)r2d(angles[1][3]), (int)r2d(angles[1][4]), (int)r2d(angles[1][5]) );
		printf( "[ Arm   L ]: %+3d, %+3d, %+3d, %+3d, %+3d, %+3d\n", (int)r2d(angles[2][0]), (int)r2d(angles[2][1]), (int)r2d(angles[2][2]), (int)r2d(angles[2][3]), (int)r2d(angles[2][4]), (int)r2d(angles[2][5]) );
		printf( "[ Hand  R ]: %+3d, %+3d, %+3d, %+3d\n", (int)r2d(angles[3][0]), (int)r2d(angles[3][1]), (int)r2d(angles[3][2]), (int)r2d(angles[3][3]));
		printf( "[ Hand  L ]: %+3d, %+3d, %+3d, %+3d\n", (int)r2d(angles[4][0]), (int)r2d(angles[4][1]), (int)r2d(angles[4][2]), (int)r2d(angles[4][3]));
	}
#	endif
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HiroGate::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroGate::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroGate::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroGate::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroGate::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void HiroGateInit(RTC::Manager* manager)
  {
    coil::Properties profile(hirogate_spec);
    manager->registerFactory(profile,
                             RTC::Create<HiroGate>,
                             RTC::Delete<HiroGate>);
  }
  
};


