#if defined( _V0514_2 ) || defined( _V0514_5 ) || defined( _V0514_6 )


#include "HiroGateRTCSVC_impl.h"
#include <cmath>
#include <fstream>
#include <string>
#include "../VSCommon/system/misc/misc.h"
#include "hgutil.h"


using namespace ikfast;
using namespace opencv;
using namespace std;


sys::ConsoleController cc;


const double HiroGateServiceSVC_impl::IKBase::RATE2SECOND = 50.0;


/*=========================================================
=========================================================*/
HiroGateServiceSVC_impl::IKBase::IKBase()
{
	//cs.initialize();
}


HiroGateServiceSVC_impl::IKBase::~IKBase()
{
	//cs.finalize();
}


void HiroGateServiceSVC_impl::IKBase::clear()
{
	solutions.clear();
	valid_indices.clear();
}


void HiroGateServiceSVC_impl::IKBase::convertSolutionToJointAngles( int part_idx )
{
	int i, n = angles[part_idx].length();
	for( i = 0; i < n; ++ i )
		angles[part_idx][i] = solution[i];
}


/*---------------------------------------------------------
	IK߂Ċ֐ߊpxŃtB^O
---------------------------------------------------------*/
long HiroGateServiceSVC_impl::IKBase::solve(
	const RTC::Pose3D& pose_m_rad,
	bool (*ik_solver)( const ikfast::IKReal*, const ikfast::IKReal*, const ikfast::IKReal*, std::vector< ikfast::IKSolution >& ),
	bool (*ik_filter)( const std::vector< ikfast::IKSolution >&, std::vector< int > & ) )
{
	clear();

	// ʒupϊ
	cvmath::Vector< CV_64F > t( 3 );
	cvmath::Matrix< CV_64F > R( 3, 3 );
	convertPose3DToRt( pose_m_rad, t, R );
	
	// IK̉߂
	if( !ik_solver( &t(0), &R(0,0), NULL, solutions ) )
		return 0;

	// ֐ߊpxŃtB^O
	if( ik_filter( solutions, valid_indices ) ) {
		solutions.at( valid_indices.at(0) ).GetSolution( solution, NULL );

		return (int) valid_indices.size();
	}
	
	return 0; // Solution not found.
}


/*---------------------------------------------------------
	ikfast::hiro::?arm::ik()p̃p[^ϊ
---------------------------------------------------------*/
void HiroGateServiceSVC_impl::IKBase::convertPose3DToRt(
	const RTC::Pose3D& pose_m_rad,
	opencv::cvmath::Vector< CV_64F > &t,
	opencv::cvmath::Matrix< CV_64F > &R )
{
	/* ʒu */
	t(0) = pose_m_rad.position.x;
	t(1) = pose_m_rad.position.y;
	t(2) = pose_m_rad.position.z;

	/* p */
	opencv::cvmath::Vector< CV_64F > axis( 3 );
	opencv::cvmath::Quaternion< CV_64F > qy, qp, qr, qyp, qypr;
	// yaw
	axis.setZero(); axis(2) = pose_m_rad.orientation.y; qy.fromAxis( axis );

	// pitch
	axis.setZero(); axis(1) = pose_m_rad.orientation.p; qp.fromAxis( axis );

	// pitch
	axis.setZero(); axis(0) = pose_m_rad.orientation.r; qr.fromAxis( axis );

	// 
	qyp.QxQ( qy, qp );
	qypr.QxQ( qyp, qr );

	qypr.toR( R );
}


/*=========================================================
	rikfast
=========================================================*/
HiroGateServiceSVC_impl::IKL::IKL()
{
	hgutil::allocateJointAngles( angles, false, true, false );
}


HiroGateServiceSVC_impl::IKL::~IKL()
{}


void HiroGateServiceSVC_impl::IKL::convertSolutionToJointAngles()
{
	IKBase::convertSolutionToJointAngles( hgutil::AL );
}


long HiroGateServiceSVC_impl::IKL::solve( const RTC::Pose3D& pose_m_rad )
{
	return IKBase::solve( pose_m_rad, hiro::larm::ik, hiro::larm::filter );
}


/*=========================================================
	Erikfast
=========================================================*/
HiroGateServiceSVC_impl::IKR::IKR()
{
	hgutil::allocateJointAngles( angles, false, false, true );
}


HiroGateServiceSVC_impl::IKR::~IKR()
{}


void HiroGateServiceSVC_impl::IKR::convertSolutionToJointAngles()
{
	IKBase::convertSolutionToJointAngles( hgutil::AR );
}


long HiroGateServiceSVC_impl::IKR::solve( const RTC::Pose3D& pose_m_rad )
{
	return IKBase::solve( pose_m_rad, hiro::rarm::ik, hiro::rarm::filter );
}


#if 1
/*=========================================================
	IK߂
=========================================================*/
CORBA::Boolean HiroGateServiceSVC_impl::getIKSolutionL(
	const RTC::Pose3D& poseR_l_m_rad, CORBA::Double torso_angle_rad,
	HiroGateRTC::HiroGateService::JointAngles_out angles_l_rad)
{
	IKL ikL;
	return getIKSolution( &ikL, poseR_l_m_rad, torso_angle_rad, angles_l_rad );
}


CORBA::Boolean HiroGateServiceSVC_impl::getIKSolutionR(
	const RTC::Pose3D& poseR_r_m_rad, CORBA::Double torso_angle_rad,
	HiroGateRTC::HiroGateService::JointAngles_out angles_r_rad)
{
	IKR ikR;
	return getIKSolution( &ikR, poseR_r_m_rad, torso_angle_rad, angles_r_rad );
}


/*---------------------------------------------------------
	getIKSolutionL/R()̋ʕ
---------------------------------------------------------*/
bool HiroGateServiceSVC_impl::getIKSolution(
	IKBase *ik, const RTC::Pose3D& pose_m_rad, CORBA::Double torso_angle_rad, 
	HiroGateRTC::HiroGateService::JointAngles_out &angles_rad )
{	
	//printf(">> HiroGateServiceSVC_impl::getIKSolution()\n");

	// sƂłmۂKv
	// ̓X^uR[hreturn̑O
	//  angles_r_rad = _call_desc.arg_1._retn();
	// 邽߂Ǝv
	angles_rad = new HiroGateRTC::HiroGateService::JointAngles();

	if( isIKSolutionFound( ik, pose_m_rad, torso_angle_rad ) ) {
		// IKJointAnglesɕϊ
		ik->convertSolutionToJointAngles();

		// OJointAnglesւ̊i[
		*angles_rad = ik->angles;
			// ^Ȃ̂ŁC̈s̑ŐmہEi[ł悤

		hgutil::printJointAngles( ik->angles );
		hgutil::printJointAngles( *angles_rad );

		//printf("<<[true] HiroGateServiceSVC_impl::getIKSolution()\n");
		return true;
	}
	//printf("<<[false] HiroGateServiceSVC_impl::getIKSolution()\n");
	return false;
}
#endif


/*=========================================================
	IK݂邩ǂ𒲂ׂ
=========================================================*/
CORBA::Boolean HiroGateServiceSVC_impl::isIKSolutionFoundL(
	const RTC::Pose3D& poseR_l_m_rad, CORBA::Double torso_angle_rad )
{
	IKL ikL;
	return isIKSolutionFound( &ikL, poseR_l_m_rad, torso_angle_rad );
}


CORBA::Boolean HiroGateServiceSVC_impl::isIKSolutionFoundR(
	const RTC::Pose3D& poseR_r_m_rad, CORBA::Double torso_angle_rad )
{
	IKR ikR;
	return isIKSolutionFound( &ikR, poseR_r_m_rad, torso_angle_rad );
}


/*---------------------------------------------------------
 isIKSolutionFoundL/R()̋ʕ
---------------------------------------------------------*/
bool HiroGateServiceSVC_impl::isIKSolutionFound(
	IKBase *ik, const RTC::Pose3D& poseR_m_rad, double torso_angle_rad )
{

	//cout << ">> isIKSolutionFound(): " << endl;
	//cout << "\ttorso: " << cvmath::rad2deg( torso_angle_rad ) << endl;

	// ikfastɂ͓̂̊֐߂͊܂܂Ȃ̂ŁC
	// {bgnR𓷑̌nBɕϊĂ₢킹
	RTC::Pose3D poseB_m_rad;
	hgutil::R2B( poseR_m_rad, torso_angle_rad, poseB_m_rad );

	//cout << "\t: poseR"; hgutil::printPose3D( poseR_m_rad );
	//cout << "\t: poseB"; hgutil::printPose3D( poseB_m_rad );

	//cout << "<< isIKSolutionFound(): " << endl;
	if( ik->solve( poseB_m_rad ) )
		return true;
	
	cc.setColor( cc.BLACK, cc.YELLOW );
	printf( "\aHiroGateServiceSVC_impl::isIKSolutionFound(): IK solution not found." );
	cc.setColor();
	putchar( '\n' );
	return false;
}


#endif