#include "HiroGateRTCSVC_impl.h"
#include <cmath>
#include <fstream>
#include <string>
#include "../VSCommon/system/misc/misc.h"
#include "hgutil.h"


#if defined( _V0514_2 ) || defined( _V0514_5 ) || defined( _V0514_6 )
using namespace ikfast;
using namespace opencv;
using namespace std;
#endif


/*=========================================================
	r/Erʒupw蓮
	@	return: 0; I / -1; rIKs / -2; ErIKs
=========================================================*/
CORBA::Long HiroGateServiceSVC_impl::setHandPoseL(
	const RTC::Pose3D& pose_l_m_rad, 
	CORBA::Double rate, CORBA::Boolean wait_for_motion )
{
# if defined( _V0514_5 ) //---------------------------------------
	
	IKL ikL;
	return setHandPose( svc_ptrs.acsL_if, &ikL, pose_l_m_rad, rate, wait_for_motion );

#	elif defined( _V0514_2 ) //---------------------------------------

	IKL ikL;
	return setHandPose( &ikL, pose_l_m_rad, rate, wait_for_motion );

#	else //----------------------------------------------------------------------

	return setHandPose( svc_ptrs.acsL_if, pose_l_m_rad, rate, wait_for_motion );

#	endif //---------------------------------------------------------------------
}
CORBA::Long HiroGateServiceSVC_impl::setHandPoseR(
	const RTC::Pose3D& pose_r_m_rad, 
	CORBA::Double rate, CORBA::Boolean wait_for_motion )
{
# if defined( _V0514_5 ) //---------------------------------------
	
	IKR ikR;
	return setHandPose( svc_ptrs.acsR_if, &ikR, pose_r_m_rad, rate, wait_for_motion );

#	elif defined( _V0514_2 ) //---------------------------------------

	IKR ikR;
	return setHandPose( &ikR, pose_r_m_rad, rate, wait_for_motion );

#	else //----------------------------------------------------------------------

	return setHandPose( svc_ptrs.acsR_if, pose_r_m_rad, rate, wait_for_motion );

#	endif //---------------------------------------------------------------------
}

/*---------------------------------------------------------
	setHandPoseL/R()̋ʕ
---------------------------------------------------------*/
#if defined( _V0514_5 ) //------------------------------------------

long HiroGateServiceSVC_impl::setHandPose(
	RTC::CorbaConsumer<OpenHRP::ArmControlService> *acs_if,	IKBase *ik, 
	const RTC::Pose3D& pose_m_rad, double rate, bool wait_for_motion )
{
	// ̊֐߂̊px擾
	OpenHRP::RobotHardwareService::RobotState_var state;
	(*svc_ptrs.rbhw_if)->getStatus( state );

	//printf( "setHandPose(): %f [deg]\n", cvmath::rad2deg( state->angle[0] ) );

	// ݂̓̊pxڕWʒupɓ͂ǂ𔻒
	if( isIKSolutionFound( ik, pose_m_rad, state->angle[0] ) )
		return setHandPose( acs_if, pose_m_rad, rate, wait_for_motion );
	return ik->returnIKFailed();
}

#endif //----------------------------------------------------------------------

#if defined( _V0514_2 ) //------------------------------------------

long HiroGateServiceSVC_impl::setHandPose(
	IKBase *ik, 
	const RTC::Pose3D& pose_m_rad, double rate, bool wait_for_motion )
{
	if( isIKSolutionFound( ik, pose_m_rad ) ) {
		// IKJointAnglesɕϊ
		ik->convertSolutionToJointAngles();

		// rate-secZ͓K
		return setJointAngles( ik->angles, IKBase::RATE2SECOND/rate, wait_for_motion );
	}
	return ik->returnIKFailed();
}

#else //-----------------------------------------------------------------------

long HiroGateServiceSVC_impl::setHandPose(
	RTC::CorbaConsumer<OpenHRP::ArmControlService> *acs_if,
	const RTC::Pose3D& pose_m_rad, double rate, bool wait_for_motion )
{
	if( CORBA::is_nil( acs_if->getObject() ) )
		return 0x80000000;

#	if 1
	(*acs_if)->setTargetAngular(
		pose_m_rad.position.x, pose_m_rad.position.y, pose_m_rad.position.z,
		pose_m_rad.orientation.r, pose_m_rad.orientation.p, pose_m_rad.orientation.y,
		rate );
#	else
	///// v: [Esb`E[̏
		///// 2011/10/21
		///// ]̓IC[pw, yaw(Z)pitch(Y)roll(X)ŉ
		///// initial pose  pitch = -90Ȃ̂ŁCZXdȂĂ܂
	///// v: Ԗߒl
		///// 2011/10/21
		///// setTargetAngular() ͈ړ\ȏꍇCړɂ鎞[s]ԂĂ悤Ɍ
		///// ړs\ȏꍇ͉ԂQNXRTCI.
	printf( "=========== %f\n", (*acs_if)->setTargetAngular(
		pose_m_rad.position.x, pose_m_rad.position.y, pose_m_rad.position.z,
		pose_m_rad.orientation.r, pose_m_rad.orientation.p, pose_m_rad.orientation.y,
		rate ) );
#	endif

	while( wait_for_motion && ((*acs_if)->checkStatus() == OpenHRP::ArmBusyState) )
		Sleep( 50 );

	return 0;
}

#endif //----------------------------------------------------------------------


CORBA::Boolean HiroGateServiceSVC_impl::isFingerMovingR()
{
	if( CORBA::is_nil( svc_ptrs.gs_if->getObject() ) )
		return true;
	return (*svc_ptrs.gs_if)->isEmpty( 0x1F, false ) ? false : true;
}


/*=========================================================
	rʒupw蓮
	@	return	:	
		0		; I
		-1	;	rIKs / -2	;	ErIKs	/	-3	;	rIKs
=========================================================*/
CORBA::Long HiroGateServiceSVC_impl::setHandPoseLR(
	const RTC::Pose3D& pose_l_m_rad, const RTC::Pose3D& pose_r_m_rad, 
	CORBA::Double rate, CORBA::Boolean wait_for_motion)
{

#	if defined( _V0514_2 ) || defined( _V0514_5 ) || defined( _V0514_6 )

	// ikfastɂ͓̂̊֐߂͊܂܂Ȃ̂ŁC
	// {bgnR𓷑̌nBɕϊĂ₢킹
	// ̊֐߂̊px擾
	HiroGateRTC::HiroGateService::JointAngles_var angles;
	getJointAngles( angles );
	double torso_angle_rad = angles[hgutil::BH][0];

	RTC::Pose3D poseB_l_m_rad, poseB_r_m_rad;

	hgutil::R2B( pose_l_m_rad, torso_angle_rad, poseB_l_m_rad );
	hgutil::R2B( pose_r_m_rad, torso_angle_rad, poseB_r_m_rad );

	// IK̉̃`FbN
	IKL ikL;
	IKR ikR;

	bool fL, fR; // flag
	fL = ikL.solve( poseB_l_m_rad ) ? true : false;
	fR = ikR.solve( poseB_r_m_rad ) ? true : false;

	if( !fL && !fR )	return -3;	// rIKs
	if( !fL )					return ikL.returnIKFailed();	// rIKs
	if( !fR )					return ikR.returnIKFailed();	// ErIKs

	// ֐ߊp̏
	HiroGateRTC::HiroGateService::JointAngles jLR;
	hgutil::allocateJointAngles( jLR, false, true, true );
	ikL.convertSolutionToJointAngles();	jLR[hgutil::AL] = ikL.angles[hgutil::AL];
	ikR.convertSolutionToJointAngles();	jLR[hgutil::AR] = ikR.angles[hgutil::AR];

	// 쎞ԁiقgj
	double tL, tR;
	tL = hgutil::rate2t( hgutil::LEFT, angles, jLR, rate, 0 );
	tR = hgutil::rate2t( hgutil::RIGHT, angles, jLR, rate, 0 );
	
	// 
	return setJointAngles( jLR, tL > tR ? tL : tR, wait_for_motion );

#	else //----------------------------------------------------------------------
	
	int res_l, res_r;
	res_l = setHandPose( svc_ptrs.acsL_if, pose_l_m_rad, rate, false );
	res_r = setHandPose( svc_ptrs.acsR_if, pose_r_m_rad, rate, false );

	// ҂
	while( wait_for_motion && 
		((*svc_ptrs.acsL_if)->checkStatus() == OpenHRP::ArmBusyState || 
		 (*svc_ptrs.acsR_if)->checkStatus() == OpenHRP::ArmBusyState) )
		Sleep( 50 );

	return 0;

#	endif //---------------------------------------------------------------------
}