// -*- C++ -*-
/*!
 * @file  HiroPickerAndPlacer.h
 * @brief ${rtcParam.description}
 * @date  $Date$
 *
 * $Id$
 */

#ifndef HIROPICKERANDPLACER_H
#define HIROPICKERANDPLACER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>
#include "HiroPnP-impl.h"


// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "ObjectPoseEstimationGateRTCStub.h"
#include "HiroGateRTCStub.h"
#include "HiroNXStub.h"
#include "HIROControllerStub.h"

// </rtc-template>

using namespace RTC;

/*!
 * @class HiroPickerAndPlacer
 * @brief ${rtcParam.description}
 *
 */
class HiroPickerAndPlacer
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  HiroPickerAndPlacer(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~HiroPickerAndPlacer();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
# if 0
  /*!
   * 
   * - Name:  xWorldR_m
   * - DefaultValue: 0.285
   */
  double m_xWorldR_m;
  /*!
   * 
   * - Name:  yWorldR_m
   * - DefaultValue: -0.122
   */
  double m_yWorldR_m;
  /*!
   * 
   * - Name:  zWorldR_m
   * - DefaultValue: -0.087
   */
  double m_zWorldR_m;
  /*!
   * 
   * - Name:  xPalletPlacedB_m
   * - DefaultValue: 0.425
   */
  double m_xPalletPlacedB_m;
  /*!
   * 
   * - Name:  yPalletPlacedB_m
   * - DefaultValue: 0.0
   */
  double m_yPalletPlacedB_m;
  /*!
   * 
   * - Name:  zPalletPlacedB_m
   * - DefaultValue: -0.087
   */
  double m_zPalletPlacedB_m;
  /*!
   * 
   * - Name:  rPalletPlacedB_deg
   * - DefaultValue: 0.0
   */
  double m_rPalletPlacedB_deg;
  /*!
   * 
   * - Name:  pPalletPlacedB_deg
   * - DefaultValue: -90.0
   */
  double m_pPalletPlacedB_deg;
  /*!
   * 
   * - Name:  yPalletPlacedB_deg
   * - DefaultValue: 0.0
   */
  double m_yPalletPlacedB_deg;
  /*!
   * 
   * - Name:  directionPalletPlacedR_deg
   * - DefaultValue: -90.0
   */
  double m_directionPalletPlacedR_deg;
  /*!
   * 
   * - Name:  liftDistancePalletR_m
   * - DefaultValue: 0.05
   */
  double m_liftDistancePalletR_m;
  /*!
   * 
   * - Name:  xPalletToBePlacedB_m
   * - DefaultValue: 0.425
   */
  double m_xPalletToBePlacedB_m;
  /*!
   * 
   * - Name:  yPalletToBePlacedB_m
   * - DefaultValue: 0.000
   */
  double m_yPalletToBePlacedB_m;
  /*!
   * 
   * - Name:  zPalletToBePlacedB_m
   * - DefaultValue: -0.087
   */
  double m_zPalletToBePlacedB_m;
  /*!
   * 
   * - Name:  directionPalletToBePlacedR_deg
   * - DefaultValue: 0.0
   */
  double m_directionPalletToBePlacedR_deg;
  /*!
   * 
   * - Name:  xOPEOffsetR_m
   * - DefaultValue: 0.0
   */
  double m_xOPEOffsetR_m;
  /*!
   * 
   * - Name:  yOPEOffsetR_m
   * - DefaultValue: 0.0
   */
  double m_yOPEOffsetR_m;
  /*!
   * 
   * - Name:  speedBase
   * - DefaultValue: 15
   */
  double m_speedBase;
  /*!
   * 
   * - Name:  serialNo
   * - DefaultValue: 3
   */
  int m_serialNo;
  /*!
   * 
   * - Name:  yWorldR_deg
   * - DefaultValue: 0.0
   */
  double m_yWorldR_deg;
#	else
	pnp::configuration_parameter_t conf;
#	endif
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedStringSeq m_input;
  /*!
   */
  InPort<TimedStringSeq> m_inputIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedStringSeq m_output;
  /*!
   */
  OutPort<TimedStringSeq> m_outputOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   */
  RTC::CorbaPort m_ObjectPoseEstimationGateServicePort;
  /*!
   */
  RTC::CorbaPort m_HiroGateServicePort;
  /*!
   */
  RTC::CorbaPort m_HiroNXPort;
  /*!
   */
  RTC::CorbaPort m_HIROPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   */
  RTC::CorbaConsumer<HiroGateRTC::HiroGateService> hg;
  /*!
   */
	RTC::CorbaConsumer<ObjectPoseEstimationGateRTC::ObjectPoseEstimationGateService> ope;
  /*!
   */
  RTC::CorbaConsumer<HiroNX> manipu;
  /*!
   */
  RTC::CorbaConsumer<CommonCommands> common;
  /*!
   */
  RTC::CorbaConsumer<MotionCommands> motion;
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  int process();
	static unsigned int __stdcall processObj( void *hiro_pnp_ptr );
	static unsigned int __stdcall processTray( void *hiro_pnp_ptr );

	bool initializePose();
	bool initializeObjectModel();
	bool initializeWorldParameter();
	bool initializePalletParameter();
	bool initializePlacingSlot();

		int activate();
	int deactivate();

	void stopThread( bool *in_operation );

private:
	// ̂̃}js[Viv+sj
	bool manipulateSingleObject(
		int model_id, const RTC::Size3D &obj_size_m, const RTC::Pose3D &obj_poseR_m_rad );
	// XbgT
	int findEmptyPlacingSlot( int model_id, RTC::Point3D &slot_xyzB_m );

	// gC̃}js[Viv+sj
	bool manipulateTray();

private:
	// v
	int planObjectPicking(
		const RTC::Size3D &obj_size_m, const RTC::Pose3D &obj_poseR_m_rad, 
		const RTC::Point3D &slot_xyzB_m, double finger_depth_m, 
		std::vector< hgutil::OperationNode* > &ope_nodes, hgutil::dominant_side_t *pck_side );
	int planObjectPlacing(
		const RTC::Size3D &obj_size_m,
		const RTC::Point3D &slot_xyzB_m, double finger_depth_m, hgutil::dominant_side_t 
		picking_side, std::vector< hgutil::OperationNode* > &ope_nodes );

	int planTrayPicking( double finger_depth_m, std::vector< hgutil::OperationNode* > &ope_nodes );
	int planTrayPlacing( double finger_depth_m, std::vector< hgutil::OperationNode* > &ope_nodes );

private:
	// s
	int actPlan(
		const std::vector< hgutil::OperationNode* > &ope_nodes,
		const HiroGateRTC::HiroGateService::JointAngles *angles_base_rad = NULL );

	// v̍폜
	void deletePlan( std::vector< hgutil::OperationNode* > &ope_nodes );
	/*
	void W2R(
		const RTC::Pose3D &pose_objW,
		opencv::cvmath::Vector< CV_64F > &p_objR, opencv::cvmath::Matrix< CV_64F > &R_objR );
	*/

  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void HiroPickerAndPlacerInit(RTC::Manager* manager);
};

#endif // HIROPICKERANDPLACER_H
