#include "HiroPickerAndPlacer.h"
#include "../VSCommon/system/misc/misc.h"


using namespace std;
using namespace opencv;


using hgutil::OperationNode;
using hgutil::JointOperation;
using hgutil::FingerAngleOperation;
using hgutil::FingerAngleLROperation;
using hgutil::FingerDistanceOperation;
using hgutil::FingerDistanceLROperation;
using hgutil::HandOperation;
using hgutil::HandLROperation;
using hgutil::SleepOperation;


/**********************************************************
	gC̑
**********************************************************/
bool HiroPickerAndPlacer::manipulateTray()
{

	int rc;

	/* sv */
	// w̒
	double finger_depth_mm, finger_depth_m;
	hgutil::calculateFingerDepthFromDistance( 0, &finger_depth_mm, true, false );
	finger_depth_mm += (conf.serial_no == 11) ? 59 : 0;	// AISThiro011p
	finger_depth_m = 0.001*finger_depth_mm;

	vector< OperationNode* > ope_nodes;
	
	
	rc = planTrayPicking( finger_depth_m, ope_nodes );
	if( rc ) {
		return sys::returnByError( false,
			"\a\aHiroPickerAndPlacer::manipulateTray(): "
			"cannot calculate picking parameters." );
	}

	rc = planTrayPlacing( finger_depth_m, ope_nodes );
	if( rc ) {
		return sys::returnByError( false,
			"\a\aHiroPickerAndPlacer::manipulateTray(): "
			"cannot calculate placing parameters." );
	}
	
	cout << "[Tray picking & placing plan] ------------------" << endl;
	for( vector< OperationNode* >::iterator plc_it = ope_nodes.begin();
		plc_it != ope_nodes.end(); ++ plc_it )
		(*plc_it)->print();

	cout << "[Tray pick and place] --------------------------" << endl;
	rc = actPlan( ope_nodes );
	if( rc ) {
		return sys::returnByError( false,
			"\a\aHiroPickerAndPlacer::manipulateTray(): "
			"cannot pick & place tray." );
	}
	deletePlan( ope_nodes );

	return true;
}


/**********************************************************
	sbLOv
**********************************************************/
int HiroPickerAndPlacer::planTrayPicking(
	double finger_depth_m, std::vector< OperationNode* > &ope_nodes )
{
	printf( "Planning / Tray picking" ), fflush( stdout );

	const pnp::configuration_parameter_t::pallet_param_t &plt = conf.pallet;

	pnp::operation_t pckL0, pckR0;
	RTC::Pose3D LR_tmp, RR_tmp;
	HiroGateRTC::HiroGateService::JointAngles torso_angles_rad;
	torso_angles_rad = pnp::operation_t::pose_torso_only;

	// gCuĂ
	pckL0.torso_angle_rad = cvmath::deg2rad( plt.direction_placedR_deg );

	// gC̎̈ʒu
	pckL0.B.position.x = plt.xyz_placedB_m(0);
	pckL0.B.position.z = plt.xyz_placedB_m(2) + plt.hole_bottom_height_m + finger_depth_m;
	pckL0.B.orientation.r = cvmath::deg2rad( plt.rpy_placedB_deg(0) );
	pckL0.B.orientation.p = cvmath::deg2rad( plt.rpy_placedB_deg(1) );
	pckL0.B.orientation.y = cvmath::deg2rad( plt.rpy_placedB_deg(2) );

	pckR0 = pckL0;

	// ̎
	pckL0.B.position.y = plt.xyz_placedB_m(1) + 0.5*plt.width_m;
	hgutil::B2R( pckL0.B, pckL0.torso_angle_rad, pckL0.R );
	
	// E̎
	pckR0.B.position.y = plt.xyz_placedB_m(1) - 0.5*plt.width_m;
	hgutil::B2R( pckR0.B, pckR0.torso_angle_rad, pckR0.R );

	putch( '.' ), fflush( stdout );

	/* 1.̂] */
	{
		torso_angles_rad[0][0] = pckL0.torso_angle_rad;
		ope_nodes.push_back(
			new JointOperation( hg,
				torso_angles_rad, conf.t_s(), true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 2.̏ */
	{
		LR_tmp = pckL0.R;
		LR_tmp.position.z += 0.1;

		RR_tmp = pckR0.R;
		RR_tmp.position.z += 0.1;

		if( !HandLROperation::isIKSolutionFound( hg,
			LR_tmp, RR_tmp, pckL0.torso_angle_rad ) )
			return sys::returnByError( -2, "[IK failed. (2)]" );

		ope_nodes.push_back(
			new HandLROperation( hg,
				LR_tmp, RR_tmp, 1.5*conf.speed_base, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 3.wJ */
	{
		ope_nodes.push_back(
			new FingerDistanceLROperation( hg,
				30, 30, 0.3, true ) );
	}

	/* 4. */
	{
		if( !HandLROperation::isIKSolutionFound( hg,
			pckL0.R, pckR0.R, pckL0.torso_angle_rad ) )
			return sys::returnByError( -4, "[IK failed. (4)]" );

		ope_nodes.push_back(
			new HandLROperation( hg,
				pckL0.R, pckR0.R, conf.speed_base, true ) );
	}

	/* 5.҂ */
	{
		ope_nodes.push_back( new SleepOperation( 0.2 ) );
	}

	putch( '.' ), fflush( stdout );

	/* 6.w */
	{
		ope_nodes.push_back( 
			new FingerDistanceLROperation( hg,
				0, 0, 0.1, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 7.gC𗼎ŋ */
	{
		RTC::Pose3D LB_tmp = pckL0.B, RB_tmp = pckR0.B;

		LB_tmp.position.y = conf.pallet.xyz_placedB_m(1) + 0.5*plt.hold_width_m();
		RB_tmp.position.y = conf.pallet.xyz_placedB_m(1) - 0.5*plt.hold_width_m();

		hgutil::B2R( LB_tmp, pckL0.torso_angle_rad, LR_tmp );
		hgutil::B2R( RB_tmp, pckR0.torso_angle_rad, RR_tmp );

		if( !HandLROperation::isIKSolutionFound( hg,
			LR_tmp, RR_tmp, pckL0.torso_angle_rad ) )
			return sys::returnByError( -7, "[IK failed. (7)]" );

		ope_nodes.push_back(
			new HandLROperation( hg,
				LR_tmp, RR_tmp, conf.speed_base, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 8.҂ */
	{
		ope_nodes.push_back( new SleepOperation( 0.2 ) );
	}

	putch( '.' ), fflush( stdout );

	/* 7. */
	{
		/* uĂꏊƒuꏊ̍قɍ킹Ďグ */
		const double PLAN_A = plt.hole_bottom_height_m + finger_depth_m + plt.lift_distance_R_m
			+ plt.xyz_to_be_placedB_m(2); // uꏊ̍+tg
		const double PLAN_B = plt.hole_bottom_height_m + finger_depth_m + plt.lift_distance_R_m
			+ plt.xyz_placedB_m(2); // ݈ʒu+tg

		LR_tmp.position.z = RR_tmp.position.z = PLAN_A > PLAN_B ? PLAN_A : PLAN_B;

		if( !HandLROperation::isIKSolutionFound( hg,
			LR_tmp, RR_tmp, pckL0.torso_angle_rad ) )
			return sys::returnByError( -9, "[IK failed. (9)]" );

		ope_nodes.push_back(
			new HandLROperation( hg,
				LR_tmp, RR_tmp, conf.speed_base, true ) );
	}

	printf( "[OK]\n" );

	return 0;
}


/**********************************************************
	v[VOv
**********************************************************/
int HiroPickerAndPlacer::planTrayPlacing(
	double finger_depth_m, std::vector< OperationNode* > &ope_nodes )
{
	/* Ō̈uIpbgړ */
	const pnp::configuration_parameter_t::pallet_param_t &plt = conf.pallet;

	pnp::operation_t plcL0, plcR0;
	RTC::Pose3D LR_tmp, RR_tmp;
	HiroGateRTC::HiroGateService::JointAngles torso_angles_rad;
	torso_angles_rad = pnp::operation_t::pose_torso_only;

	// gCuĂ
	plcL0.torso_angle_rad = cvmath::deg2rad( plt.direction_to_be_placedR_deg );

	// gC̎̈ʒu
	plcL0.B.position.x = plt.xyz_to_be_placedB_m(0);
	plcL0.B.position.z = plt.xyz_to_be_placedB_m(2) + plt.hole_bottom_height_m + finger_depth_m;
	plcL0.B.orientation.r = cvmath::deg2rad( plt.rpy_placedB_deg(0) );
	plcL0.B.orientation.p = cvmath::deg2rad( plt.rpy_placedB_deg(1) );
	plcL0.B.orientation.y = cvmath::deg2rad( plt.rpy_placedB_deg(2) );

	plcR0 = plcL0;

	// ̎
	plcL0.B.position.y = plt.xyz_to_be_placedB_m(1) + 0.5*plt.hold_width_m();
	hgutil::B2R( plcL0.B, plcL0.torso_angle_rad, plcL0.R );
	
	// E̎
	plcR0.B.position.y = plt.xyz_to_be_placedB_m(1) - 0.5*plt.hold_width_m();
	hgutil::B2R( plcR0.B, plcR0.torso_angle_rad, plcR0.R );

	putch( '.' ), fflush( stdout );

	/* 10.̂] */
	{
		torso_angles_rad[0][0] = cvmath::deg2rad( plt.direction_to_be_placedR_deg );
		ope_nodes.push_back(
			new JointOperation( hg,
				torso_angles_rad, conf.t_s(), true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 11.gCu */
	{
		if( !HandLROperation::isIKSolutionFound( hg,
			plcL0.R, plcR0.R, plcL0.torso_angle_rad ) )
			return sys::returnByError( -9, "[IK failed. (11)]" );

		ope_nodes.push_back(
			new HandLROperation( hg,
				plcL0.R, plcR0.R, conf.speed_base, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 12.̊ԊuL */
	{
		RTC::Pose3D LB_tmp = plcL0.B, RB_tmp = plcR0.B;

		LB_tmp.position.y = conf.pallet.xyz_to_be_placedB_m(1) + 0.5*plt.width_m;
		RB_tmp.position.y = conf.pallet.xyz_to_be_placedB_m(1) - 0.5*plt.width_m;

		hgutil::B2R( LB_tmp, plcL0.torso_angle_rad, LR_tmp );
		hgutil::B2R( RB_tmp, plcR0.torso_angle_rad, RR_tmp );

		if( !HandLROperation::isIKSolutionFound( hg,
			LR_tmp, RR_tmp, plcL0.torso_angle_rad ) )
			return sys::returnByError( -10, "[IK failed. (10)]" );

		ope_nodes.push_back(
			new HandLROperation( hg,
				LR_tmp, RR_tmp, conf.speed_base, true ) );
	}
	
	putch( '.' ), fflush( stdout );

	/* 13.҂ */
	{
		ope_nodes.push_back( new SleepOperation( 0.2 ) );
	}

	putch( '.' ), fflush( stdout );

	/* 14.wJ */
	{
		ope_nodes.push_back(
			new FingerDistanceLROperation( hg,
				30, 30, 0.3, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 15. */
	{
		LR_tmp.position.z += 0.1;
		RR_tmp.position.z += 0.1;
		if( !HandLROperation::isIKSolutionFound( hg,
			LR_tmp, RR_tmp, plcL0.torso_angle_rad ) )
			return sys::returnByError( -15, "[IK failed. (15)]" );

		ope_nodes.push_back(
			new HandLROperation( hg,
				LR_tmp, RR_tmp, conf.speed_base, true ) );
	}

	/* 16.p */
	{
		ope_nodes.push_back( new JointOperation( hg, 
			pnp::operation_t::pose_all_initial, 
			conf.t_s(), true ) );
	}

	printf( "[OK]\n" );

	return 0;
}