// -*-C++-*-
/*!
 * @file  ObjectPoseEstimationGateRTCSVC_impl.cpp
 * @brief Service implementation code of ObjectPoseEstimationGateRTC.idl
 *
 */

#include "ObjectPoseEstimationGateRTCSVC_impl.h"
#include "../rtcutil/rtcutil.h"
#include "../VSCommon/system/misc/misc.h"

using namespace std;
using namespace opencv;


/*
 * Example implementational code for IDL interface ObjectPoseEstimationGateRTC::ObjectPoseEstimationGateService
 */
ObjectPoseEstimationGateServiceSVC_impl::ObjectPoseEstimationGateServiceSVC_impl(
	RTC::CorbaConsumer<ModelDefinitionRTC::idl::MDService> *_md_svc)
	: md_svc( _md_svc)
{
  resetEstimatedResult();
}


ObjectPoseEstimationGateServiceSVC_impl::~ObjectPoseEstimationGateServiceSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::Long ObjectPoseEstimationGateServiceSVC_impl::getLastEstimatedResult(
	const RTC::Pose3D& W2R_m_rad,
	CORBA::Long& id, RTC::Size3D& size_m, RTC::Pose3D& poseR_m_rad)
{
	cout << ">> OPEGService::getLastEstimatedResult()" << endl;
	cout << "id = " << this->id << endl;

	if( this->id == -1 )	return -1;

	/* E{bgWn̉]s */
	// ]piʂł̉]݂̂Ɖj
	cvmath::Vector< CV_64F > angle_axis(3);
	angle_axis.setZero();
	angle_axis(2) = W2R_m_rad.orientation.y;

	// l
	cvmath::Quaternion< CV_64F > Q_WtoR;
	Q_WtoR.fromAxis( angle_axis );

	// ]s
	cvmath::Matrix< CV_64F > R_WtoR(3,3);
	Q_WtoR.toR( R_WtoR );

	/* E{bgWn̕iړ */
	cvmath::Vector< CV_64F > t_WtoR(3);
	t_WtoR(0) = W2R_m_rad.position.x;
	t_WtoR(1) = W2R_m_rad.position.y;
	t_WtoR(2) = W2R_m_rad.position.z;

#	if 0
	// îzj
	this->id = 1;
	this->pose.position.x = 0.001*(100+10);
	this->pose.position.y = 0.001*(0);
	this->pose.position.z = 0.001*(40);

	this->pose.orientation.r = cvmath::deg2rad(+94.544);
	this->pose.orientation.p = cvmath::deg2rad(-46.455);
	this->pose.orientation.y = cvmath::deg2rad(-88.028);
#	endif

	/* ̂̈ʒup(W: En, R: {bgn) */
	cvmath::Vector< CV_64F > p_objR(3);
	cvmath::Matrix< CV_64F > R_objR(3,3);
	W2R( R_WtoR, t_WtoR, this->pose, p_objR, R_objR );

	/* ۂɒ͂ޖʂ̃p[^߂ */
	// 茋ʂIDɃff[^[h
#	ifdef OPEG_OPERATION_WITHOUT_MD

	ModelDefinitionRTC::idl::ObjectModel mdl;
	switch( mdl.Model_id = this->id ) {
		case 0: // 
			mdl.Model_category = 0;
			mdl.wx = 0.02;
			mdl.wy = 0.015;
			mdl.wz = 0.025;
			break;
		case 1:
			mdl.Model_category = 1;
			mdl.height = 0.03;
			mdl.radius = 0.015;
			break;
		default:
			mdl.Model_category = -1;
	}

#	else

	if( !rtcutil::isInOperation( *md_svc ) )
		return sys::returnByError( -2,
			"OPEGateService: MDService not respond." );

	ModelDefinitionRTC::idl::ObjectModel
		mdl = (*md_svc)->getObjectModel( this->id );

#	endif


	// f̃JeS(/~jʂɃp[^vZ
	double width_m, height_m, yaw_rad;
	switch( mdl.Model_category ) {
		case 0:	// 
			calculateCuboidPickingParameter(
				mdl, R_objR, &width_m, &height_m, &yaw_rad );
			break;
		case 1: // ~
			calculateCylinderPickingParameter(
				mdl, &width_m, &height_m, &yaw_rad );
			break;
		default:
			return sys::returnByError( -3,
				"OPEGService: MDService cannot find model." );
	}

	
	id = this->id;

	size_m.l = 0.0;
	size_m.w = width_m;
	size_m.h = height_m;
	
	poseR_m_rad.position.x = p_objR(0);
	poseR_m_rad.position.y = p_objR(1);
	poseR_m_rad.position.z = p_objR(2);
	poseR_m_rad.orientation.y = yaw_rad;
	poseR_m_rad.orientation.p = 0;
	poseR_m_rad.orientation.r = 0;

	using opencv::cvmath::rad2deg;

	printf( "pR: (x,y,z)(r,p,y) = "
		"(%+.3f, %+.3f, %+.3f)(%+.1f, %+.1f, %+.1f)\n",
		poseR_m_rad.position.x, 
		poseR_m_rad.position.y,
		poseR_m_rad.position.z,
		rad2deg( poseR_m_rad.orientation.r ),
		rad2deg( poseR_m_rad.orientation.p ),
		rad2deg( poseR_m_rad.orientation.y ) );

	resetEstimatedResult();

	cout << "<< OPEGService::getLastEstimatedResult()" << endl;
  return id;
}



// End of example implementational code



