// -*-C++-*-
/*!
 * @file  ObjectPoseEstimationGateRTCSVC_impl.h
 * @brief Service implementation header of ObjectPoseEstimationGateRTC.idl
 *
 */

#include "BasicDataTypeSkel.h"
#include "ExtendedDataTypesSkel.h"

#include "ObjectPoseEstimationGateRTCSkel.h"

#ifndef OBJECTPOSEESTIMATIONGATERTCSVC_IMPL_H
#define OBJECTPOSEESTIMATIONGATERTCSVC_IMPL_H

#include <rtm/CorbaPort.h>
#include "ModelDefinitionCommonServiceStub.h"
#include "../VSCommon/opencv/cvmath/cvmath.h"

/*!
 * @class ObjectPoseEstimationGateServiceSVC_impl
 * Example class implementing IDL interface ObjectPoseEstimationGateRTC::ObjectPoseEstimationGateService
 */
class ObjectPoseEstimationGateServiceSVC_impl
 : public virtual POA_ObjectPoseEstimationGateRTC::ObjectPoseEstimationGateService,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~ObjectPoseEstimationGateServiceSVC_impl();

 public:
  /*!
   * @brief standard constructor
   */
	 ObjectPoseEstimationGateServiceSVC_impl(RTC::CorbaConsumer<ModelDefinitionRTC::idl::MDService> *_md_svc = NULL);
  /*!
   * @brief destructor
   */
   virtual ~ObjectPoseEstimationGateServiceSVC_impl();

    // attributes and operations
   CORBA::Long getLastEstimatedResult(
		 const RTC::Pose3D& W2R_m_rad,
		 CORBA::Long& id, RTC::Size3D& size_m, RTC::Pose3D& poseR_m_rad);

public:
	int id;
	RTC::Pose3D pose;
	
	RTC::CorbaConsumer<ModelDefinitionRTC::idl::MDService> *md_svc;

public:
	
	void setEstimatedResult( int id, const RTC::Pose3D& poseW_m_rad );
	void resetEstimatedResult();

private:
	void W2R(
		const opencv::cvmath::Matrix< CV_64F > &R_WtoR, const opencv::cvmath::Vector< CV_64F > &t_WtoR,
		const RTC::Pose3D &pose_objW,
		opencv::cvmath::Vector< CV_64F > &p_objR, opencv::cvmath::Matrix< CV_64F > &R_objR );
	
	void calculateCuboidPickingParameter(
		const ModelDefinitionRTC::idl::ObjectModel &cuboid_mdl,
		const opencv::cvmath::Matrix< CV_64F > &R_objR, 
		double *width_m, double *height_m, double *yaw_rad );
	void calculateCylinderPickingParameter( 
		const ModelDefinitionRTC::idl::ObjectModel &cylinder_mdl,
		double *width_m, double *height_m, double *yaw_rad );
};



#endif // OBJECTPOSEESTIMATIONGATERTCSVC_IMPL_H


