// ================================================================================== //
// Name of Component		: RTComponent Path Planner
// Vendor					: AISL-TUT
// Version					: AGV
// Latest Development by	: Igi Ardiyanto
// ================================================================================== //

// ================================================================================== //
// Previous version			: Yuki Ishikawa (ver. 1.0.0)
//							  Igi Ardiyanto (ver. 2.0.0)
// Dependencies				: OpenCV 2.1, OpenCV 1.0
// ================================================================================== //

// -*- C++ -*-
/*!
 * @file  PathPlannerV2.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */
#define __MAIN__
#include "PathPlannerV2.h"



void PathPlannerV2::exit_robot(void)
{
	m_robot_control.data.vx = 0;
	m_robot_control.data.va = 0;
	m_robot_controlOut.write(m_robot_control);
}

float PathPlannerV2::_robotToGoalDist(void){
	float dx,dy,dst;
	dx = xg[1]-path[step].pose.x;
	dy = xg[2]-path[step].pose.y;
	dx *= dx; dy *= dy;
	dst = dx+dy;
	return (float)(sqrt(dst));
}

void PathPlannerV2::drawing_when_error(void){


			//cvCopy(window_output, LocalImg);
			Draw_LMap_Data2(LocalImg, NULL, NULL, &warning_flag, __LINE__ );

			//LocalImg forward from here.
			IplImage2TimedImage(&m_planning_img, LocalImg);
			m_planning_imgOut.write();

if(TAKE_VIDEO)
{//if(!cvSaveImage(outFileName,src)) printf("Could not save: %s\n",outFileName);
//cvNormalize(Gauss_f32, Gauss_u8, 0.0, 255.0, CV_MINMAX, NULL);
//cvWriteToAVI(writer,Gauss_u8);
cvWriteToAVI(writer,LocalImg);
}

			/** Black area **/
			cvZero(LocalImg);

}

void PathPlannerV2::drawing_whole(void){
cvZero (LocalImg);
	for (int y = 0; y < LocalImg->height; y++) {
		for (int x = 0; x < LocalImg->width; x++) {
			if((LocalMap[(int)(x*((double)DRAW_RESOLUTION/(double)GRID_SIZE))][(int)(y*((double)DRAW_RESOLUTION/(double)GRID_SIZE))] & RFLAG) != 0){//robot
				LocalImg->imageData[LocalImg->widthStep * y + x * 3] = (unsigned char)255;//B
			}
			if((LocalMap[(int)(x*((double)DRAW_RESOLUTION/(double)GRID_SIZE))][(int)(y*((double)DRAW_RESOLUTION/(double)GRID_SIZE))] & SFLAG) != 0){//static object
				LocalImg->imageData[LocalImg->widthStep * y + x * 3 + 1] = (unsigned char)255;// G
			}
			if((LocalMap[(int)(x*((double)DRAW_RESOLUTION/(double)GRID_SIZE))][(int)(y*((double)DRAW_RESOLUTION/(double)GRID_SIZE))] & MFLAG) != 0){//moving obsject
				LocalImg->imageData[LocalImg->widthStep * y + x * 3 + 2] = (unsigned char)255;// R
			}
			if((LocalMap[(int)(x*((double)DRAW_RESOLUTION/(double)GRID_SIZE))][(int)(y*((double)DRAW_RESOLUTION/(double)GRID_SIZE))] & UnKnown_FLAG) != 0){//unknown area
				LocalImg->imageData[LocalImg->widthStep * y + x * 3] = (unsigned char)128;//B
				LocalImg->imageData[LocalImg->widthStep * y + x * 3 + 1] = (unsigned char)128;// G
				LocalImg->imageData[LocalImg->widthStep * y + x * 3 + 2] = (unsigned char)128;// R
			}
		}
	}
printf("Drawing Whole\n");
}

void PathPlannerV2::make_motion(MOTION_CONTROL *output,MOTION_CONTROL *input1){
	static int num_motion=0,num1;
	double vr,vl;
	num1=input1->data.motion_num;
	num_motion=num1*4+33;
	(*output).data.motion_num=num_motion;
	(*output).data.r_v = (RV_SET *)calloc(num_motion, sizeof(RV_SET));
	(*output).data.vl_vr = (VL_VR_SET *)calloc(num_motion, sizeof(VL_VR_SET));
	for(int i=0;i<num1;i++){
		(*output).data.r_v[i].radius=input1->data.r_v[i].radius;
		(*output).data.r_v[i].velocity=input1->data.r_v[i].velocity;
		convert_RVtoVLVR(output->data.r_v[i].radius,output->data.r_v[i].velocity,&vr,&vl);
		(*output).data.vl_vr[i].vl=(float)vl;
		(*output).data.vl_vr[i].vr=(float)vr;
	}
	for(int i=0;i<num1;i++){
		(*output).data.r_v[i+num1].radius=output->data.r_v[i].radius;
		(*output).data.r_v[i+num1].velocity=output->data.r_v[i].velocity/2;
		convert_RVtoVLVR(output->data.r_v[i+num1].radius,output->data.r_v[i+num1].velocity,&vr,&vl);
		(*output).data.vl_vr[i+num1].vl=(float)vl;
		(*output).data.vl_vr[i+num1].vr=(float)vr;
	}
	for(int i=0;i<num1;i++){
		(*output).data.r_v[i+2*num1].radius=output->data.r_v[i].radius/2;
		(*output).data.r_v[i+2*num1].velocity=output->data.r_v[i].velocity;
		convert_RVtoVLVR(output->data.r_v[i+2*num1].radius,output->data.r_v[i+2*num1].velocity,&vr,&vl);
		(*output).data.vl_vr[i+2*num1].vl=(float)vl;
		(*output).data.vl_vr[i+2*num1].vr=(float)vr;
	}
	for(int i=0;i<num1;i++){
		(*output).data.r_v[i+3*num1].radius=output->data.r_v[i].radius/2;
		(*output).data.r_v[i+3*num1].velocity=output->data.r_v[i].velocity/2;
		convert_RVtoVLVR(output->data.r_v[i+3*num1].radius,output->data.r_v[i+3*num1].velocity,&vr,&vl);
		(*output).data.vl_vr[i+3*num1].vl=(float)vl;
		(*output).data.vl_vr[i+3*num1].vr=(float)vr;

	}

	// Data added
	(*output).data.r_v[4*num1].velocity=300.f;
	(*output).data.r_v[4*num1].radius=0.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1].radius,output->data.r_v[4*num1].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1].vl=(float)vl;
	(*output).data.vl_vr[4*num1].vr=(float)vr;

	(*output).data.r_v[4*num1+1].velocity=300.f;
	(*output).data.r_v[4*num1+1].radius=1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+1].radius,output->data.r_v[4*num1+1].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+1].vl=(float)vl;
	(*output).data.vl_vr[4*num1+1].vr=(float)vr;

	(*output).data.r_v[4*num1+2].velocity=300.f;
	(*output).data.r_v[4*num1+2].radius=-1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+2].radius,output->data.r_v[4*num1+2].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+2].vl=(float)vl;
	(*output).data.vl_vr[4*num1+2].vr=(float)vr;

	(*output).data.r_v[4*num1+3].velocity=300.f;
	(*output).data.r_v[4*num1+3].radius=1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+3].radius,output->data.r_v[4*num1+3].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+3].vl=(float)vl;
	(*output).data.vl_vr[4*num1+3].vr=(float)vr;

	(*output).data.r_v[4*num1+4].velocity=300.f;
	(*output).data.r_v[4*num1+4].radius=-1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+4].radius,output->data.r_v[4*num1+4].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+4].vl=(float)vl;
	(*output).data.vl_vr[4*num1+4].vr=(float)vr;

	(*output).data.r_v[4*num1+5].velocity=300.f;
	(*output).data.r_v[4*num1+5].radius=800.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+5].radius,output->data.r_v[4*num1+5].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+5].vl=(float)vl;
	(*output).data.vl_vr[4*num1+5].vr=(float)vr;

	(*output).data.r_v[4*num1+6].velocity=300.f;
	(*output).data.r_v[4*num1+6].radius=-800.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+6].radius,output->data.r_v[4*num1+6].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+6].vl=(float)vl;
	(*output).data.vl_vr[4*num1+6].vr=(float)vr;

	(*output).data.r_v[4*num1+7].velocity=400.f;
	(*output).data.r_v[4*num1+7].radius=0.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+7].radius,output->data.r_v[4*num1+7].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+7].vl=(float)vl;
	(*output).data.vl_vr[4*num1+7].vr=(float)vr;

	(*output).data.r_v[4*num1+8].velocity=400.f;
	(*output).data.r_v[4*num1+8].radius=1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+8].radius,output->data.r_v[4*num1+8].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+8].vl=(float)vl;
	(*output).data.vl_vr[4*num1+8].vr=(float)vr;

	(*output).data.r_v[4*num1+9].velocity=400.f;
	(*output).data.r_v[4*num1+9].radius=-1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+9].radius,output->data.r_v[4*num1+9].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+9].vl=(float)vl;
	(*output).data.vl_vr[4*num1+9].vr=(float)vr;

	//data added for faster speed
	//==============================================================================================
	(*output).data.r_v[4*num1+10].velocity=500.f;
	(*output).data.r_v[4*num1+10].radius=0.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+10].radius,output->data.r_v[4*num1+10].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+10].vl=(float)vl;
	(*output).data.vl_vr[4*num1+10].vr=(float)vr;

	(*output).data.r_v[4*num1+11].velocity=500.f;
	(*output).data.r_v[4*num1+11].radius=1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+11].radius,output->data.r_v[4*num1+11].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+11].vl=(float)vl;
	(*output).data.vl_vr[4*num1+11].vr=(float)vr;

	(*output).data.r_v[4*num1+12].velocity=500.f;
	(*output).data.r_v[4*num1+12].radius=-1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+12].radius,output->data.r_v[4*num1+12].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+12].vl=(float)vl;
	(*output).data.vl_vr[4*num1+12].vr=(float)vr;

	(*output).data.r_v[4*num1+13].velocity=600.f;
	(*output).data.r_v[4*num1+13].radius=0.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+13].radius,output->data.r_v[4*num1+13].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+13].vl=(float)vl;
	(*output).data.vl_vr[4*num1+13].vr=(float)vr;

	(*output).data.r_v[4*num1+14].velocity=600.f;
	(*output).data.r_v[4*num1+14].radius=1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+14].radius,output->data.r_v[4*num1+14].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+14].vl=(float)vl;
	(*output).data.vl_vr[4*num1+14].vr=(float)vr;

	(*output).data.r_v[4*num1+15].velocity=600.f;
	(*output).data.r_v[4*num1+15].radius=-1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+15].radius,output->data.r_v[4*num1+15].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+15].vl=(float)vl;
	(*output).data.vl_vr[4*num1+15].vr=(float)vr;

	(*output).data.r_v[4*num1+16].velocity=700.f;
	(*output).data.r_v[4*num1+16].radius=0.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+16].radius,output->data.r_v[4*num1+16].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+16].vl=(float)vl;
	(*output).data.vl_vr[4*num1+16].vr=(float)vr;

	(*output).data.r_v[4*num1+17].velocity=700.f;
	(*output).data.r_v[4*num1+17].radius=1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+17].radius,output->data.r_v[4*num1+17].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+17].vl=(float)vl;
	(*output).data.vl_vr[4*num1+17].vr=(float)vr;

	(*output).data.r_v[4*num1+18].velocity=700.f;
	(*output).data.r_v[4*num1+18].radius=-1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+18].radius,output->data.r_v[4*num1+18].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+18].vl=(float)vl;
	(*output).data.vl_vr[4*num1+18].vr=(float)vr;

	// 1200 high speed
	//===============================================================================================
	(*output).data.r_v[4*num1+19].velocity=400.f;
	(*output).data.r_v[4*num1+19].radius=800.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+19].radius,output->data.r_v[4*num1+19].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+19].vl=(float)vl;
	(*output).data.vl_vr[4*num1+19].vr=(float)vr;

	(*output).data.r_v[4*num1+20].velocity=400.f;
	(*output).data.r_v[4*num1+20].radius=-800.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+20].radius,output->data.r_v[4*num1+20].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+20].vl=(float)vl;
	(*output).data.vl_vr[4*num1+20].vr=(float)vr;

	(*output).data.r_v[4*num1+21].velocity=500.f;
	(*output).data.r_v[4*num1+21].radius=1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+21].radius,output->data.r_v[4*num1+21].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+21].vl=(float)vl;
	(*output).data.vl_vr[4*num1+21].vr=(float)vr;

	(*output).data.r_v[4*num1+22].velocity=500.f;
	(*output).data.r_v[4*num1+22].radius=-1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+22].radius,output->data.r_v[4*num1+22].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+22].vl=(float)vl;
	(*output).data.vl_vr[4*num1+22].vr=(float)vr;

	(*output).data.r_v[4*num1+23].velocity=600.f;
	(*output).data.r_v[4*num1+23].radius=1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+23].radius,output->data.r_v[4*num1+23].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+23].vl=(float)vl;
	(*output).data.vl_vr[4*num1+23].vr=(float)vr;

	(*output).data.r_v[4*num1+24].velocity=600.f;
	(*output).data.r_v[4*num1+24].radius=-1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+24].radius,output->data.r_v[4*num1+24].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+24].vl=(float)vl;
	(*output).data.vl_vr[4*num1+24].vr=(float)vr;

	(*output).data.r_v[4*num1+19].velocity=700.f;
	(*output).data.r_v[4*num1+19].radius=1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+19].radius,output->data.r_v[4*num1+19].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+19].vl=(float)vl;
	(*output).data.vl_vr[4*num1+19].vr=(float)vr;

	(*output).data.r_v[4*num1+25].velocity=700.f;
	(*output).data.r_v[4*num1+25].radius=-1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+25].radius,output->data.r_v[4*num1+25].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+25].vl=(float)vl;
	(*output).data.vl_vr[4*num1+25].vr=(float)vr;

	// speed 200
	// Data added
	(*output).data.r_v[4*num1+26].velocity=200.f;
	(*output).data.r_v[4*num1+26].radius=0.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+26].radius,output->data.r_v[4*num1+26].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+26].vl=(float)vl;
	(*output).data.vl_vr[4*num1+26].vr=(float)vr;

	(*output).data.r_v[4*num1+27].velocity=200.f;
	(*output).data.r_v[4*num1+27].radius=1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+27].radius,output->data.r_v[4*num1+27].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+27].vl=(float)vl;
	(*output).data.vl_vr[4*num1+27].vr=(float)vr;

	(*output).data.r_v[4*num1+28].velocity=200.f;
	(*output).data.r_v[4*num1+28].radius=-1600.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+28].radius,output->data.r_v[4*num1+28].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+28].vl=(float)vl;
	(*output).data.vl_vr[4*num1+28].vr=(float)vr;

	(*output).data.r_v[4*num1+29].velocity=200.f;
	(*output).data.r_v[4*num1+29].radius=1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+29].radius,output->data.r_v[4*num1+29].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+29].vl=(float)vl;
	(*output).data.vl_vr[4*num1+29].vr=(float)vr;

	(*output).data.r_v[4*num1+30].velocity=200.f;
	(*output).data.r_v[4*num1+30].radius=-1200.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+30].radius,output->data.r_v[4*num1+30].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+30].vl=(float)vl;
	(*output).data.vl_vr[4*num1+30].vr=(float)vr;

	(*output).data.r_v[4*num1+31].velocity=200.f;
	(*output).data.r_v[4*num1+31].radius=800.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+31].radius,output->data.r_v[4*num1+31].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+31].vl=(float)vl;
	(*output).data.vl_vr[4*num1+31].vr=(float)vr;

	(*output).data.r_v[4*num1+32].velocity=200.f;
	(*output).data.r_v[4*num1+32].radius=-800.0f;
	convert_RVtoVLVR(output->data.r_v[4*num1+32].radius,output->data.r_v[4*num1+32].velocity,&vr,&vl);
	(*output).data.vl_vr[4*num1+32].vl=(float)vl;
	(*output).data.vl_vr[4*num1+32].vr=(float)vr;


	


	/*for(int i=0;i<num1;i++){
		(*output).data.r_v[i+4*num1].radius=output->data.r_v[i+3*num1].radius;
		(*output).data.r_v[i+4*num1].velocity=output->data.r_v[i+3*num1].velocity/2;
		convert_RVtoVLVR(output->data.r_v[i+4*num1].radius,output->data.r_v[i+4*num1].velocity,&vr,&vl);
		(*output).data.vl_vr[i+4*num1].vl=vl;
		(*output).data.vl_vr[i+4*num1].vr=vr;

	}
	for(int i=0;i<num1;i++){
		(*output).data.r_v[i+5*num1].radius=output->data.r_v[i+3*num1].radius/2;
		(*output).data.r_v[i+5*num1].velocity=output->data.r_v[i+3*num1].velocity;
		convert_RVtoVLVR(output->data.r_v[i+5*num1].radius,output->data.r_v[i+5*num1].velocity,&vr,&vl);
		(*output).data.vl_vr[i+5*num1].vl=vl;
		(*output).data.vl_vr[i+5*num1].vr=vr;

	}*/
	
	printf("Number of motion set = %d\n",num_motion);
	for(int i=0;i<num_motion;i++){
		printf("[%d]:r=%.2f\tv=%.2f\tvr=%.2f\tvl=%.2f\n",i+1, output->data.r_v[i].radius, output->data.r_v[i].velocity, output->data.vl_vr[i].vr, output->data.vl_vr[i].vl);
	}
}

void PathPlannerV2::make_motion2(MOTION_CONTROL *output){
	static int num_motion=0;
	double vr,vl;
	int velo=0,radi=0,counter=0;
	
	num_motion=80;
	(*output).data.motion_num=num_motion;
	(*output).data.r_v = (RV_SET *)calloc(num_motion, sizeof(RV_SET));
	(*output).data.vl_vr = (VL_VR_SET *)calloc(num_motion, sizeof(VL_VR_SET));

	radi=-2000;
	velo=100;
	
	while(velo<=300){
		while(radi<=-800){
			(*output).data.r_v[counter].radius=(float)radi;
			(*output).data.r_v[counter].velocity=(float)velo;
			convert_RVtoVLVR(output->data.r_v[counter].radius,output->data.r_v[counter].velocity,&vr,&vl);
			(*output).data.vl_vr[counter].vl=vl;
			(*output).data.vl_vr[counter].vr=vr;
			radi+=200;
			counter++;
		}
		velo+=100;
		radi=-2000;
	}

	radi=600;
	velo=100;
	
	while(velo<=300){
		while(radi<=2000){
			(*output).data.r_v[counter].radius=(float)radi;
			(*output).data.r_v[counter].velocity=(float)velo;
			convert_RVtoVLVR(output->data.r_v[counter].radius,output->data.r_v[counter].velocity,&vr,&vl);
			(*output).data.vl_vr[counter].vl=vl;
			(*output).data.vl_vr[counter].vr=vr;
			radi+=200;
			counter++;
		}
		velo+=100;
		radi=800;
	}

	radi=-2000;
	velo=400;
	
	while(velo<=600){
		while(radi<=-1200){
			(*output).data.r_v[counter].radius=(float)radi;
			(*output).data.r_v[counter].velocity=(float)velo;
			convert_RVtoVLVR(output->data.r_v[counter].radius,output->data.r_v[counter].velocity,&vr,&vl);
			(*output).data.vl_vr[counter].vl=vl;
			(*output).data.vl_vr[counter].vr=vr;
			radi+=200;
			counter++;
		}
		velo+=100;
		radi=-2000;
	}

	radi=1200;
	velo=400;
	
	while(velo<=600){
		while(radi<=2000){
			(*output).data.r_v[counter].radius=(float)radi;
			(*output).data.r_v[counter].velocity=(float)velo;
			convert_RVtoVLVR(output->data.r_v[counter].radius,output->data.r_v[counter].velocity,&vr,&vl);
			(*output).data.vl_vr[counter].vl=vl;
			(*output).data.vl_vr[counter].vr=vr;
			radi+=200;
			counter++;
		}
		velo+=100;
		radi=1200;
	}

	velo=100;

	while(velo<=600){
			(*output).data.r_v[counter].radius=0.f;
			(*output).data.r_v[counter].velocity=(float)velo;
			convert_RVtoVLVR(output->data.r_v[counter].radius,output->data.r_v[counter].velocity,&vr,&vl);
			(*output).data.vl_vr[counter].vl=vl;
			(*output).data.vl_vr[counter].vr=vr;
			counter++;
			velo+=100;
	}

		
	
	printf("Number of motion set = %d\n",num_motion);
	for(int i=0;i<num_motion;i++){
		printf("[%d]:r=%.2f\tv=%.2f\tvr=%.2f\tvl=%.2f\n",i+1, output->data.r_v[i].radius, output->data.r_v[i].velocity, output->data.vl_vr[i].vr, output->data.vl_vr[i].vl);
	}
}

void PathPlannerV2::pathArrayControl(void){

	if(++step >= MAXSTEP){
		printf("Path Number Exceed....Restarting path....\n");
		step = MAXSTEP / 2;
		for(int i=0;i<(MAXSTEP / 2);i++){
			path[i] = path[i+(MAXSTEP / 2)];
		}
	}

}

void PathPlannerV2::error_action(float velocity, float radius){
			
	path[step+1] = path[step];
	path[step+1].velocity = velocity;//0 the expected speed of change the next time
	path[step+1].radius = radius;
	path[step+2] = path[step+1];
	path[step+2].velocity = velocity;//0 the expected speed of change the next time
	path[step+2].radius = radius;
	pathArrayControl();
	path_depth=0;
			
drawing_whole();
WriteMovingObject(LocalImg , array_Mobje , NumOfMobje , robot_data);
Draw_LMap_Data_Err(LocalImg, path, tree, xs, xg_local_sub, n, path_depth, step, window_title_Local, robot_data, NULL, &robot_state_tmp, &warning_flag);
drawing_when_error();

#ifdef ___DEBUG_FILE___
			fprintf_s(debug_file, "--------------------\nonExecute end(line %d): Robot collision\n--------------------\n", __LINE__);
#endif
}

bool PathPlannerV2::check_LM_connection(){
	if( CORBA::is_nil(m_RelativeMapService.getObject()) ){

		printf("Service port is not conected with service provider.\n");
		return false;
	}

	if( m_RelativeMapService.getObject()->_non_existent() ){
	
		printf("Service provider is not activated.\n");
		return false;
	}

	return true;
}


// ############################ EXECUTE FUNCTION ################################### //
// ################################################################################# //
void PathPlannerV2::send_next_control(){
	m_robot_control.data.vx = path[step+1].velocity*0.001;
	if(path[step+1].radius != 0.0){
		m_robot_control.data.va = (path[step+1].velocity)/(path[step+1].radius*(-1));
	}else{
		m_robot_control.data.va = 0;
	}
	//m_robot_controlOut.write(m_robot_control);
	m_robot_controlOut.write();
#ifdef ___DEBUG_FILE___
	fprintf_s(debug_file, "Velocity:%.6lf[m/s] Radius:%.6lf[rad/s]\n", m_robot_control.data.vx, m_robot_control.data.va);
#endif
	printf("m_robot_control::Velocity:%.6lf[m/s] Radius:%.6lf[rad/s]\n", m_robot_control.data.vx, m_robot_control.data.va);

}

void PathPlannerV2::memory_consumption(){
	if(consumption_memory_size < 1000){
		printf("Memory consumptionF%d Byte\n", consumption_memory_size);
	}else if((consumption_memory_size >= 1000) && (consumption_memory_size < 1000000)){
		printf("Memory consumptionF%.1lf KByte\n", consumption_memory_size/1000.0);
	}else if((consumption_memory_size >= 1000000) && (consumption_memory_size < 1000000000)){
		printf("Memory consumptionF%.1lf MByte\n", consumption_memory_size/1000000.0);
	}else{
		printf("Memory consumptionF%.1lf GByte\n", consumption_memory_size/1000000000.0);
	}
}

void PathPlannerV2::get_robot_state(){

time_tmp = timeGetTime();

	
#ifdef ___DEBUG_FILE___
		fprintf_s(debug_file, "Start getting the robot position LINE: %d\n", __LINE__);
#endif
		while(m_robot_poseIn.isNew() == false);
		m_robot_poseIn.read();
		printf("Robot position acquisition time: %d[msec]\n", timeGetTime()-time_tmp);
#ifdef ___DEBUG_FILE___
		fprintf_s(debug_file, "Get robot position completed LINE: %d, time: %d[msec]\n", __LINE__, timeGetTime()-time_tmp);
#endif
		//Change units
		robot_state_tmp.x = m_robot_pose.data.position.x * 1000;
		robot_state_tmp.y = m_robot_pose.data.position.y * 1000;
		robot_state_tmp.theta = m_robot_pose.data.heading * R2D;
	

// Newly add 24-5-11
	while(robot_state_tmp.theta >= 180.f){
		robot_state_tmp.theta -= 360.f;
	}
	while(robot_state_tmp.theta < -180.f){
		robot_state_tmp.theta += 360.f;
	}

}

void PathPlannerV2::get_robot_velocity(){

#ifdef ___DEBUG_FILE___
		fprintf_s(debug_file, "Start getting the robot speed LINE: %d\n", __LINE__);
#endif
	time_tmp = timeGetTime();
	while(m_robot_velocityIn.isNew() == false);
	m_robot_velocityIn.read();
	//printf("Robot speed acquisition time: %d[msec]\n", timeGetTime()-time_tmp);
#ifdef ___DEBUG_FILE___
		fprintf_s(debug_file, "Get robot speed completed LINE: %d, time: %d[msec]\n", __LINE__, timeGetTime()-time_tmp);
#endif
	robot_state_tmp.v =m_robot_velocity.data.vx*1000;
	robot_state_tmp.w = m_robot_velocity.data.va;//[rad]
	robot_state_tmp.vl = robot_state_tmp.v - robot_state_tmp.w * (Wheel_Tread/2);
	robot_state_tmp.vr = robot_state_tmp.v + robot_state_tmp.w * (Wheel_Tread/2);


#ifdef ___DEBUG_FILE___
	if(path[step].radius != 0.0){
		fprintf_s(debug_file, "Assumed value:vl = %.4lf, vr = %.4lf\n", path[step].velocity * (1 + Wheel_Tread/(2*path[step].radius)), path[step].velocity * (1 - Wheel_Tread/(2*path[step].radius)));
	}else{
		fprintf_s(debug_file, "Assumed value:vl = %.4lf, vr = %.4lf\n", path[step].velocity, path[step].velocity);
	}
	fprintf_s(debug_file, "Measured value:vl = %.4lf, vr = %.4lf\n", robot_state_tmp.vl, robot_state_tmp.vr);
#endif
}





void PathPlannerV2::predict_and_store_robot_pose(){

	/***** Current Location *****/
	path[step].direction = (float)(xs[3] - robot_state_tmp.theta*DEG_to_RAD);
	double u,v;
	u = (double)robot_state_tmp.x * _cos_rad  +  (double)(-robot_state_tmp.y) * _sin_rad;//Rounds
	v = (double)(-robot_state_tmp.y) * _cos_rad  -  (double)robot_state_tmp.x * _sin_rad;//Rounds
	path[step].pose.y    = (float)v + xs[2];
	path[step].pose.x    = (float)u + xs[1];

	printf("Robot is hereFx=%.2f , y=%.2lf , direction=%.2lf\n", path[step].pose.x , path[step].pose.y , path[step].direction*RAD_to_DEG);

	/***** Prediction of the position the next time *****/
	//Predict the location and position of the next rate
	//Acquisition speed

// Using Old Method
	float vr,vl,R,delta_L_dash;

	if(path[step+1].radius != 0.0){
		vl = float(path[step+1].velocity * (1 + Wheel_Tread/(2*path[step+1].radius)));
		vr = float(path[step+1].velocity * (1 - Wheel_Tread/(2*path[step+1].radius)));
	}else{
		vl = path[step+1].velocity;
		vr = path[step+1].velocity;
	}
	// Shape of the function will be replaced with the following speed settings
	vl += (float)robot_state_tmp.vl;
	vr += (float)robot_state_tmp.vr;
	vr /= 2;
	vl /= 2;
	float omega,delta_theta;
	omega = float((vl - vr) / Wheel_Tread);//Velocity calculation
	//delta_theta = float(omega*(LoopTime-((timeGetTime()-loopTimer)/1000)));//rad
	delta_theta = float(omega*(LoopTime));//rad

	if(vr != vl){
		R = float((Wheel_Tread/2.0) * (vr + vl)/(vl - vr));//Turning radius
		delta_L_dash = 2.f*R*sin(delta_theta/2);
	}
	else{
		//delta_L_dash = float(vr*( LoopTime - ((timeGetTime()-loopTimer)/1000) ));
		delta_L_dash = float(vr*( LoopTime));
	}
	path[step+1].direction = path[step].direction + delta_theta;
	path[step+1].pose.x    = float(path[step].pose.x + delta_L_dash * cos(path[step].direction + delta_theta/2.0));
	path[step+1].pose.y    = float(path[step].pose.y + delta_L_dash * sin(path[step].direction + delta_theta/2.0));

//Using New Method
	/*float omega_motion,velo_motion;
	velo_motion=(path[step+1].velocity+path[step].velocity)/2;
	if(path[step+1].radius != 0.0)omega_motion=velo_motion/((path[step+1].radius));
	if(path[step+1].radius != 0.0){
		path[step+1].pose.x=path[step].pose.x-(path[step+1].radius*sin(path[step].direction))+(path[step+1].radius*sin(path[step].direction+(omega_motion*(LoopTime))));
		path[step+1].pose.y=path[step].pose.y+(path[step+1].radius*cos(path[step].direction))-(path[step+1].radius*cos(path[step].direction+(omega_motion*(LoopTime))));
		path[step+1].direction=path[step].direction+omega_motion*(LoopTime);
	}else{
		path[step+1].pose.x=path[step].pose.x+velo_motion*cos(path[step].direction)*(LoopTime);
		path[step+1].pose.y=path[step].pose.y+velo_motion*sin(path[step].direction)*(LoopTime);
		path[step+1].direction=path[step].direction;
	}*/

	printf("path predict x=%f y=%f theta=%f\n",path[step+1].pose.x,path[step+1].pose.y,path[step+1].direction);



//################# Store current robot position and next predicted robot position ##################//

	//Given the current robot position
	robot.SetPosition(path[step].pose.x , path[step].pose.y);//Robot location update
	robot.direction = path[step].direction;//Angle Update

	
	//Current Location
	robot_data.now.direction  = path[step].direction;
	robot_data.now.pose       = path[step].pose;
	robot_data.now.local_position.x = G_SELL_SIZE/GRID_SIZE/2;
	robot_data.now.local_position.y = G_SELL_SIZE/GRID_SIZE/2;

	//Setting might be in the position the next time the robot
	robot_data.next.direction = path[step+1].direction;
	robot_data.next.pose      = path[step+1].pose;
	robot_data.next.local_position = Convert_Global2Local(robot_data.now.pose,robot_data.now.direction , path[step+1].pose);//I get on like this.


}



void PathPlannerV2::get_data_waypoint(){

float goalAreaRadiusTmp;

waypoint_end_flag = 0;

		if(m_subgoal_waypointIn.isNew() == false){//not updated
			if((int)m_subgoal_waypoint.data.length()>0&&initial_waypoint==true){
				if(waypoint_pointer == (waypoint_data_length - 1)){//last waypoint
					goalAreaRadiusTmp = (float)goalAreaRadius;
				}else{
					goalAreaRadiusTmp = (float)goalAreaRadius;
				}

				while(_robotToGoalDist() < goalAreaRadiusTmp){//waypoint less than some metric
					waypoint_pointer++;//next waypoint transition
					if(waypoint_pointer == (waypoint_data_length - 1)){
						goalAreaRadiusTmp = (float)goalAreaRadius;
					}
					if(waypoint_pointer >= waypoint_data_length){//last waypoint
						waypoint_pointer = waypoint_data_length-1;
						waypoint_end_flag = 1;//reach end waypoint flag
					}
					try{
					xg[1] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.position.x * 1000.f;//x
					xg[2] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.position.y) * 1000.f;//y
					//xg[3] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.heading*D2R;//theta(degree)
					xg[3] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.heading);//theta(radian)
					}
					catch(int param){
						warning_flag |= WAYPOINT_DATA_LENGTH_IS_0;
						waypoint_end_flag = 1;
					}
					if(waypoint_end_flag == 1){
						break;
					}
				}
			
			}
			else{
				xg[1] = robot_data.now.pose.x;//x
				xg[2] = robot_data.now.pose.y;//y
				xg[3] = robot_data.now.direction;//theta(radian)
				warning_flag |= WAYPOINT_DATA_LENGTH_IS_0;
				waypoint_end_flag = 1;
			}
		}else{//updated waypoint
			initial_waypoint=true;
			waypoint_pointer = 0;
			//if(use_waypoint_heading==1)waypoint_state_change=1;else waypoint_state_change=2;
			waypoint_trace_complete_num = 0;//initialization

			//do{//read latest data buffer
				m_subgoal_waypointIn.read();
			//}while(m_subgoal_waypointIn.isNew());
						
			waypoint_data_length = m_subgoal_waypoint.data.length();//waypoint aquisition length
			if(waypoint_data_length == 0){//stop at that point (not implemented)
				waypoint_end_flag = 1;//reach end waypoint flag
			}else{//there is a data
				if(waypoint_data_length == 1){//last waypoint
					goalAreaRadiusTmp = (float)goalAreaRadius;
				}else{
					goalAreaRadiusTmp = (float)goalAreaRadius;
				}


				xg[1] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.position.x * 1000.f;//x
				xg[2] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.position.y) * 1000.f;//y
				//xg[3] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.heading*D2R;//theta(degree)
				xg[3] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.heading);//theta(radian)

				while(_robotToGoalDist() < (float)goalAreaRadiusTmp){//when reach waypoint
					waypoint_pointer++;//next waypoint transition
					if(waypoint_pointer == (waypoint_data_length - 1)){
						goalAreaRadiusTmp = (float)goalAreaRadius;
					}
					if(waypoint_pointer >= waypoint_data_length){//last waypoint
						waypoint_pointer = waypoint_data_length-1;
						waypoint_end_flag = 1;//reach end waypoint flag
					}
					xg[1] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.position.x*1000.f;//x
					xg[2] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.position.y)*1000.f;//y
					//xg[3] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.heading*D2R;//theta(degree)
					xg[3] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.heading);//theta(radian)
					if(waypoint_end_flag == 1){
						break;
					}
				}
			}
		}

				
//state change between waypoint//
		if(temp_way_x==xg[1]&&temp_way_y==xg[2]){}
		else{
		//if(temp_way_x!=xg[1]||temp_way_y!=xg[2]){// if waypoint
			temp_way_x=xg[1];
			temp_way_y=xg[2];
			if(use_waypoint_heading==1)waypoint_state_change=1;else waypoint_state_change=2;// change state of waypont to turn state or turn waypoint
			if(waypoint_pointer-1>=0)xg[3]=(float)(-m_subgoal_waypoint.data[waypoint_pointer-1].pose.heading);//theta(radian)//xg[3]=(float)m_subgoal_waypoint.data[waypoint_pointer-1].pose.heading*D2R;//theta(degree)
		}

}

void PathPlannerV2::get_data_waypoint2(){

float goalAreaRadiusTmp;

waypoint_end_flag = 0;

		if(m_subgoal_waypointIn.isNew() == false){//not updated
			if((int)m_subgoal_waypoint.data.length()>0&&initial_waypoint==true){
				if(waypoint_pointer == (waypoint_data_length - 1)){//last waypoint
					goalAreaRadiusTmp = (float)goalAreaRadius;
				}else{
					goalAreaRadiusTmp = 1000.f;//(float)goalAreaRadius;
				}

				while(_robotToGoalDist() < goalAreaRadiusTmp){//waypoint less than some metric
					waypoint_pointer++;//next waypoint transition
					if(waypoint_pointer == (waypoint_data_length - 1)){
						goalAreaRadiusTmp = (float)goalAreaRadius;
					}
					if(waypoint_pointer >= waypoint_data_length){//last waypoint
						waypoint_pointer = waypoint_data_length-1;
						waypoint_end_flag = 1;//reach end waypoint flag
					}
					try{
					xg[1] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.position.x * 1000.f;//x
					xg[2] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.position.y) * 1000.f;//y
					//xg[3] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.heading*D2R;//theta(degree)
					xg[3] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.heading);//theta(radian)
					}
					catch(int param){
						warning_flag |= WAYPOINT_DATA_LENGTH_IS_0;
						waypoint_end_flag = 1;
					}
					if(waypoint_end_flag == 1){
						break;
					}
				}
			
			}
			else{
				xg[1] = robot_data.now.pose.x;//x
				xg[2] = robot_data.now.pose.y;//y
				xg[3] = robot_data.now.direction;//theta(radian)
				warning_flag |= WAYPOINT_DATA_LENGTH_IS_0;
				waypoint_end_flag = 1;
			}
		}else{//updated waypoint
			initial_waypoint=true;
			waypoint_pointer = 0;
			//if(use_waypoint_heading==1)waypoint_state_change=1;else waypoint_state_change=2;
			waypoint_trace_complete_num = 0;//initialization

			//do{//read latest data buffer
				m_subgoal_waypointIn.read();
			//}while(m_subgoal_waypointIn.isNew());
						
			waypoint_data_length = m_subgoal_waypoint.data.length();//waypoint aquisition length
			if(waypoint_data_length == 0){//stop at that point (not implemented)
				waypoint_end_flag = 1;//reach end waypoint flag
			}else{//there is a data
				if(waypoint_data_length == 1){//last waypoint
					goalAreaRadiusTmp = (float)goalAreaRadius;
				}else{
					goalAreaRadiusTmp = 1000.f;//(float)goalAreaRadius;
				}


				xg[1] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.position.x * 1000.f;//x
				xg[2] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.position.y) * 1000.f;//y
				//xg[3] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.heading*D2R;//theta(degree)
				xg[3] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.heading);//theta(radian)

				while(_robotToGoalDist() < (float)goalAreaRadiusTmp){//when reach waypoint
					waypoint_pointer++;//next waypoint transition
					if(waypoint_pointer == (waypoint_data_length - 1)){
						goalAreaRadiusTmp = (float)goalAreaRadius;
					}
					if(waypoint_pointer >= waypoint_data_length){//last waypoint
						waypoint_pointer = waypoint_data_length-1;
						waypoint_end_flag = 1;//reach end waypoint flag
					}
					xg[1] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.position.x*1000.f;//x
					xg[2] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.position.y)*1000.f;//y
					//xg[3] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.heading*D2R;//theta(degree)
					xg[3] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.heading);//theta(radian)
					if(waypoint_end_flag == 1){
						break;
					}
				}
			}
		}

		waypoint_direction_temp=atan2(xg[2]-(float)robot_data.now.pose.y,xg[1]-(float)robot_data.now.pose.x)-robot_data.now.direction;
		while(waypoint_direction_temp > PI){
			waypoint_direction_temp -= 2.f*PI;
		}
		while(waypoint_direction_temp < -PI){
			waypoint_direction_temp += 2.f*PI;
		}
		if(ABS(waypoint_direction_temp) < (PI/180.f)*45){
			temp_way_x=xg[1];
			temp_way_y=xg[2];
		}
		
//state change between waypoint//
		if(temp_way_x==xg[1]&&temp_way_y==xg[2]){}
		else{
		//if(temp_way_x!=xg[1]||temp_way_y!=xg[2]){// if waypoint
			temp_way_x=xg[1];
			temp_way_y=xg[2];
			if(use_waypoint_heading==1)waypoint_state_change=1;else waypoint_state_change=2;// change state of waypont to turn state or turn waypoint
			if(waypoint_pointer-1>=0)xg[3]=(float)(-m_subgoal_waypoint.data[waypoint_pointer-1].pose.heading);//theta(radian)//xg[3]=(float)m_subgoal_waypoint.data[waypoint_pointer-1].pose.heading*D2R;//theta(degree)
		}

}

void PathPlannerV2::get_data_waypoint_global(){
static float dx,dy,dr;
static int i;
static CvPoint local;
static Position pose;
float goalAreaRadiusTmp;

if(m_subgoal_waypointIn.isNew() == true){//updated
		initial_waypoint=true;
		waypoint_pointer = 0;
		m_subgoal_waypointIn.read();
		waypoint_data_length = m_subgoal_waypoint.data.length();
		if(waypoint_data_length>1){
			i=0;
			for(int j=0;j<waypoint_data_length;j++){
				xg[1] = (float)m_subgoal_waypoint.data[j].pose.position.x * 1000.f;//x
				xg[2] = (float)(-m_subgoal_waypoint.data[j].pose.position.y) * 1000.f;//y
				//xg[3] = (float)m_subgoal_waypoint.data[j].pose.heading*D2R;//theta(degree)
				xg[3] = (float)(-m_subgoal_waypoint.data[j].pose.heading);//theta(radian)
				pose.x=xg[1];
				pose.y=xg[2];
				local=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,pose);
				if(local.x<0 || local.y<0){if(j>0)i=j-1;else i=0;break;}
			}
			if(i>2)i=2;
			if(i==0)i=1;
			xg[1] = (float)m_subgoal_waypoint.data[i].pose.position.x * 1000.f;//x
			xg[2] = (float)(-m_subgoal_waypoint.data[i].pose.position.y) * 1000.f;//y
			//xg[3] = (float)m_subgoal_waypoint.data[i].pose.heading*D2R;//theta(degree)
			xg[3] = (float)(-m_subgoal_waypoint.data[i].pose.heading);//theta(radian)
			//goalAreaRadius = int(m_GOAL_AREA*1000/4);
			//dx=(float)m_subgoal_waypoint.data[0].pose.position.x * 1000.f;
			//dy=-(float)m_subgoal_waypoint.data[0].pose.position.y * 1000.f;
			//dr=(float)sqrt(POW(dx-robot_data.now.pose.x)+POW(dy-robot_data.now.pose.y));
			//if(dr<2*goalAreaRadius){
			//	if(waypoint_data_length>2){
			//	xg[1] = (float)m_subgoal_waypoint.data[2].pose.position.x * 1000.f;//x
			//	xg[2] = (float)(-m_subgoal_waypoint.data[2].pose.position.y) * 1000.f;//y
			//	xg[3] = (float)m_subgoal_waypoint.data[2].pose.heading*D2R;//theta(degree)
			//	}
			//	else{
			//	xg[1] = (float)m_subgoal_waypoint.data[1].pose.position.x * 1000.f;//x
			//	xg[2] = (float)(-m_subgoal_waypoint.data[1].pose.position.y) * 1000.f;//y
			//	xg[3] = (float)m_subgoal_waypoint.data[1].pose.heading*D2R;//theta(degree)
			//	}
		//}
			//else{
			//	xg[1] = (float)m_subgoal_waypoint.data[0].pose.position.x * 1000.f;//x
			//	xg[2] = (float)(-m_subgoal_waypoint.data[0].pose.position.y) * 1000.f;//y
			//	xg[3] = (float)m_subgoal_waypoint.data[0].pose.heading*D2R;//theta(degree)
			//}
		}
		else{
			goalAreaRadius = int(m_GOAL_AREA*1000);
			xg[1] = (float)m_subgoal_waypoint.data[0].pose.position.x * 1000.f;//x
			xg[2] = (float)(-m_subgoal_waypoint.data[0].pose.position.y) * 1000.f;//y
			//xg[3] = (float)m_subgoal_waypoint.data[0].pose.heading*D2R;//theta(degree)
			xg[3] = (float)(-m_subgoal_waypoint.data[0].pose.heading);//theta(radian)
		}
	}
else{
	if((int)m_subgoal_waypoint.data.length()<=0||initial_waypoint==false){
		xg[1] = robot_data.now.pose.x;//x
		xg[2] = robot_data.now.pose.y;//y
		xg[3] = robot_data.now.direction;//theta(radian)
		warning_flag |= WAYPOINT_DATA_LENGTH_IS_0;
		waypoint_end_flag = 1;
	}
	else{
		if(waypoint_pointer == (waypoint_data_length - 1)){//last waypoint
					goalAreaRadiusTmp = (float)goalAreaRadius;
				}else{
					goalAreaRadiusTmp = 3500.f;//(float)goalAreaRadius*4;
				}

				while(_robotToGoalDist() < goalAreaRadiusTmp){//waypoint less than some metric
					waypoint_pointer++;//next waypoint transition
					if(waypoint_pointer == (waypoint_data_length - 1)){
						goalAreaRadiusTmp = (float)goalAreaRadius;
					}
					if(waypoint_pointer >= waypoint_data_length){//last waypoint
						waypoint_pointer = waypoint_data_length-1;
						waypoint_end_flag = 1;//reach end waypoint flag
					}
				
					xg[1] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.position.x * 1000.f;//x
					xg[2] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.position.y) * 1000.f;//y
					//xg[3] = (float)m_subgoal_waypoint.data[waypoint_pointer].pose.heading*D2R;//theta(degree)
					xg[3] = (float)(-m_subgoal_waypoint.data[waypoint_pointer].pose.heading);//theta(radian)
					if(waypoint_end_flag == 1){
						break;
					}
				}
	}
}

}

void PathPlannerV2::escape_from_collision(){
	//if(collision_state==10){m_robot_control.data.vx = -250;keep_velocity=1;collision_state=9;}
	//else if(collision_state==10){m_robot_control.data.vx = -250;keep_velocity=1;collision_state=1;}
	//else{ 
		if(collision_state==1)orientation=quadran(Gauss_f32);
		//printf("quadran = %d\n",orientation);
		exit_robot();
		warning_flag|=TURNING;
		
		if((orientation==2 || orientation==3) && collision_state==1)
			{
				collision_state=2;
				collision_direction=robot_data.now.direction-PI/3;
				collision_director.x=robot_data.now.pose.x+1000*cos(collision_direction);
				collision_director.y=robot_data.now.pose.y+1000*sin(collision_direction);
			}
		else if((orientation==1 || orientation==4) && collision_state==1)
			{
				collision_state=3;
				collision_direction=robot_data.now.direction+PI/3;
				collision_director.x=robot_data.now.pose.x+1000*cos(collision_direction);
				collision_director.y=robot_data.now.pose.y+1000*sin(collision_direction);
			}
		xg[3]=atan2(collision_director.y-(float)robot_data.now.pose.y,collision_director.x-(float)robot_data.now.pose.x)-robot_data.now.direction;

		while(xg[3] > PI){
			xg[3] -= 2.f*PI;
		}
		while(xg[3] < -PI){
			xg[3] += 2.f*PI;
		}

		if(ABS(xg[3]) > (PI/180.f)*5){//If more than 5 degrees angle difference
			m_robot_control.data.vx = 0;
			if(ABS(xg[3]) > (PI/180.f)*30){//If it was 15 degrees or more, capping
				if(xg[3] > 0){
					m_robot_control.data.va = -0.523;//rad(30Degree/sec)
				}else{
					m_robot_control.data.va = 0.523;//rad(30Degree/sec)
				}
			}else{
				m_robot_control.data.va = -xg[3]/2;//rad
			}
			m_robot_controlOut.write(m_robot_control);
			keep_velocity=1;
		}else{
			m_robot_control.data.vx = 0;
			m_robot_control.data.va = 0;
			m_robot_controlOut.write(m_robot_control);
			collision_state=0;
			keep_velocity=0;
		}
	//}

}

float PathPlannerV2::read_distance_wall_right(){
int x,y;
float distance;

x=100;y=90;

while(1){
	if((LocalMap[x][y] & (SFLAG) !=0))break;
	if(x>=0)x++;else break;
}

distance=(float)(x-100)*50;

printf("distance wall right %.2lf \n",distance);

return distance;

}

float PathPlannerV2::read_distance_wall_left(){
int x,y;
float distance;

x=100;y=90;

while(1){
	if((LocalMap[x][y] & (SFLAG) !=0))break;
	if(x>=0)x--;else break;
}

distance=(float)(100-x)*50;

printf("distance wall left %.2lf \n",distance);

return distance;

}

float PathPlannerV2::read_distance(){
int x,y;
float distance;

x=100;y=100;

while(1){
	if((LocalMap[x][y] & (SFLAG) !=0))break;
	if(y>=0)y--;else break;
}

distance=(float)(100-y)*50;

printf("distance %.2lf \n",distance);

return distance;

}

void PathPlannerV2::rotation(float sudut,int status){
			xg[3]=sudut*D2R;
			xg[3]-=robot_data.now.direction;
			while(xg[3] > PI){
				xg[3] -= 2.f*PI;
			}
			while(xg[3] < -PI){
			xg[3] += 2.f*PI;
			}

			if(ABS(xg[3]) > (PI/180.f)*5){//If more than 10 degrees, angle difference
				m_robot_control.data.vx = 0;
				if(ABS(xg[3]) > (PI/180.f)*30){//If it was 15 degrees or more, capping
					if(xg[3] > 0){
						m_robot_control.data.va = -0.523;//rad(30Degree/sec)
					}else{
						m_robot_control.data.va = 0.523;//rad(30Degree/sec)
					}
				}else{
					m_robot_control.data.va = -xg[3]/2;//rad
				}
				m_robot_controlOut.write(m_robot_control);
				keep_velocity=1;
				error_action(0,0);
				states=status-1;
				
			}else{
				path[step+2].velocity=0;
				path[step+2].radius=0;
				keep_velocity=0;
				states=status;
			}
}

// ################################# END OF EXECUTE FUNCTION ###################################### //

inline void SetIgray(IplImage *image,int x,int y,unsigned char value){
	if(image->nChannels!=1){
		fprintf(stderr,"Error SetIgray\n");
		exit(-1);
	}

	if(x<0 || x>=image->width || y<0 || y>=image->height){
		fprintf(stderr,"index error(SetIgray):x=%d,y=%d\n",x,y);
		exit(-1);
	}

	((unsigned char*)(image->imageData + image->widthStep*y))[x]=value;
}

inline bool emergency(unsigned char **LocalMap,IplImage *image,CvPoint local_pose)
{
	static int m,n;
	static float value_left,value_right;
	static bool dir;

	value_left=0;value_right=0;
for(int i=-4;i<4;i++)
for(int j=-4;j<4;j++){
{
	m=local_pose.x+i;n=local_pose.y+j;
	if((LocalMap[m][n] & (R_AvailableFLAG) !=0) && i<0)value_left+=cvGetReal2D(image,m,n);
	else if((LocalMap[m][n] & (R_AvailableFLAG) !=0) && i>=0)value_right+=cvGetReal2D(image,m,n);
}
}
if(value_left>value_right)dir=false;else dir=true;

return dir;

}

/* Time measurement */
DWORD skeleton_time = 0, tmp_time1 = 0, tmp_time2 = 0;
int skeleton_func_num = 0;

DWORD levelset_time1=0, levelset_time2=0, levelset_time3=0;
int levelset_num = 0;




// Module specification
// <rtc-template block="module_spec">
static const char* pathplannerv2_spec[] =
  {
    "implementation_id", "PathPlannerV2",
    "type_name",         "PathPlannerV2",
    "description",       "ModuleDescription",
    "version",           "2.0.0",
    "vendor",            "AISL",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.GOAL_AREA", "0.200",
    "conf.default.LOOP_TIME", "0.50",
    "conf.default.ROBOT_ACCELERATION", "0.300",
    "conf.default.ROBOT_MAX_SPEED", "0.300",
	"conf.default.SETPOINT_1", "300",
	"conf.default.SETPOINT_2", "800",
	"conf.default.CHECKPOINT_1", "500",
	"conf.default.CHECKPOINT_2", "800",
	"conf.default.CHECKPOINT_3", "1000",
	"conf.default.CHECKPOINT_4", "1000",
	
    // Widget
    "conf.__widget__.GOAL_AREA", "text",
    "conf.__widget__.LOOP_TIME", "text",
    "conf.__widget__.ROBOT_ACCELERATION", "text",
    "conf.__widget__.ROBOT_MAX_SPEED", "text",
	"conf.__widget__.SETPOINT_1", "text",
	"conf.__widget__.SETPOINT_2", "text",
	"conf.__widget__.CHECKPOINT_1", "text",
	"conf.__widget__.CHECKPOINT_2", "text",
	"conf.__widget__.CHECKPOINT_3", "text",
	"conf.__widget__.CHECKPOINT_4", "text",
	
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
PathPlannerV2::PathPlannerV2(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_robot_poseIn("robot_pose", m_robot_pose),
    m_robot_velocityIn("robot_velocity", m_robot_velocity),
    m_subgoal_waypointIn("subgoal_waypoint", m_subgoal_waypoint),
	m_in_messageIn("in_message",m_in_message),
    m_robot_controlOut("robot_control", m_robot_control),
    m_planning_imgOut("planning_img", m_planning_img),
	m_messageOut("message", m_message),
    m_RelativeMapServicePort("RelativeMapService")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
PathPlannerV2::~PathPlannerV2()
{
}



RTC::ReturnCode_t PathPlannerV2::onInitialize()
{
printf("PathPlanner: onInitialize\n");

	move_obje = NULL;
	for(int i=0;i<OBJECT_MAX_NUM;i++){
		array_Mobje[i] = NULL;
	}
	conter = NULL;
	simulator = NULL;
	LocalImg = NULL;
	Gauss_f32 = NULL;
	Gauss_u8 = NULL;
	LocalMap = NULL;
	window_output = NULL;
	activate_count= 0;
	for(int i=0;i<BUF;i++){
		window_title_simu[i] = 0;
		window_title_Local[i] = 0;
	}
	end3 = 0;
	total3 = 0;
	max3 = 0;
	draw_time = 0;
	int step = 0;
	int n = 0;
	bool flag = 0;
	for(int i=0;i<4;i++){
		xs[i] = 0;//0:nan 1:x-coordinate 2:y coordinate 3:Direction
		xg[i] = 0;//0:nan 1:x-coordinate 2:y coordinate 3:Direction
		xg_global_sub[i] = 0;
		xg_local_sub[i] = 0;
	}
	path_depth = 0;
	NumOfMobje = 0;
	simu_start = 0;
	simu_end = 0;
	calc_Draw_ave = 0;
	calc_Draw_max = 0;
	_cos_rad = 0;
	_sin_rad = 0;
	warning_flag = 0;//Collision detection, time-over detection flag
	trace_id = 0;
	waypoint_trace_complete_num = 0;//Waypoint available when using.
	waypoint_data_length = 0;//
	waypoint_pointer = 0;
	activate_count = 0;
	pre_arrive_flag = false;
	data_people = NULL;
	m_counter = 0;
	keep_velocity=0;

	robot_data.now.pose.x=0;
	robot_data.now.pose.y=0;
	robot_data.now.direction=0;

#if FP_TEST//Data storage
	saveFile.make_node_counter = 0;
	saveFile.average_make_node_num = 0;
#endif

  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("robot_pose", m_robot_poseIn);
  addInPort("robot_velocity", m_robot_velocityIn);
  addInPort("subgoal_waypoint", m_subgoal_waypointIn);
  addInPort("in_message", m_in_messageIn);
    
  // Set OutPort buffer
  addOutPort("robot_control", m_robot_controlOut);
  addOutPort("planning_img", m_planning_imgOut);
  addOutPort("message", m_messageOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_RelativeMapServicePort.registerConsumer("RelativeMapService", "MRFC::RelativeMapService", m_RelativeMapService);
  
  // Set CORBA Service Ports
  addPort(m_RelativeMapServicePort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("GOAL_AREA", m_GOAL_AREA, "0.200");
  bindParameter("LOOP_TIME", m_LOOP_TIME, "0.100");
  bindParameter("ROBOT_ACCELERATION", m_ROBOT_ACCELERATION, "0.300");
  bindParameter("ROBOT_MAX_SPEED", m_ROBOT_MAX_SPEED, "0.300");
  bindParameter("SETPOINT_1", m_setpoint1, "800");
  bindParameter("SETPOINT_2", m_setpoint2, "1500");
  bindParameter("CHECKPOINT_1", m_check1, "1000");
  bindParameter("CHECKPOINT_2", m_check2, "1500");
  bindParameter("CHECKPOINT_3", m_check3, "1000");
  bindParameter("CHECKPOINT_4", m_check4, "1000");

  
  
  // </rtc-template>
FILE *fp;
	char buf[BUF] = {0};
	fopen_s(&fp, "rtc.conf", "r");
	if(fp == NULL){
		printf("rtc.conf:Failed to open the file.\n");
		exit();
	}
	while(fscanf(fp, "%s", &buf) != EOF){
		if(strcmp("exec_cxt.periodic.rate:", buf) == 0){
			fscanf(fp, "%s\n", &buf);
			exec_Hz = atof(buf);
		}
	}
	fclose(fp);
	printf("Run FrequencyF%.2lf\n", exec_Hz);

	
	timeBeginPeriod( 1 );//Set to 1ms accuracy


	data_people = NULL;
	data_people = (RECEIVE_OBST_DATA *)calloc(MAX_TRACE_PERSON_NUM,sizeof(RECEIVE_OBST_DATA));
	consumption_memory_size += (int)malloc_usable_size(data_people);


	LocalMap = (unsigned char **)calloc(G_SELL_SIZE/GRID_SIZE, sizeof(unsigned char *));
	consumption_memory_size += (int)malloc_usable_size(LocalMap);
	for(int i=0;i<G_SELL_SIZE/GRID_SIZE;i++){
		LocalMap[i] = (unsigned char *)calloc(G_SELL_SIZE/GRID_SIZE, sizeof(unsigned char));
		consumption_memory_size += (int)malloc_usable_size(LocalMap[i]);
	}

	
	
  return RTC::RTC_OK;
}


RTC::ReturnCode_t PathPlannerV2::onFinalize()
{
printf("PathPlanner: onFinalizedd\n");

	if(data_people != NULL){
		free(data_people);
	}

	for(int i=0;i<G_SELL_SIZE/GRID_SIZE;i++){
		if(LocalMap[i] != NULL){
			free(LocalMap[i]);
			LocalMap[i] = NULL;
		}
	}
	if(LocalMap != NULL){
		free(LocalMap);
		LocalMap = NULL;
	}

	
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t PathPlannerV2::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PathPlannerV2::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

extern short GetMobjeNum_pre;
RTC::ReturnCode_t PathPlannerV2::onActivated(RTC::UniqueId ec_id)
{
	printf("PathPlanner: onActivated\n");
	activate_count++;//Count the number of activated

	//Configuration global variable declaration
	tread = 0.5388*1000;//wheel(mm)
	loopTime = m_LOOP_TIME;//loop time per step(sec)
	timeOut_s = 0.05*1000;//time out of creating tree(msec)
	goalAreaRadius = int(m_GOAL_AREA*1000);//radius of goal(mm)
	moveObjeRadius = int(0.200*1000);//radius of moving object(mm)
	robot_acc = m_ROBOT_ACCELERATION*1000;//robot acc(mm/sec^2)
	robot_dec = m_ROBOT_ACCELERATION*1000;//robot dcc(mm/sec^2)
	robotRadius = int(0.310*1000);
	//recycle_flag = m_RECYCLE_FLAG;
	motion_file_name = "MotionSet.mtn";
	img_transfer = 1;
	sound_set = 0;
	use_waypoint_port = 1;
	use_localization_port = 0;
	use_people_tracking_port = 0;
	robot_to_target_distance_warning_threshold = float(5 * 1000);
	use_waypoint_heading=0;
	get_video=0;
	use_map_service=1;
	use_people_service=0;
	robot_max=m_ROBOT_MAX_SPEED*1000;


#ifdef __SOUND
	if(sound_set == 1){//Sound effects
		sprintf(SOUND_ACTIVATE, "SOUND/Effect/Activate.wav");
		sprintf(SOUND_COLLISION, "SOUND/Effect/Collision.wav");
		sprintf(SOUND_TIMEOUT, "SOUND/Effect/time_over.wav");
		sprintf(SOUND_DEACTIVATE, "SOUND/Effect/Deactivate.wav");
		sprintf(SOUND_DTC_CONNECT_ERROR, "SOUND/Effect/connect_error.wav"); 
		sprintf(SOUND_LMC_CONNECT_ERROR, "SOUND/Effect/connect_error.wav"); 
		sprintf(SOUND_ARRIVE, "SOUND/Effect/Arrive.wav");
		sprintf(SOUND_CALC_ERROR, "SOUND/Effect/calc_error.wav"); 
		sprintf(SOUND_GOAL_DETECT_ERROR, "SOUND/Effect/goal_detect_error.wav");//Sound telling  missed the target person
		sprintf(SOUND_GOAL_FIRST_DETECT, "SOUND/Effect/target_first_detection.wav");//Sound informing you that the target person is found 
		sprintf(SOUND_TARGET_FAR_WARNING, "SOUND/Effect/test.wav");
	}else if(sound_set == 2){//Artificial voice
		sprintf(SOUND_ACTIVATE, "SOUND/VOICE/activate091111.wav");
		sprintf(SOUND_COLLISION, "SOUND/VOICE/object_collision_error091108.wav");
		sprintf(SOUND_TIMEOUT, "SOUND/VOICE/time_over_error091111.wav");
		sprintf(SOUND_DEACTIVATE, "SOUND/VOICE/deactivate091108.wav");
		sprintf(SOUND_DTC_CONNECT_ERROR, "SOUND/VOICE/dynamic_object_tracking_component_connection_error091112.wav");
		sprintf(SOUND_LMC_CONNECT_ERROR, "SOUND/VOICE/local_map_comp_connection_error091112.wav"); 
		sprintf(SOUND_ARRIVE, "SOUND/VOICE/goal_arrive091110.wav"); 
		sprintf(SOUND_CALC_ERROR, "SOUND/VOICE/calc_error091108.wav"); 
		sprintf(SOUND_GOAL_DETECT_ERROR, "SOUND/VOICE/target_detection_error091110.wav");
		sprintf(SOUND_GOAL_FIRST_DETECT, "SOUND/VOICE/target_first_detection.wav");
		sprintf(SOUND_TARGET_FAR_WARNING, "SOUND/VOICE/distance_far_warning091112.wav");
		sprintf(SOUND_CONNECT_ENON, "SOUND/VOICE/connect_enon091112.wav");
		sprintf(SOUND_CONNECT_PeopleBot, "SOUND/VOICE/connect_peoplebot091113.wav");
		sprintf(SOUND_CONNECT_UnknownROBOT, "SOUND/VOICE/connect_UnknownROBOT091113.wav");
	}
#endif

#ifndef __SOUND	
loopTimer=timeGetTime();
while(((double)(timeGetTime()-loopTimer)/1000.0) < 2){
			Sleep(1);
		}
loopTimer=0;
printf("waiting completed.\n");
#endif

	//sound_set=0;
#ifdef __SOUND

	if(sound_set != 0) PlaySound(TEXT(SOUND_ACTIVATE), NULL, SND_SYNC | SND_FILENAME);//Start-up Sound

	if(sound_set == 2){//If artificial speech
		int machine_kind = -1;
		if (m_TREAD == ENON_TREAD){
			machine_kind = CONNECT_ENON;
			PlaySound(TEXT(SOUND_CONNECT_ENON), NULL, SND_SYNC | SND_FILENAME);
			printf("Connected to ENON.\n");
		}else if (m_TREAD == PeopleBot_TREAD){
			machine_kind = CONNECT_PeopleBot;
			PlaySound(TEXT(SOUND_CONNECT_PeopleBot), NULL, SND_SYNC | SND_FILENAME);
			printf("Connected to PeopleBot.\n");
		}else if (m_TREAD == PatraFour_TREAD){
			machine_kind = CONNECT_PatraFour;
			PlaySound(TEXT(SOUND_CONNECT_UnknownROBOT), NULL, SND_SYNC | SND_FILENAME);
			printf("Connected to PatraFour.\n");
		}else{
			PlaySound(TEXT(SOUND_CONNECT_UnknownROBOT), NULL, SND_SYNC | SND_FILENAME);
			printf("UnKnown robot\n");
		}
	}
	if(sound_set == 0){
		loopTimer=timeGetTime();
		while(((double)(timeGetTime()-loopTimer)/1000.0) < 2){
			Sleep(1);
		}
		loopTimer=0;
		printf("waiting completed.\n");
	}
#endif

	motion.destroy();
	//motion.read(motion_file_name);
	motion1.read(motion_file_name);
	make_motion(&motion,&motion1);
	//make_motion2(&motion);

#ifdef ___DEBUG_FILE___
	sprintf(debug_file_name, "debug_log%04d.txt", activate_count);
	fopen_s(&debug_file, debug_file_name, "w");
	if(debug_file == NULL){
		printf("Unable to save debugging information. PathPlanning will be cancelled\n");
		getchar();
		return RTC::RTC_ERROR;
	}
#endif

	robot.~M_OBJE();//Destructor call now
	robot.construct(RR*2/GRID_SIZE,RR*2/GRID_SIZE);
	robot.SetCircle(RR/GRID_SIZE, RFLAG);//Robotic imaging
	robot.SetOptionalPolygon();

	/***** Initialize variables *****/
	//Initialization string buffer
	for(int i=0;i<BUF;i++){
		window_title_simu[i] = window_title_Local[i] = 0;
	}
	/*** Draw for the relationship LocalMap Simulator ***/
	simulator=NULL;
	Draw_init(window_title_simu, &simulator, cvPoint(G_SELL_SIZE / GRID_SIZE, G_SELL_SIZE / GRID_SIZE), CHECK_RD, "Map simulator", 1);

	/*** Local Map drawn between ***/
	LocalImg=NULL;
	Draw_init(window_title_Local, &LocalImg, cvPoint(G_SELL_SIZE / DRAW_RESOLUTION, G_SELL_SIZE / DRAW_RESOLUTION), CHECK_LM, "Local Map");


	window_output = cvCreateImage(cvSize(G_SELL_SIZE / DRAW_RESOLUTION, G_SELL_SIZE / DRAW_RESOLUTION), IPL_DEPTH_8U, 3);
	consumption_memory_size += window_output->nSize + window_output->imageSize;
	cvZero(window_output);

	/***** Test output *****/	
	Gauss_f32 = cvCreateImage (cvSize (G_SELL_SIZE/GRID_SIZE, G_SELL_SIZE/GRID_SIZE), IPL_DEPTH_32F, 1);
	consumption_memory_size += Gauss_f32->nSize + Gauss_f32->imageSize;
	Gauss_u8 = cvCreateImage(cvSize(G_SELL_SIZE / GRID_SIZE, G_SELL_SIZE / GRID_SIZE), IPL_DEPTH_8U, 1);

	dt_32 = cvCreateImage (cvSize (G_SELL_SIZE/GRID_SIZE, G_SELL_SIZE/GRID_SIZE), IPL_DEPTH_32F, 1);



	for(int i=0;i<G_SELL_SIZE/GRID_SIZE;i++){
		for(int j=0;j<G_SELL_SIZE/GRID_SIZE;j++){
			LocalMap[i][j] = 0;
		}
	}

	total3 = 0;
	max3 = 0;
	draw_time = 0;
	step = 0;
	path_depth = 0;
	NumOfMobje = 0;
	GetMobjeNum_pre = 0;
	m_counter=0;
	waypoint_trace_complete_num = 0;
	waypoint_data_length = 0;
	waypoint_pointer = 0;
	waypoint_state_change=0;

	collision_state=0;

	xs[1] = XS; xs[2] = YS;//xs[3] = D2R*TS;
	//xg[1] = XE; xg[2] = YE;// 
	//xg[3] = 0;
	

	//xs[3]  = (float)direction_norm((double)TS);//Normalized angle
	//xs[3] *= (float)D2R;
	//_cos_rad = cos(-xs[3]);
	//_sin_rad = sin(-xs[3]);
	xs[3] = 0;
	_cos_rad = cos(xs[3]);
	_sin_rad = sin(xs[3]);

	/***** Path Initialization *****/
	InitState(path, MAXNODE + MAXSTEP);
	InitState(tree, MAXNODE);

	/***Given initial position***/
	//You are here
	path[0].pose.x = xs[1];
	path[0].pose.y = xs[2];
	path[0].direction = xs[3];
	//Predictions the next time
	path[1].pose.x = xs[1];
	path[1].pose.y = xs[2];
	path[1].direction = xs[3];


	/***** Robot Information *****/

	robot.SetPosition(xs[1], xs[2]);//Robot location update
	robot.direction = xs[3];
	robot.velocity = 0.f;
	robot.omega = 0.f;

	if(move_obje != NULL){
		move_obje->~M_OBJE();
		move_obje=NULL;
	}

	for(int i=0;i<OBJECT_MAX_NUM;i++){
		if(array_Mobje[i] != NULL){
			free(array_Mobje[i]);
		}
		array_Mobje[i] = NULL;
	}

	pre_arrive_flag = false;
	pre_waypoint_end_flag = false;

/*	if( CORBA::is_nil(m_PeopleTrackingService.getObject()) == true){
	//if( CORBA::is_nil(m_PeopleTrackingService.getObject()) || m_PeopleTrackingService.getObject()->_non_existent()){
		printf("Tracking service is not connected\n");
		if(sound_set != 0) PlaySound(TEXT(SOUND_DTC_CONNECT_ERROR), NULL, SND_SYNC | SND_FILENAME);//Moving object detection and tracking noise component is not connected
	}
	else printf("Tracking service IS CONNECTED\n");*/

	/*if( CORBA::is_nil(m_MapService.getObject()) == true){
	//if( CORBA::is_nil(m_LocalMapService.getObject()) || m_LocalMapService.getObject()->_non_existent()){
		printf("Local map service is not connected\n");
		if(sound_set != 0) PlaySound(TEXT(SOUND_LMC_CONNECT_ERROR), NULL, SND_SYNC | SND_FILENAME);//Unconnected sound LocalMap Component
	}
	else printf("Local Map service IS CONNECTED\n");*/



	
	/*****start of simulation*****/
	calc_Draw_ave = 0;
	calc_Draw_max = 0;
	simu_start = timeGetTime();

if(TAKE_VIDEO){
//writer = cvCreateVideoWriter("out.avi",-1,30,cvSize(200,200));
	writer = cvCreateVideoWriter("out_ori.avi",-1,30,cvSize(400,400));
//char* outFileName="img.bmp";
}

initial_waypoint=false;
after_collision=true;
first_start=true;

states=0;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t PathPlannerV2::onDeactivated(RTC::UniqueId ec_id)
{
printf("\n\n##########################################\n");
printf("PathPlanner: onDeactivated\n");
printf("##########################################\n");
	exit_robot();

if(TAKE_VIDEO){
cvReleaseVideoWriter(&writer);
}

	simu_end = timeGetTime();
	printf("Total time:%d msec\n" , simu_end-simu_start);
	if(step != 0){
		printf("Total computation timeF% dmsec  An average computation time of the loopF%d msec\n",total3 , total3/step);
	}
	printf("The maximum computation timeF%d msec\n",max3);

	if(skeleton_func_num != 0){
		printf("LSM generation time:%d msec\n", skeleton_time/skeleton_func_num);
		printf("levelset_time1 = %d, levelset_time2 = %d, levelset_time3 = %d\n", levelset_time1/skeleton_func_num, levelset_time2/skeleton_func_num, levelset_time3/skeleton_func_num);
	}

	if((LocalImg != NULL) && (&LocalImg != NULL)){
		consumption_memory_size -= (LocalImg->nSize + LocalImg->imageSize);
		cvReleaseImage(&LocalImg);
		LocalImg = NULL;
	}

	if((window_output != NULL) && (&window_output != NULL)){
		consumption_memory_size -= (window_output->nSize + window_output->imageSize);
		cvReleaseImage(&window_output);
		window_output = NULL;
	}

	if((Gauss_f32 != NULL) && (&Gauss_f32 != NULL)){
		consumption_memory_size -= (Gauss_f32->nSize + Gauss_f32->imageSize);
		cvReleaseImage(&Gauss_f32);
		Gauss_f32 = NULL;
	}

	if((dt_32 != NULL) && (&dt_32 != NULL)){
		consumption_memory_size -= (dt_32->nSize + dt_32->imageSize);
		cvReleaseImage(&dt_32);
		dt_32 = NULL;
	}

#ifdef ___DEBUG_FILE___
	if(debug_file != NULL){
		printf("fclose(debug_file)Start\n");
		fclose(debug_file);
	}else{
		printf("debug_file == NULL!!n");
	}
	printf("fclose(debug_file) Complete\n");
#endif

	printf("onDeactivate Complete\n");
#ifdef __SOUND
	if(sound_set != 0) PlaySound(TEXT(SOUND_DEACTIVATE), NULL, SND_SYNC | SND_FILENAME);//End sound reproduction
#endif

  return RTC::RTC_OK;
}


RTC::ReturnCode_t PathPlannerV2::onExecute(RTC::UniqueId ec_id)
{
	//########### Init time #############//

	warning_flag = 0;
	if(step != 0){
		while(((double)(timeGetTime()-loopTimer)/1000.0) < LoopTime){
			Sleep(1);
		}
		printf("--------------------\nonExecute Start \tstep=%d\ call period t :%.3lf[sec]\n--------------------\n", step, (double)(timeGetTime()-loopTimer)/1000.0);
#ifdef ___DEBUG_FILE___
		fprintf_s(debug_file, "--------------------\nonExecute start \tstep=%d\call period t :%.3lf[sec]\n--------------------\n", step, (double)(timeGetTime()-loopTimer)/1000.0);
#endif
		loopTimer = timeGetTime();
	}else{
		loopTimer = timeGetTime();
		printf("--------------------\nonExecute start\tstep=0\n--------------------\n");
#ifdef ___DEBUG_FILE___
		fprintf_s(debug_file, "--------------------\nonExecute start \tstep=0\n--------------------\n");
#endif
	}

//############# Send Out next control #############//

if(keep_velocity==0)send_next_control();

//########## Tes Connection of Local Map Services ############//

if(use_map_service==1){
//if( CORBA::is_nil(m_MapService.getObject()) || m_MapService.getObject()->_non_existent()){
	if(!check_LM_connection()){
//if( CORBA::is_nil(m_MapService.getObject())){

	warning_flag |= LMC_CONNECTION_ERROR;
	printf("Local map service is not connected\n");
	//Draw_LMap_Data(LocalImg, path, tree, xs, xg_local_sub, n, path_depth, step, window_title_Local, robot_data, NULL, &robot_state_tmp, &warning_flag);
	drawing_when_error();
#ifdef __SOUND
		if(sound_set != 0){
			//PlaySound(NULL, NULL, SND_PURGE);//
			PlaySound(TEXT(SOUND_LMC_CONNECT_ERROR), NULL, SND_NOSTOP | SND_ASYNC | SND_FILENAME);//Detect sound reproduction over time
		}
#endif
#ifdef ___DEBUG_FILE___
	fprintf_s(debug_file, "--------------------\nonExecuteEnd(line %d): Local map service is not connected\n--------------------\n", __LINE__);
#endif
	return RTC::RTC_OK;
}
}

//################ READ DATA OF LOCAL MAP from LocalMapServices ###################//


#ifdef ___DEBUG_FILE___
	fprintf_s(debug_file, "Start getting a local map LINE: %d\n", __LINE__);
#endif
	time_tmp = timeGetTime();
	/*Mapping Simulation*/
	//RTC::TimedFloatLocalMapData* map_data;
	MRFC::TimedFloatRelativeOGMapData *map_data;
	//map_data = m_MapService->getFloatLocalMap();
	map_data=m_RelativeMapService->getFloatRelativeOGMap();
	//printf("Get a local map state time: %d[msec]\n", timeGetTime()-time_tmp);
#ifdef ___DEBUG_FILE___
	fprintf_s(debug_file, "Get local map completed LINE: %d, time: %d[msec]\n", __LINE__, timeGetTime()-time_tmp);
#endif
	for(int y=0;y<map_data->mapconfig.height;y++){
		for(int x=0;x<map_data->mapconfig.width;x++){
			SetIgray(simulator,y,x,(unsigned char)(255*map_data->cells[map_data->mapconfig.width*((map_data->mapconfig.height-1)-y)+(map_data->mapconfig.width-x)-1]));
		}
	}

//################## MAKE LOCAL MAP FOR STATIC OBJECT AND UNKNOWN AREA OF LOCAL MAP ##############//

	Make_LocalMap(LocalMap , simulator);

//############## Retrieving Data Robot #############//

/********** Get the current state of robots *********/
	get_robot_state();

/********* Get Data velocity Robot *********/
	get_robot_velocity();

//################ Setting Current Location of Robot ##############//
//############ Predict next position of Robot and Store it ###############//
	predict_and_store_robot_pose();
	


//################## GET DATA PERSON WHO WILL BE TRACKED ################//
//######################## Then store it to XG ##########################//

	previous_trace_flag = 0;//Whether the person was lost last time (1: 0 was the last person to find: the last person did not) had lost count store

	
	if(states==0){
		if(m_in_messageIn.isNew() == false){
			m_robot_control.data.vx = 0;
			m_robot_control.data.va = 0;
			m_robot_controlOut.write(m_robot_control);
			keep_velocity=1;
			states=0;
			return RTC::RTC_OK;
		}
		else{
			m_in_messageIn.read();
			kata=m_in_message.data;
			if(kata=="ROBOT_START"){
				states=1;
				keep_velocity=0;
			}

		}
	}
	else if(states==1){//get exact position of line 1
			jarak=read_distance();
			if(jarak<=m_check1){
				path[step+2].velocity=0;
				path[step+2].radius=0;
				states=2;
			}
			else {
				//if(jarak>=2000.f)path[step+2].velocity = robot_max;else path[step+2].velocity = 150.f;
				path[step+2].velocity = robot_max;
				path[step+2].radius=0;
				states=1;
			}
	}
			
	else if(states==2){
			path[step+2].velocity=0;
			path[step+2].radius=0;
			states=3;
	}
	else if(states==3){
		rotation(90,4);
		if(states==3)return RTC::RTC_OK;
			
	}
	else if(states==4){
		jarak=read_distance();
			if(jarak<=1500){
				m_robot_control.data.vx = 0;
				m_robot_control.data.va = -(PI/180.f)*15;//rad
				m_robot_controlOut.write(m_robot_control);
				keep_velocity=1;
				states=4;
				return RTC::RTC_OK;
			}
			else {states=5;keep_velocity=0;}
	}
	
	else if(states==5){
			path[step+2].velocity=0;
			path[step+2].radius=0;
			states=6;
	}
	else if(states==6){
		jarak=read_distance();
		jarak_dinding=read_distance_wall_left();
			if(jarak<=m_check2){
				path[step+2].velocity=0;
				path[step+2].radius=0;
				states=7;
			}
			else {
				if(jarak_dinding>=m_setpoint1+100){path[step+2].velocity = robot_max;path[step+2].radius=-3000;}
				else if(jarak_dinding<=m_setpoint1-100){path[step+2].velocity = robot_max;path[step+2].radius=3000;}
				else{path[step+2].velocity = robot_max;path[step+2].radius=0;}
				states=6;
			}
			
	}
	
	else if(states==7){
			m_message.data="ROBOT_ARRIVED";
			m_messageOut.write();
			states=8;

	}
	else if(states==8){
		if(m_in_messageIn.isNew() == false){
			m_robot_control.data.vx = 0;
			m_robot_control.data.va = 0;
			m_robot_controlOut.write(m_robot_control);
			keep_velocity=1;
			states=8;
			return RTC::RTC_OK;
		}
		else{
			m_in_messageIn.read();
			kata=m_in_message.data;
			if(kata=="ROBOT_GO_BACK"){
				states=9;
				keep_velocity=0;
				count_geser=0;
			}

		}
	}
	//else if(states==8){
	//		path[step+2].velocity=0;
	//		path[step+2].radius=0;
	//		states=9;
	//}
	else if(states==9){
			//rotation(80,10);
			//if(states==9)return RTC::RTC_OK;
			m_robot_control.data.vx = 0;
			m_robot_control.data.va = (PI/180.f)*30;//rad
			m_robot_controlOut.write(m_robot_control);
			keep_velocity=1;
			states=10;
	}
	else if(states==10){
			keep_velocity=0;
			jarak=read_distance();
			if(jarak>=1000){
				path[step+2].velocity=0;
				path[step+2].radius=0;
				states=110;
			}
			else {
				//if(jarak>=2000.f)path[step+2].velocity = robot_max;else path[step+2].velocity = 150.f;
				path[step+2].velocity = -robot_max;
				path[step+2].radius=0;
				states=10;
			}
	}
	else if(states==110){
			rotation(0,120);
			if(states==110)return RTC::RTC_OK;
	}
	else if(states==120){
			keep_velocity=0;
			jarak=read_distance();
			if(jarak>=500){
				path[step+2].velocity=0;
				path[step+2].radius=0;
				states=12;
			}
			else {
				//if(jarak>=2000.f)path[step+2].velocity = robot_max;else path[step+2].velocity = 150.f;
				path[step+2].velocity = -robot_max;
				path[step+2].radius=0;
				states=120;
			}
	}
	else if(states==11){
			rotation(-90,12);
			if(states==11)return RTC::RTC_OK;
	}
	else if(states==12){
		jarak=read_distance();
			if(jarak<=1500){
				m_robot_control.data.vx = 0;
				m_robot_control.data.va = (PI/180.f)*15;//rad
				m_robot_controlOut.write(m_robot_control);
				keep_velocity=1;
				states=12;
				return RTC::RTC_OK;
			}
			else {states=13;keep_velocity=0;}
	}
	else if(states==13){
		jarak=read_distance();
		jarak_dinding=read_distance_wall_right();
			if(jarak<=m_check1){
				path[step+2].velocity=0;
				path[step+2].radius=0;
				states=14;
			}
			else {
				if(jarak_dinding>=m_check3+100){path[step+2].velocity = robot_max;path[step+2].radius=3000;}
				else if(jarak_dinding<=m_check3-100){path[step+2].velocity = robot_max;path[step+2].radius=-3000;}
				else{path[step+2].velocity = robot_max;path[step+2].radius=0;}
				states=13;
			}
			
	}
	else if(states==14){
			rotation(-180,15);
			if(states==14)return RTC::RTC_OK;
	}
	else if(states==15){
		jarak=read_distance();
			if(jarak<=1500){
				m_robot_control.data.vx = 0;
				m_robot_control.data.va = (PI/180.f)*15;//rad
				m_robot_controlOut.write(m_robot_control);
				keep_velocity=1;
				states=15;
				return RTC::RTC_OK;
			}
			else {states=16;keep_velocity=0;}
	}
	else if(states==16){
		jarak=read_distance();
		jarak_dinding=read_distance_wall_right();
			if(jarak<=m_check4){
				path[step+2].velocity=0;
				path[step+2].radius=0;
				states=17;
			}
			else {
				if(jarak_dinding>=m_setpoint2+100){path[step+2].velocity = robot_max;path[step+2].radius=3000;}
				else if(jarak_dinding<=m_setpoint2+100){path[step+2].velocity = robot_max;path[step+2].radius=-3000;}
				else{path[step+2].velocity = robot_max;path[step+2].radius=0;}
				states=16;
			}
			
	}
	else if(states==17){
			rotation(-90,18);
			if(states==17)return RTC::RTC_OK;
	}
	else if(states==18){
			rotation(0,19);
			if(states==18)return RTC::RTC_OK;
	}

	else if(states==19){
			m_message.data="ROBOT_FINISH";
			m_messageOut.write();
			states=20;
	}
	else if(states==20){
		jarak=read_distance();
			if(jarak<=1500){
				m_robot_control.data.vx = 0;
				m_robot_control.data.va = -(PI/180.f)*15;//rad
				m_robot_controlOut.write(m_robot_control);
				keep_velocity=1;
				states=20;
				return RTC::RTC_OK;
			}
			else {states=21;keep_velocity=0;}
	}

	

	else{
			path[step+2].velocity=0;
			path[step+2].radius=0;
			states=0;
	}



	

	//printf("Goal Position x=%.2lf  y=%.2lf\n",xg[1],xg[2]);


//################### WRITE GOAL TO LOCAL MAP ######################//
//############ and check if goal area near static object############//

	CvPoint xg_local;
	//xg_local = Convert_Global2Local(&robot , xg);
	Position xg_local_position;
	xg_local_position.x = xg[1];
	xg_local_position.y = xg[2];
	xg_local = Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,xg_local_position);

	if(xg_local.x != -1){//Is within the range
		for(int x=max(xg_local.x - MobjeR*2/GRID_SIZE , 0);x<min(xg_local.x + MobjeR*2/GRID_SIZE , G_SELL_SIZE/GRID_SIZE);x++){
			for(int y=max(xg_local.y - MobjeR*2/GRID_SIZE , 0);y<min(xg_local.y + MobjeR*2/GRID_SIZE , G_SELL_SIZE/GRID_SIZE);y++){
				if((LocalMap[x][y] & SFLAG) != 0){//If the object still exists
					LocalMap[x][y] &= ~SFLAG;//Clear flag stationary object
					LocalMap[x][y] |= UnKnown_FLAG;//Set as unknown areas.
				}
			}
		}
		
	}

//########## PREDICT NEXT POSITION OF ROBOT ON LOCAL MAP #################//

	path[step+1].local_pose = Convert_Global2Local(robot_data.now.pose,robot_data.now.direction, path[step+1].pose);//To calculate the position on a local map the next time
	if(
		(path[step+1].local_pose.x == -1) || 
		//((LocalMap[path[step+1].local_pose.x][path[step+1].local_pose.y] & (R_AvailableFLAG)) == 0) ||
		(
			/*4 neighborhood*/
			((LocalMap[path[step+1].local_pose.x+1][path[step+1].local_pose.y] & (UnKnown_FLAG | MFLAG | SFLAG)) != 0) &&
			((LocalMap[path[step+1].local_pose.x-1][path[step+1].local_pose.y] & (UnKnown_FLAG | MFLAG | SFLAG)) != 0) &&
			((LocalMap[path[step+1].local_pose.x][path[step+1].local_pose.y+1] & (UnKnown_FLAG | MFLAG | SFLAG)) != 0) &&
			((LocalMap[path[step+1].local_pose.x][path[step+1].local_pose.y-1] & (UnKnown_FLAG | MFLAG | SFLAG)) != 0) &&
			/*Themselves*/
			((LocalMap[path[step+1].local_pose.x][path[step+1].local_pose.y] & (UnKnown_FLAG | MFLAG | SFLAG)) != 0) 
		)
		){
 //Immediately suspended robot
			exit_robot();
			bool flag_e=emergency(LocalMap,Gauss_f32,path[step+1].local_pose);
			m_robot_control.data.vx = 0*0.001;
			if(flag_e==true)m_robot_control.data.va = 0;else m_robot_control.data.va = 0;
			m_robot_controlOut.write(m_robot_control);
			warning_flag |= COLLISION;
#ifdef __SOUND
			if(sound_set != 0) {
				//PlaySound(NULL, NULL, SND_PURGE);//
				PlaySound(TEXT(SOUND_COLLISION), NULL, SND_NOSTOP | SND_ASYNC | SND_FILENAME);//
			}
#endif
			collision_state=1;
			error_action(0,0);

			if(path[step+1].local_pose.x == -1){//Abnormal values were
				printf("path[step+1].local_pose.x == -1\n");//Signs show that conflict with existing static objects
			}else{
				printf("danger!!!:COLLISION\n");//Signs show that conflict with existing static objects
			}
			printf("continue\n");

			return RTC::RTC_OK;
	}
	LocalMap[path[step+1].local_pose.x][path[step+1].local_pose.y] &= ~(UnKnown_FLAG | MFLAG | SFLAG);
	LocalMap[path[step+1].local_pose.x][path[step+1].local_pose.y] |= (RFLAG | R_AvailableFLAG);//Robot and Available

	
//############# DRAW THE LOCAL MAP ###############//

	drawing_whole();
	
//############# CHECKING IF HIT SOMETHING IN ROBOT AREA  ################//
/*printf("dilation calc start\n");

	if(collision_state>5){
		m_robot_control.data.vx = -ABS(robot_acc)*0.001;
		m_robot_control.data.va = temp_rad;
		m_robot_controlOut.write(m_robot_control);

		if(collision_state>6){collision_state-=1;}else {collision_state=1;exit_robot();}
		keep_velocity=1;
		
		error_action(0,0);
		printf("danger!!!:BACK\n");//
		printf("continue\n");

		return RTC::RTC_OK;
	}
	
	warning_flag |= Dilation2(LocalMap , &path[step+1], array_Mobje , NumOfMobje);//Expanding circle
	if(warning_flag & COLLISION){//If you already hit
		after_collision=true;
		exit_robot();
		m_message.data="COLLISION";
		m_messageOut.write();
		//bool flag_e=emergency(LocalMap,Gauss_f32,path[step+1].local_pose);
		
		//if(flag_e==true)m_robot_control.data.va = (path[step+1].velocity)/(path[step+1].radius*(-1));else m_robot_control.data.va = path[step].radius*0.001;
		if(path[step].radius!=0){
			temp_vel= -path[step].velocity*0.001;
			temp_rad=(path[step].velocity)/(path[step].radius*(1));
		}
		m_robot_control.data.vx = temp_vel;
		m_robot_control.data.va = temp_rad;
		m_robot_controlOut.write(m_robot_control);
		
		//collision_state=1;
		if(collision_state==0)collision_state=6+m_BACK_STEP;else collision_state=1;
		error_action(0,0);
		keep_velocity=1;
		//exit_robot();
		
#ifdef __SOUND
		if(sound_set != 0) {
			PlaySound(NULL, NULL, SND_PURGE);//
			PlaySound(TEXT(SOUND_COLLISION), NULL, SND_NOSTOP | SND_ASYNC | SND_FILENAME);//
		}
#endif
		printf("danger!!!:COLLISION\n");//
		printf("continue\n");

		return RTC::RTC_OK;
	}
*/
/*//################### CALCULATE FREE SPACE IN LOCAL MAP (BLUE AREA) ###################//
printf("free space calc start\n");

CalcRobotAvailableField(LocalMap , path[step+1].local_pose.x , path[step+1].local_pose.y);

	for (int y = 0; y < LocalImg->height; y++) {
		for (int x = 0; x < LocalImg->width; x++) {
			if((LocalMap[(int)(x*((double)DRAW_RESOLUTION/(double)GRID_SIZE))][(int)(y*((double)DRAW_RESOLUTION/(double)GRID_SIZE))] & R_AvailableFLAG) != 0){//Unknown area
				LocalImg->imageData[LocalImg->widthStep * y + x * 3] |= (unsigned char)255;//B
				//LocalImg->imageData[LocalImg->widthStep * y + x * 3+1] = (unsigned char)180;//G
				//LocalImg->imageData[LocalImg->widthStep * y + x * 3+2] = (unsigned char)180;//R
			}
		}
	}*/

//################ CALCULATE GOAL IN THE LOCAL MAP (SUBGOAL IF OUT OF LOCAL MAP) ####################//
printf("subgoal calc start\n");
	/*Calculation of subgoals Drawing*/

	CvPoint local_subgoal;
	local_subgoal = Calc_SubGoal(LocalMap , &robot ,robot_data, xg);
//################ CALCULATE SUBGOAL IN GLOBAL MAP IF GOAL OUTSIDE LOCAL MAP ####################//

	Position global_subgoal;
	global_subgoal   = Convert_Local2Global(local_subgoal , robot_data.now.pose,robot_data.now.direction);

	xg_global_sub[1] = global_subgoal.x;
	xg_global_sub[2] = global_subgoal.y;
	xg_global_sub[3] = 0;

	xg_local_sub[1] = (float)local_subgoal.x;
	xg_local_sub[2] = (float)local_subgoal.y;
	xg_local_sub[3] = 0;

	
	goal.x=global_subgoal.x;
	goal.y=global_subgoal.y;

	//float dist=read_distance();
	//if(dist>=2000.f)path[step+2].velocity=500.f;else path[step+2].velocity=150.f;
	//path[step+2].radius=0.f;
	//keep_velocity=0;


//########################## DRAWING MOVING OBJECT IN LOCAL MAP #####################//


	CvPoint xg_local2;
	if(xg_local.x != -1){//Is within the range
		xg_local2.x = xg_local.x;
		xg_local2.y = xg_local.y;
		xg_local2.x = int(xg_local2.x * ((double)GRID_SIZE/(double)DRAW_RESOLUTION));
		xg_local2.y = int(xg_local2.y * ((double)GRID_SIZE/(double)DRAW_RESOLUTION));
		cvCircle(LocalImg , xg_local2 , 5 , RED , CV_FILLED);
	}

	//printf("success drawing \n");
	//Draw_LMap_Data(LocalImg, path, tree, xs, xg_local_sub, n, path_depth, step, window_title_Local, robot_data, NULL, &robot_state_tmp, &warning_flag);
	cvCircle(LocalImg , cvPoint(200,200) , 8 , ORANGE , CV_FILLED);
	cvCopy(LocalImg, window_output);

//############################################################################################//
//###################### SEND IMAGE TO IMAGE VIEWER COMPONENT ################################//

	IplImage2TimedImage(&m_planning_img,LocalImg);
	m_planning_imgOut.write();

if(TAKE_VIDEO){
//if(!cvSaveImage(outFileName,src)) printf("Could not save: %s\n",outFileName);
//cvNormalize(Gauss_f32, Gauss_u8, 0.0, 255.0, CV_MINMAX, NULL);
//cvWriteToAVI(writer,Gauss_u8);
cvWriteToAVI(writer,LocalImg);
cvWaitKey(10);
}

	
	/** Blacking area **/
	cvZero(LocalImg);


	end3 = timeGetTime();
	total3 += end3-loopTimer;
	if(max3 < end3-loopTimer){
		max3 = end3-loopTimer;
	}

#if FP_TEST
	saveFile.calc_data(n, &(path[step]));//Data storage
#endif
	//printf("save path START \n");
	//save_path_for_next_use(path,tree,step,path_depth);
	
	pathArrayControl();
	keep_velocity=0;

	memory_consumption();

	printf("calculation period for this step : %d [msec]\n",timeGetTime()-loopTimer);

		
#ifdef ___DEBUG_FILE___
	fprintf_s(debug_file, "--------------------\nonExecute end(line %d): calculation period %.3lf[sec]\n--------------------\n", __LINE__ , (double)(timeGetTime()-loopTimer)/1000.0);
#endif

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t PathPlannerV2::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PathPlannerV2::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PathPlannerV2::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PathPlannerV2::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t PathPlannerV2::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void PathPlannerV2Init(RTC::Manager* manager)
  {
    coil::Properties profile(pathplannerv2_spec);
    manager->registerFactory(profile,
                             RTC::Create<PathPlannerV2>,
                             RTC::Delete<PathPlannerV2>);
  }
  
};
