// ================================================================================== //
// Name of File				: draw.cpp
// Vendor					: AISL-TUT
// Version					: 2.0.3
// ================================================================================== //

#include "draw.h"

IMAGE_STORE::IMAGE_STORE(void){//Constructor
	start_pointer = NULL;
	end_pointer   = NULL;
	now_pointer   = NULL;
	image_num     = 0;
}

IMAGE_STORE::~IMAGE_STORE(void){//Destructor

	ClearList(start_pointer);
}
void IMAGE_LIST::ClearList(IMAGE_LIST *pointer){
	if(pointer != NULL){
		if(pointer->next != NULL){
			ClearList(pointer->next);//Turn off after its own
		}
		//Turn off your
		cvReleaseImage(&(pointer->save_image));
		free(pointer);
		pointer=NULL;
	}
}

void IMAGE_STORE::Reset(){
	this->ClearList(start_pointer);
	start_pointer = NULL;
	end_pointer   = NULL;
	now_pointer   = NULL;
	image_num     = 0;
}



void IMAGE_STORE::SaveImage(IplImage *image){
	if(start_pointer == NULL){//Were first
		start_pointer = end_pointer = now_pointer = (IMAGE_LIST *)calloc(1,sizeof(IMAGE_LIST));
		consumption_memory_size += (int)malloc_usable_size(start_pointer);
	}else{
		end_pointer->next = (IMAGE_LIST *)calloc(1 , sizeof(IMAGE_LIST));//List formation
		consumption_memory_size += (int)malloc_usable_size(end_pointer->next);
		end_pointer = end_pointer->next;
	}
	end_pointer->save_image = image;//Save pointer
}

void IMAGE_STORE::DiscribeImage(char *name){//Accumulated data released to the outside
	if(now_pointer != NULL){
		sprintf_s(file_name , sizeof(char)*BUF,"%s/step%04d.bmp", name , image_num);
		cvSaveImage(file_name , now_pointer->save_image);
		image_num++;
		now_pointer = now_pointer->next;
		DiscribeImage(name);
	}
}

class _ARROW{
public:
	int  height , width;
	unsigned char **img_data_original , **img_data_optional;
	int  SetOptionalPolygon(double direction);
	void Draw(IplImage *LocalImg , CvPoint local_position , float velocity);
	_ARROW();//Constructor
};

void _ARROW::Draw(IplImage *LocalImg , CvPoint local_position , float velocity){
	local_position.x = local_position.x - width/2;
	local_position.y = local_position.y - height/2;
	int x=0 , y=0;
	for(int i=max(0,local_position.y);i<min(local_position.y+height , G_SELL_SIZE/DRAW_RESOLUTION);i++){
		x=0;
		for(int j=max(0,local_position.x);j<min(local_position.x+width , G_SELL_SIZE/DRAW_RESOLUTION);j++){
			//if(optional_arrow->imageData[optional_arrow->widthStep * y + x * 3 + 2] != 0){
			if(img_data_optional[y][x] == 2){
				LocalImg->imageData[LocalImg->widthStep * i + j * 3]     = (unsigned char)255;//B
				LocalImg->imageData[LocalImg->widthStep * i + j * 3 + 1] = (unsigned char)255;// G
				LocalImg->imageData[LocalImg->widthStep * i + j * 3 + 2] = (unsigned char)255;// R
			}else if(img_data_optional[y][x] == 1){//Advanced Color
				if(velocity < 100){//Slow. Green.
					LocalImg->imageData[LocalImg->widthStep * i + j * 3]     = (unsigned char)0;//B
					LocalImg->imageData[LocalImg->widthStep * i + j * 3 + 1] = (unsigned char)255;// G
					LocalImg->imageData[LocalImg->widthStep * i + j * 3 + 2] = (unsigned char)0;// R
				}else if(velocity < 200){//Slower than the speed of robot. Yellow
					LocalImg->imageData[LocalImg->widthStep * i + j * 3]     = (unsigned char)0;//B
					LocalImg->imageData[LocalImg->widthStep * i + j * 3 + 1] = (unsigned char)212;// G
					LocalImg->imageData[LocalImg->widthStep * i + j * 3 + 2] = (unsigned char)255;// R
				}else{//Faster than the speed of the robot. Red.
					LocalImg->imageData[LocalImg->widthStep * i + j * 3]     = (unsigned char)0;//B
					LocalImg->imageData[LocalImg->widthStep * i + j * 3 + 1] = (unsigned char)0;// G
					LocalImg->imageData[LocalImg->widthStep * i + j * 3 + 2] = (unsigned char)255;// R
				}
			}
			x++;
		}
		y++;
	}
}

_ARROW::_ARROW(){
	height = (int)(MobjeR*2*1.5/DRAW_RESOLUTION);
	width = (int)(MobjeR*2*1.5/DRAW_RESOLUTION);
	CvPoint start = cvPoint(width/6 , height/6);
	CvPoint end = cvPoint(width*5/6 , height*5/6);
	CvPoint center = cvPoint(width/2 , height/2);
	img_data_original  = (unsigned char**)calloc(height , sizeof(unsigned char *));
	img_data_optional  = (unsigned char**)calloc(height , sizeof(unsigned char *));
	consumption_memory_size += (int)malloc_usable_size(img_data_original);
	consumption_memory_size += (int)malloc_usable_size(img_data_optional);
	for(int i=0;i<height;i++){
		img_data_original[i] = (unsigned char *)calloc(width , sizeof(unsigned char));
		img_data_optional[i] = (unsigned char *)calloc(width , sizeof(unsigned char));
		consumption_memory_size += (int)malloc_usable_size(img_data_optional[i]);
		consumption_memory_size += (int)malloc_usable_size(img_data_original[i]);
	}
	int i=0;
	for(int y=start.y;y<end.y;y++){
		for(int x=center.x-i;x<=center.x+i;x++){
			if(i*2 >= width)break;
			if(y<center.y-1){//Red
				img_data_original[y][x] = 1;//Advanced
			}
			else{//White
				img_data_original[y][x] = 2;//Moving object
			}
		}
		i++;
	}
}

int _ARROW::SetOptionalPolygon(double direction){
	static int i , j , x , y ;
	static double u , v;
	double _cos_rad = cos( -1.0 * direction ) , _sin_rad = sin( -1.0 * direction );

	if( img_data_original == NULL ){
		fprintf(stderr,"error _ARROW::SetOptionalPolygon : Input == NULL");
		return -1;
	}
	if( img_data_optional == NULL ){
		fprintf(stderr,"error _ARROW::SetOptionalPolygon : Output == NULL\n");
		return -1;
	}
	if( height <= 0 ){
		fprintf(stderr,"error _ARROW::SetOptionalPolygon : height=%d\n",height);
		return -1;
	}
	if( width <= 0 ){
		fprintf(stderr,"error _ARROW::SetOptionalPolygon : width=%d\n",width);
		return -1;
	}
	static double r_centralpoint_h , r_centralpoint_w;
	r_centralpoint_h = (double)height/2.0;
	r_centralpoint_w = (double)width/2.0;

	for(i=0;i<height;i++){
		y = i - (int)r_centralpoint_h;
		for(j=0;j<width;j++){
			x = j - (int)r_centralpoint_w;
			u = (double)x * _cos_rad  +  (double)y * _sin_rad + r_centralpoint_w + 0.5;//round
			v = (double)y * _cos_rad  -  (double)x * _sin_rad + r_centralpoint_h + 0.5;//round
			if ( (int)v >= 0 && (int)v < height && (int)u >= 0 && (int)u < width ){
				img_data_optional[i][j] = img_data_original[(int)v][(int)u];
			}
			else{
				img_data_optional[i][j] = 0;
			}
		}
	}
	return 0;
}
/***** See related *****/

/***** Prototype declaration *****/
inline void Draw_Start_Goal_Aria2(IplImage *mpimg , float *xs , float *xg , DATA_LOGGER robot_data);
inline void Draw_Start_Goal_Aria(IplImage *mpimg , float *xs , float *xg , DATA_LOGGER robot_data);
inline void Draw_Pass_Line2(IplImage *mpimg , TREE *path , DATA_LOGGER robot_data , int step);
inline void Draw_Tree2(IplImage *mpimg , TREE *tree , DATA_LOGGER robot_data , int n);
inline void draw_tree3(IplImage *mpimg,TREE *tree,int num_node,DATA_LOGGER robot_data);
inline void Draw_Selected_Path2(IplImage *mpimg , TREE *path , int step , int path_depth , DATA_LOGGER robot_data);
inline void draw_selected_path3(IplImage *mpimg,TREE *path,int path_depth,int step,DATA_LOGGER robot_data);
inline void Draw_string_debug(IplImage *mpimg, unsigned short *warning_flag, int line_num);

inline CvPoint direct_search(IplImage *gauss_f32,IplImage *dt_f32,CvPoint point);

int num_col;

/***** Drawing of a moving object *****/
void WriteMovingObject(IplImage *LocalImg , M_OBJE *array_Mobje[OBJECT_MAX_NUM] ,short NumOfMobje , DATA_LOGGER robot_data)
{
	static _ARROW arrow;
	static int i;
	static CvPoint local_position;
	for(i=0;i<NumOfMobje;i++){
		//local_position = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , array_Mobje[i]->CPosition);
		local_position=array_Mobje[i]->LocalCPosition;
		if(local_position.x != -1){//Were within the range
			local_position.x = (int)(local_position.x*GRID_SIZE/DRAW_RESOLUTION);
			local_position.y = (int)(local_position.y*GRID_SIZE/DRAW_RESOLUTION);
			if(abs(array_Mobje[i]->velocity) <= 3.0){//If you are close to zero velocity
				cvCircle(LocalImg , local_position , MobjeR/DRAW_RESOLUTION , GREEN , CV_FILLED);
			}else{
				//cvCircle(LocalImg , local_position , MobjeR/DRAW_RESOLUTION , RED , CV_FILLED);//Whether this is a place to display subtle
				arrow.SetOptionalPolygon(-array_Mobje[i]->direction+robot_data.now.direction);
				arrow.Draw(LocalImg , local_position , abs(array_Mobje[i]->velocity));
			}
		}
	}

	//Let me also draw a robot or incidentally.
	local_position = robot_data.next.local_position;
	local_position.x *= (int)((double)GRID_SIZE/(double)DRAW_RESOLUTION);
	local_position.y *= (int)((double)GRID_SIZE/(double)DRAW_RESOLUTION);
	cvCircle(LocalImg , local_position , MobjeR/DRAW_RESOLUTION , ORANGE , CV_FILLED);//Draw the next predicted point of the robot
}

CvPoint Convert_Global2LocalForSubSubGoal(R_OBJE *robot , float *xg)
{
	static CvPoint local;

	static int r_centralpoint_h , r_centralpoint_w;
	static double _cos_rad , _sin_rad;
	static float x,y;

	_cos_rad = cos(1.0*robot->direction + PI/2);
	_sin_rad = sin(1.0*robot->direction + PI/2);

	y = xg[2] - robot->CPosition.y;//PʁFmm
	x = xg[1] - robot->CPosition.x;

	local.x = (int)((double)x * _cos_rad  +  (double)y * _sin_rad + 0.5);
	local.y = (int)((double)y * _cos_rad  -  (double)x * _sin_rad + 0.5);

	local.x /= GRID_SIZE;
	local.y /= GRID_SIZE;

	local.x += G_SELL_SIZE/GRID_SIZE/2;
	local.y += G_SELL_SIZE/GRID_SIZE/2;

	//if( (local.x < 0) || (local.y < 0) || (local.x >= G_SELL_SIZE/GRID_SIZE) || (local.y >= G_SELL_SIZE/GRID_SIZE)){
	//	local.x = -1;
	//	local.y = -1;
	//}

	return local;
}

/***** Calculation of subgoals *****/
/*CvPoint Calc_SubGoal(unsigned char **LocalMap_tmp , R_OBJE *robot , float *xg)
{
	static int i,j;
	static long long distance , dst;//8byte
	static unsigned char flag;
	static Position global_position ;
	static CvPoint local_position;
	static double x , y;
	static CvPoint local_goal;

	local_goal = Convert_Global2Local(robot, xg);

	if(local_goal.x == -1){
		flag = 0;
	}
	else{
		if((LocalMap_tmp[local_goal.x][local_goal.y] & R_AvailableFLAG) != 0){//if Reachable
			flag = 1;
		}
		else{
			if((LocalMap_tmp[local_goal.x][local_goal.y] & UnKnown_FLAG) != 0){//If the unknown region
				flag = 0;
			}else{//Still, if the object region
				flag = 2;
			}
		}
	}

	if(flag == 2){//I was still wearing a goal if an obstacle, to explore possible areas to move the robot on a straight line from the goal
		for(i = 0;i<(G_SELL_SIZE/GRID_SIZE)/2;i++){
			if((LocalMap_tmp[int(local_goal.x+((G_SELL_SIZE/GRID_SIZE)/2-local_goal.x)*(i/double((G_SELL_SIZE/GRID_SIZE)/2)))][int(local_goal.y+((G_SELL_SIZE/GRID_SIZE)/2-local_goal.y)*(i/double((G_SELL_SIZE/GRID_SIZE)/2)))] & R_AvailableFLAG) != 0){//If the movable area
				local_goal.x = int(local_goal.x+((G_SELL_SIZE/GRID_SIZE)/2-local_goal.x)*(i/double((G_SELL_SIZE/GRID_SIZE)/2)));
				local_goal.y = int(local_goal.y+((G_SELL_SIZE/GRID_SIZE)/2-local_goal.y)*(i/double((G_SELL_SIZE/GRID_SIZE)/2)));
				return local_goal;
			}
		}
		printf("look weird. %d\n\a", __LINE__);
		flag = 0;//you cannot go there
	}
	
	if(flag == 0){//If the goal is to see
		distance = 0x7FFFFFFFFFFFFFFF;
		for(i=0;i<G_SELL_SIZE/GRID_SIZE;i++){
			for(j=0;j<G_SELL_SIZE/GRID_SIZE;j++){
				if((LocalMap_tmp[i][j] & R_AvailableFLAG) != 0){//If the movable area
					if( i==0 || i==G_SELL_SIZE/GRID_SIZE-1 || j==0 || j==G_SELL_SIZE/GRID_SIZE-1){//If the wall around
						//The point is a candidate.
						//Calculate the distance to the goal
						local_position.x = i;
						local_position.y = j;
						global_position = Convert_Local2Global(local_position , robot);
						x = xg[1] - global_position.x;
						y = xg[2] - global_position.y;
						x *= x;
						y *= y;
						dst = (long long)x+(long long)y;
						if( dst < distance ){//Kattara coordinates near the goal so far in most if
							distance = dst;
							local_goal.x = i;
							local_goal.y = j;
						}
					}
					else if(   ((LocalMap_tmp[i+1][j] & UnKnown_FLAG) !=0 )
						||     ((LocalMap_tmp[i-1][j] & UnKnown_FLAG) !=0 )
						||     ((LocalMap_tmp[i][j+1] & UnKnown_FLAG) !=0 )
						||     ((LocalMap_tmp[i][j-1] & UnKnown_FLAG) !=0 ) 
						||     (LocalMap_tmp[i+1][j]==0 )
						||     (LocalMap_tmp[i-1][j]==0 )
						||     (LocalMap_tmp[i][j+1]==0 )
						||     (LocalMap_tmp[i][j-1]==0 )){//If the surrounding area is unknown or zero
							//The point is a candidate.
							//Calculate the distance to the goal
							local_position.x = i;
							local_position.y = j;
							global_position = Convert_Local2Global(local_position , robot);
							x = xg[1] - global_position.x;
							y = xg[2] - global_position.y;
							x *= x;
							y *= y;
							dst = (long long)x+(long long)y;
							if( dst < distance ){//Kattara coordinates near the goal so far in most if
								distance = dst;
								local_goal.x = i;
								local_goal.y = j;
							}
					}
				}
			}
		}
	}
	return local_goal;
}*/


// New Subgoal calculation
CvPoint Calc_SubGoal(unsigned char **LocalMap_tmp , R_OBJE *robot ,DATA_LOGGER robot_data, float *xg)
{
	static int i,j;
	static long long distances , dst;//8byte
	static unsigned char flag;
	static Position global_position,pose; 
	static CvPoint local_position;
	static double x , y;
	static CvPoint local_goal;

	float xg2[4];//sub-subgoal

	//local_goal = Convert_Global2Local(robot, xg);
	pose.x=xg[1];
	pose.y=xg[2];
	local_goal = Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,pose);


	xg2[1] = xg[1];
	xg2[2] = xg[2];

	if(local_goal.x == -1){//If no goals in the region
		flag = 0;

		CvPoint tmp;
		double slope;
		tmp = Convert_Global2LocalForSubSubGoal(robot, xg);

		if(tmp.x < 0){//Set a goal to the left
			// y=ax+b 
			int x_inc, y_inc;
			x_inc = (G_SELL_SIZE/GRID_SIZE/2)-tmp.x;
			y_inc = tmp.y-(G_SELL_SIZE/GRID_SIZE/2);
			slope = (double)y_inc / (double)x_inc;

			tmp.y = (double)(G_SELL_SIZE/GRID_SIZE/2) + slope*(double)(G_SELL_SIZE/GRID_SIZE/2);
			tmp.x = 0;
		}else if(tmp.x >= G_SELL_SIZE/GRID_SIZE){//right
			int x_inc, y_inc;
			x_inc = (G_SELL_SIZE/GRID_SIZE/2)-tmp.x;
			y_inc = tmp.y-(G_SELL_SIZE/GRID_SIZE/2);
			slope = (double)y_inc / (double)x_inc;
			tmp.y = (double)(G_SELL_SIZE/GRID_SIZE/2) - slope * (double)(G_SELL_SIZE/GRID_SIZE/2);
			tmp.x = G_SELL_SIZE/GRID_SIZE-1;
		}if(tmp.y < 0){//up
			int x_inc, y_inc;
			x_inc = (G_SELL_SIZE/GRID_SIZE/2)-tmp.x;
			y_inc = tmp.y-(G_SELL_SIZE/GRID_SIZE/2);

			if(x_inc == 0){
				tmp.x = G_SELL_SIZE/GRID_SIZE/2;
			}else{
				slope = (double)y_inc / (double)x_inc;
				tmp.x = (double)(G_SELL_SIZE/GRID_SIZE/2) + (1.0/slope)*(double)(G_SELL_SIZE/GRID_SIZE/2);
			}
			tmp.y = 0;
		}else if(tmp.y >= G_SELL_SIZE/GRID_SIZE){//down
			int x_inc, y_inc;
			x_inc = (G_SELL_SIZE/GRID_SIZE/2)-tmp.x;
			y_inc = tmp.y-(G_SELL_SIZE/GRID_SIZE/2);

			if(x_inc == 0){
				tmp.x = G_SELL_SIZE/GRID_SIZE/2;
			}else{
				slope = (double)y_inc / (double)x_inc;
				tmp.x = (double)(G_SELL_SIZE/GRID_SIZE/2) - (1.0/slope)*(double)(G_SELL_SIZE/GRID_SIZE/2);
			}
			tmp.y = G_SELL_SIZE/GRID_SIZE-1;
		}
		local_position.x = tmp.x;
		local_position.y = tmp.y;
		global_position = Convert_Local2Global(local_position , robot_data.now.pose,robot_data.now.direction);
		xg2[1] = global_position.x;
		xg2[2] = global_position.y;



	}
	else{
		if((LocalMap_tmp[local_goal.x][local_goal.y] & R_AvailableFLAG) != 0){//reachable
			flag = 1;
		}
		else{
			if((LocalMap_tmp[local_goal.x][local_goal.y] & UnKnown_FLAG) != 0 || LocalMap_tmp[local_goal.x][local_goal.y]==0 || (LocalMap_tmp[local_goal.x][local_goal.y] & MFLAG) != 0){//unknown region
				flag = 0;
			}else{//
				flag = 2;
			}
		}
	}

	if(flag == 2){//
		for(i = 0;i<(G_SELL_SIZE/GRID_SIZE)/2;i++){
			if((LocalMap_tmp[int(local_goal.x+((G_SELL_SIZE/GRID_SIZE)/2-local_goal.x)*(i/double((G_SELL_SIZE/GRID_SIZE)/2)))][int(local_goal.y+((G_SELL_SIZE/GRID_SIZE)/2-local_goal.y)*(i/double((G_SELL_SIZE/GRID_SIZE)/2)))] & R_AvailableFLAG) != 0){//movable area
				local_goal.x = int(local_goal.x+((G_SELL_SIZE/GRID_SIZE)/2-local_goal.x)*(i/double((G_SELL_SIZE/GRID_SIZE)/2)));
				local_goal.y = int(local_goal.y+((G_SELL_SIZE/GRID_SIZE)/2-local_goal.y)*(i/double((G_SELL_SIZE/GRID_SIZE)/2)));
				return local_goal;
			}
		}
		printf("Something wrong at LINE %d\n\a", __LINE__);
		flag = 0;//
	}
	
	if(flag == 0){//
		distances = 0x7FFFFFFFFFFFFFFF;
		for(i=0;i<G_SELL_SIZE/GRID_SIZE;i++){
			for(j=0;j<G_SELL_SIZE/GRID_SIZE;j++){
				if((LocalMap_tmp[i][j] & R_AvailableFLAG) != 0){//
					if( i==0 || i==G_SELL_SIZE/GRID_SIZE-1 || j==0 || j==G_SELL_SIZE/GRID_SIZE-1){
						
						local_position.x = i;
						local_position.y = j;
						global_position = Convert_Local2Global(local_position , robot_data.now.pose,robot_data.now.direction);
						x = xg2[1] - global_position.x;
						y = xg2[2] - global_position.y;
						x *= x;
						y *= y;
						dst = (long long)x+(long long)y;
						if( dst < distances && dst > (long long)(robotRadius*robotRadius) ){//new
							distances = dst;
							local_goal.x = i;
							local_goal.y = j;
						}
					}
					else if(   ((LocalMap_tmp[i+1][j] & UnKnown_FLAG) !=0 )
						||     ((LocalMap_tmp[i-1][j] & UnKnown_FLAG) !=0 )
						||     ((LocalMap_tmp[i][j+1] & UnKnown_FLAG) !=0 )
						||     ((LocalMap_tmp[i][j-1] & UnKnown_FLAG) !=0 ) 
						||     (LocalMap_tmp[i+1][j]==0 )
						||     (LocalMap_tmp[i-1][j]==0 )
						||     (LocalMap_tmp[i][j+1]==0 )
						||     (LocalMap_tmp[i][j-1]==0 )){
							
							local_position.x = i;
							local_position.y = j;
							global_position = Convert_Local2Global(local_position , robot_data.now.pose,robot_data.now.direction);
							x = xg2[1] - global_position.x;
							y = xg2[2] - global_position.y;
							x *= x;
							y *= y;
							dst = (long long)x+(long long)y;
							if( dst < distances && dst > (long long)(robotRadius*robotRadius) ){
								distances = dst;
								local_goal.x = i;
								local_goal.y = j;
							}
					}
				}
			}
		}
	}
	return local_goal;
}


/***** Drawing Initialization *****/
void Draw_init(char *str, IplImage **img, CvPoint environment_size, int hNum, const char str2[], int channels)
{

	/** Partitioning **/
	*img = cvCreateImage(cvSize(environment_size.x, environment_size.y), IPL_DEPTH_8U, channels);
	consumption_memory_size += (*img)->nSize + (*img)->imageSize;
	/** The white areas **/
	//cvSet(*img, cvScalarAll(255), 0);
	cvSet(*img, cvScalarAll(0), 0);
}

void cvCreateImage2(IplImage **img, CvSize size, int depth, int channels){
	*img = cvCreateImage(size, depth, channels);
	consumption_memory_size += (*img)->nSize + (*img)->imageSize;
}

/***** Drawing ends *****/
void Draw_end(IplImage **img , char *str)
{
	cvDestroyWindow(str);
	cvReleaseImage(img);
}


void convert_RVtoVLVR(double r , double vel, double *vr, double *vl)//Specifies the radius of gyration
{
	double inside_r , outside_r;
	double inside_v , outside_v;
	//Seek inner radius and outer radius
	inside_r  = (double)(r-(Wheel_Tread/2));
	outside_r = (double)(r+(Wheel_Tread/2));
	//Calculating the speed of the inner ring and outer ring
	if(ABS(r) >= Wheel_Tread/2){
		inside_v  = vel * (inside_r  / (double)r);
		outside_v = vel * (outside_r / (double)r);
	}else if(r == 0){
		inside_v = vel;
		outside_v = vel;
	}
	else{inside_v=-vel;outside_v=vel;}
	//inside_v = left, outside_v = right;
	*vr = inside_v;
	*vl = outside_v;
}

/********** Draw String **********/
inline void Draw_string(IplImage *mpimg ,TREE *path , int step , ROBOT_STATE *robot_state)
{
	static CvFont font;
	cvInitFont (&font, CV_FONT_HERSHEY_SIMPLEX, 0.5, 0.5,0,1);
	static char moji[BUF];
	/* Status of the robot */
	//sprintf_s(moji , sizeof(char)*BUF , "receive:  robot:x=%.2lf,  y=%.2lf" , robot_state->x , robot_state->y );
	//cvPutText (mpimg,moji, cvPoint (15 ,15), &font, WHITE);
	//sprintf_s(moji , sizeof(char)*BUF , "assume: robot:x=%.2lf,  y=%.2lf" , path[step+1].pose.x , path[step+1].pose.y );
	//cvPutText (mpimg,moji, cvPoint (15 ,35), &font, WHITE);

	//sprintf_s(moji , sizeof(char)*BUF , "received:  robot:vl=%.2lf,  vr=%.2lf" , robot_state->vl , robot_state->vr );	
	//cvPutText (mpimg,moji, cvPoint (15 ,75), &font, WHITE);
	double vl, vr;
	convert_RVtoVLVR(path[step].radius, path[step].velocity, &vr, &vl);
	sprintf_s(moji , sizeof(char)*BUF , "sent: robot:vl=%.2lf,  vr=%.2lf" , vl , vr );	
	cvPutText (mpimg,moji, cvPoint (15 ,15), &font, WHITE);
}


/********** Draw String **********/
inline void Draw_string_debug(IplImage *mpimg, unsigned short *warning_flag, int line_num)
{

	static CvFont font;
	cvInitFont (&font, CV_FONT_HERSHEY_SIMPLEX, 0.5, 0.5,0,1);
	static char moji[BUF];
	CvPoint display_locate = cvPoint(15, mpimg->height-15);//Bottom left of the first letter
	if( (*warning_flag & COLLISION) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"DANGER!! COLLISION!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & TIME_OVER) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"ERROR!! TIME OVER!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & CANNOT_MAKE_NODE) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"ERROR!! CANNOT MAKE NODE!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & HUMAN_NOT_FIND) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"ERROR!! HUMAN IS NOT FIND!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & CALCULATION_ERROR) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"CALCULATION ERROR!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & GOAL_ARRIVE_FLAG) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"GOAL ACHIEVED");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}if( (*warning_flag & DOC_CONNECTION_ERROR) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"Dynamic object detect component is not connected!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & LMC_CONNECTION_ERROR) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"Local map component is not connected!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & WAYPOINT_DATA_LENGTH_IS_0) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"waypoint data length is 0!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & TURNING) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"TURNING!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & TURNING_STATE) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"TURNING_STATE!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}
	if( (*warning_flag & TURNING_WAYPOINT) != 0){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"TURNING_WAYPOINT!");
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
		display_locate.y -= 20;
	}

	/*if(line_num != -1){
		cvRectangle(mpimg, cvPoint(0,display_locate.y-17), cvPoint(mpimg->width-1, display_locate.y+3), BLACK, CV_FILLED, CV_AA);
		sprintf_s(moji, sizeof(char)*BUF ,"EXCEPTION_LOCATE:: line %d", line_num);
		cvPutText (mpimg, moji, display_locate, &font, WHITE);
	}*/


}



/********** Drawing debug statement **********/
void Draw_LMap_Data2(IplImage *mpimg, char *str , IMAGE_STORE *image_stack_LM, unsigned short *warning_flag, int line_num)
{

	/** Draw String **/
	Draw_string_debug(mpimg, warning_flag, line_num);

}

/********** Drawing **********/
void Draw_LMap_Data(IplImage *mpimg , TREE *path , TREE *tree , float *xs , float *xg , int n , int path_depth , int step , char *str , DATA_LOGGER robot_data , IMAGE_STORE *image_stack_LM, ROBOT_STATE *robot_state, unsigned short *warning_flag)
{
	/** The black **/
	//cvZero (mpimg);
	/** Drawing Objects **/
	//Draw_Object(mpimg,LocalMap);
	num_col=0;
	/** Initial position, the end point drawing **/
	Draw_Start_Goal_Aria2(mpimg , xs , xg , robot_data);

	/** Draw a path so far has been through **/
	Draw_Pass_Line2(mpimg , path , robot_data , step);

	/** Tree Drawing **/
	//Draw_Tree2(mpimg , tree, robot_data, n);
	draw_tree3(mpimg,tree,n,robot_data);

	/** Drawing on the selected path **/
	//Draw_Selected_Path2(mpimg , path , step , path_depth , robot_data);
	draw_selected_path3(mpimg,path,path_depth,step,robot_data);


	/** Display string **/
	Draw_string(mpimg, path, step, robot_state);
	if(*warning_flag != 0){
		Draw_string_debug(mpimg, warning_flag, -1);
	}


}

void Draw_LMap_Data_Err(IplImage *mpimg , TREE *path , TREE *tree , float *xs , float *xg , int n , int path_depth , int step , char *str , DATA_LOGGER robot_data , IMAGE_STORE *image_stack_LM, ROBOT_STATE *robot_state, unsigned short *warning_flag)
{
	/** The black **/
	//cvZero (mpimg);
	/** Drawing Objects **/
	//Draw_Object(mpimg,LocalMap);
	num_col=0;
	/** Initial position, the end point drawing **/
	Draw_Start_Goal_Aria(mpimg , xs , xg , robot_data);

	/** Draw a path so far has been through **/
	Draw_Pass_Line2(mpimg , path , robot_data , step);

	/** Tree Drawing **/
	//Draw_Tree2(mpimg , tree, robot_data, n);
	//draw_tree3(mpimg,tree,n,robot_data);

	/** Drawing on the selected path **/
	//Draw_Selected_Path2(mpimg , path , step , path_depth , robot_data);
	//draw_selected_path3(mpimg,path,path_depth,step,robot_data);


	/** Display string **/
	Draw_string(mpimg, path, step, robot_state);
	if(*warning_flag != 0){
		Draw_string_debug(mpimg, warning_flag, -1);
	}


}


/********** Starting point, drawing the target point **********/
inline void Draw_Start_Goal_Aria2(IplImage *mpimg , float *xs , float *xg , DATA_LOGGER robot_data)
{
	static CvPoint start_local;
	static Position start={xs[1] , xs[2]};//Do not change once you put
	//Coordinate transformation
	start_local = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , start);
	if(start_local.x != -1){//If you have a starting point if the current display area
		cvCircle (mpimg, cvPoint((int)(start_local.x*((float)GRID_SIZE/(float)DRAW_RESOLUTION)),(int)(start_local.y*((float)GRID_SIZE/(float)DRAW_RESOLUTION))), 3 , YELLOW , 1);//Starting point, red
	}

	cvCircle (mpimg, cvPoint((int)(xg[1]*((float)GRID_SIZE/(float)DRAW_RESOLUTION)),(int)(xg[2]*((float)GRID_SIZE/(float)DRAW_RESOLUTION))) , 3 , RED , 1);//The finish line, blue
}

inline void Draw_Start_Goal_Aria(IplImage *mpimg , float *xs , float *xg , DATA_LOGGER robot_data)
{
	static CvPoint start_local;
	static Position start={xs[1] , xs[2]};//Do not change once you put
	//Coordinate transformation
	start_local = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , start);
	if(start_local.x != -1){//If you have a starting point if the current display area
		cvCircle (mpimg, cvPoint((int)(start_local.x*((float)GRID_SIZE/(float)DRAW_RESOLUTION)),(int)(start_local.y*((float)GRID_SIZE/(float)DRAW_RESOLUTION))), 3 , YELLOW , 1);//Starting point, red
	}

	//cvCircle (mpimg, cvPoint((int)(xg[1]*((float)GRID_SIZE/(float)DRAW_RESOLUTION)),(int)(xg[2]*((float)GRID_SIZE/(float)DRAW_RESOLUTION))) , 3 , RED , 1);//The finish line, blue
}

/********** Show lines passing through **********/
inline void Draw_Pass_Line2(IplImage *mpimg , TREE *path , DATA_LOGGER robot_data , int step)
{
	static Position node[MAXSTEP];
	static int i;
	static CvPoint local1 , local2;
	//node[step] = robot_data.now.pose;//Zero-step

	for(i = 1; i < step+1; i++) {
		//local1 = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , node[i]);
		//local2 = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , node[i-1]);
		local1 = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , path[i].pose);
		local2 = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , path[i-1].pose);
		if((local1.x != -1) && (local2.x != -1)){//Were in range
			local1.x = (int)(local1.x * (float)GRID_SIZE/(float)DRAW_RESOLUTION);
			local1.y = (int)(local1.y * (float)GRID_SIZE/(float)DRAW_RESOLUTION);
			local2.x = (int)(local2.x * (float)GRID_SIZE/(float)DRAW_RESOLUTION);
			local2.y = (int)(local2.y * (float)GRID_SIZE/(float)DRAW_RESOLUTION);
			cvLine(mpimg, local2, local1, ORANGE , 1, 8, 0);//orange
		}
	}
}

/**********Tree Drawing**********/
inline void Draw_Tree2(IplImage *mpimg , TREE *tree , DATA_LOGGER robot_data , int n)
{
	static int j,k;
	static CvPoint local1 , local2;

	for(j = n-1; j > 0; j--){//n is the number of nodes.
		if(tree[j].density) {
			k = j;
			local2 = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , tree[j].pose);
			do {
				tree[k].density = 0;//Initialize the density of the tree. Used to avoid having to override.
				local1 = local2;
				k = tree[k].parent;
				local2 = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , tree[k].pose);
				if((local1.x != -1) && (local2.x != -1)){//You can view the display when
					cvLine(mpimg, cvPoint((int)(local1.x*((float)GRID_SIZE/(float)DRAW_RESOLUTION)) , (int)(local1.y*((float)GRID_SIZE/(float)DRAW_RESOLUTION))), cvPoint((int)(local2.x*((float)GRID_SIZE/(float)DRAW_RESOLUTION)) , (int)(local2.y*((float)GRID_SIZE/(float)DRAW_RESOLUTION))) , YELLOW , 1, 8, 0);//yellow
				}
			} while(k > 0);
		}
	}
}

inline void draw_tree3(IplImage *mpimg,TREE *tree,int num_node,DATA_LOGGER robot_data)
{
CvPoint dota1,dota2;
	for(int i=num_node;i>=0;i--){
	dota1=Convert_Global2Local(robot_data.now.pose , robot_data.now.direction ,tree[i].pose);
	dota2=Convert_Global2Local(robot_data.now.pose , robot_data.now.direction ,tree[(tree[i].parent)].pose);
	if(i>(int)(2*num_node/3))cvLine(mpimg,cvPoint((dota1.x*(GRID_SIZE/DRAW_RESOLUTION)),(dota1.y*(GRID_SIZE/DRAW_RESOLUTION))),cvPoint((dota2.x*(GRID_SIZE/DRAW_RESOLUTION)),(dota2.y*(GRID_SIZE/DRAW_RESOLUTION))),YELLOW,1,8,0);
	else if(i<(int)(num_node/3))cvLine(mpimg,cvPoint((dota1.x*(GRID_SIZE/DRAW_RESOLUTION)),(dota1.y*(GRID_SIZE/DRAW_RESOLUTION))),cvPoint((dota2.x*(GRID_SIZE/DRAW_RESOLUTION)),(dota2.y*(GRID_SIZE/DRAW_RESOLUTION))),YELLOW,1,8,0);
	else cvLine(mpimg,cvPoint((dota1.x*(GRID_SIZE/DRAW_RESOLUTION)),(dota1.y*(GRID_SIZE/DRAW_RESOLUTION))),cvPoint((dota2.x*(GRID_SIZE/DRAW_RESOLUTION)),(dota2.y*(GRID_SIZE/DRAW_RESOLUTION))),YELLOW,1,8,0);
	}

}


/********** Drawing on the selected path **********/
inline void Draw_Selected_Path2(IplImage *mpimg , TREE *path , int step , int path_depth , DATA_LOGGER robot_data)
{
	static CvPoint pt1 , pt2;
	static int i;
	//pt2.x = (int)(gf_data.x2*((float)GRID_SIZE/(float)DRAW_RESOLUTION));//You are here
	//pt2.y = (int)(gf_data.y2*((float)GRID_SIZE/(float)DRAW_RESOLUTION));

	//pt2.x = (int)((G_SELL_SIZE/2)/(float)DRAW_RESOLUTION);//You are here
	//pt2.y = (int)((G_SELL_SIZE/2)/(float)DRAW_RESOLUTION);

	pt2 = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , robot_data.next.pose);//You are here
	pt2.x *= (int)((float)GRID_SIZE/(float)DRAW_RESOLUTION);
	pt2.y *= (int)((float)GRID_SIZE/(float)DRAW_RESOLUTION);

	for(i = step+1; i < path_depth; i++) {

		pt1 = Convert_Global2Local(robot_data.now.pose , robot_data.now.direction , path[i].pose);
		pt1.x *= (int)((float)GRID_SIZE/(float)DRAW_RESOLUTION);
		pt1.y *= (int)((float)GRID_SIZE/(float)DRAW_RESOLUTION);
		
		if(	(pt1.x >= 0) && (pt2.x >= 0) ){//Were in range
			cvLine(mpimg, pt2, pt1, RED, 1, 8, 0);//Brownish yellow//CV_RGB(128,255,128)
		}
		pt2 = pt1;
	}
}

inline void draw_selected_path3(IplImage *mpimg,TREE *path,int path_depth,int step,DATA_LOGGER robot_data)
{
static CvPoint dota1,dota2;

dota1=Convert_Global2Local(robot_data.now.pose , robot_data.now.direction,robot_data.next.pose);
for(int i=1;i<path_depth;i++){
	dota2=Convert_Global2Local(robot_data.now.pose , robot_data.now.direction,path[step+i].pose);
	//dota2=path[step+i].local_pose;
	cvLine(mpimg,cvPoint((dota1.x*(GRID_SIZE/DRAW_RESOLUTION)),(dota1.y*(GRID_SIZE/DRAW_RESOLUTION))),cvPoint((dota2.x*(GRID_SIZE/DRAW_RESOLUTION)),(dota2.y*(GRID_SIZE/DRAW_RESOLUTION))),RED,1,8,0);
	dota1=dota2;
}


}

void dilation3(unsigned char **LocalMap,M_OBJE *array_Mobje[OBJECT_MAX_NUM],short NumOfMobje,DATA_LOGGER robot_data,R_OBJE *robot ,float *xg)
{
static CvPoint local_obje;
static CvPoint local_goal;
static int s,t;
static unsigned char LocalMap2[G_SELL_SIZE/GRID_SIZE][G_SELL_SIZE/GRID_SIZE];


for(int i=0;i<G_SELL_SIZE/GRID_SIZE;i++){
		for(int j=0;j<G_SELL_SIZE/GRID_SIZE;j++){
			LocalMap2[i][j] = LocalMap[i][j];
		}
	}

for(int m=0;m<NumOfMobje;m++){
	local_obje=Convert_Global2Local(robot_data.now.pose , robot_data.now.direction ,array_Mobje[m]->CPosition);
	if(local_obje.x!=-1&&local_obje.x>=0 && local_obje.y>=0&&local_obje.x<G_SELL_SIZE/GRID_SIZE&&local_obje.y<G_SELL_SIZE/GRID_SIZE&&array_Mobje[m]->velocity <=15.0)
	{
	for(s=-(2*MobjeR) / GRID_SIZE;s<=(2*MobjeR) / GRID_SIZE;s++){
					for(t=-(2*MobjeR) / GRID_SIZE;t<=(2*MobjeR) / GRID_SIZE;t++){
						if(POW(s)+POW(t)<=POW((2*MobjeR) / GRID_SIZE)){
						LocalMap2[local_obje.x+s][local_obje.y+t]=0;
						//LocalMap2[local_obje.x+s][local_obje.y+t]|=SFLAG;
						}
					}
	}
	}
	
}

local_goal=Convert_Global2Local(robot, xg);
	if(local_goal.x!=-1&&local_goal.x>=0 && local_goal.y>=0&&local_goal.x<G_SELL_SIZE/GRID_SIZE&&local_goal.y<G_SELL_SIZE/GRID_SIZE)
	{
	for(s=-MobjeR / GRID_SIZE;s<=MobjeR / GRID_SIZE;s++){
					for(t=-MobjeR / GRID_SIZE;t<=MobjeR / GRID_SIZE;t++){
						if(POW(s)+POW(t)<=POW(MobjeR / GRID_SIZE)){
						LocalMap2[local_goal.x+s][local_goal.y+t]=0;
						LocalMap2[local_goal.x+s][local_goal.y+t]|=R_AvailableFLAG;
						}
					}
	}
	}

for(int i=0;i<G_SELL_SIZE/GRID_SIZE;i++){
		for(int j=0;j<G_SELL_SIZE/GRID_SIZE;j++){
			LocalMap[i][j] = LocalMap2[i][j];
		}
	}

}

void dilation4(unsigned char **LocalMap,R_OBJE *robot ,float *xg)
{
static CvPoint local_goal;
static int s,t;
static unsigned char LocalMap2[G_SELL_SIZE/GRID_SIZE][G_SELL_SIZE/GRID_SIZE];


for(int i=0;i<G_SELL_SIZE/GRID_SIZE;i++){
		for(int j=0;j<G_SELL_SIZE/GRID_SIZE;j++){
			LocalMap2[i][j] = LocalMap[i][j];
		}
	}

local_goal=Convert_Global2Local(robot, xg);
	if(local_goal.x!=-1&&local_goal.x>=0 && local_goal.y>=0&&local_goal.x<G_SELL_SIZE/GRID_SIZE&&local_goal.y<G_SELL_SIZE/GRID_SIZE)
	{
	for(s=-MobjeR/2 / GRID_SIZE;s<=MobjeR/2 / GRID_SIZE;s++){
					for(t=-MobjeR/2 / GRID_SIZE;t<=MobjeR/2 / GRID_SIZE;t++){
						if(POW(s)+POW(t)<=POW(MobjeR/2 / GRID_SIZE)){
						//LocalMap2[local_goal.x+s][local_goal.y+t]=0;
						LocalMap2[local_goal.x+s][local_goal.y+t]|=R_AvailableFLAG;
						LocalMap2[G_SELL_SIZE/GRID_SIZE/2][G_SELL_SIZE/GRID_SIZE/2]|=R_AvailableFLAG;
						}
					}
	}
	}

for(int i=0;i<G_SELL_SIZE/GRID_SIZE;i++){
		for(int j=0;j<G_SELL_SIZE/GRID_SIZE;j++){
			LocalMap[i][j] = LocalMap2[i][j];
		}
	}

}

inline CvPoint direct_search(IplImage *gauss_f32,IplImage *dt_f32,CvPoint point){
static float temp,value;
static CvPoint local;

temp=0;value=0;

for(int i=-50;i<=50;i++){
	for(int j=-50;j<=0;j++){
		if(point.x+i>=0 && point.y+j>=0 && point.x+i<200 && point.y+j<200)
		{
			value=(float)cvGetReal2D(gauss_f32,(point.x+i),(point.y+j))+(float)2.0f*cvGetReal2D(dt_f32,(point.x+i),(point.y+j));
			if(value>temp){temp=value;local.x=point.x+i;local.y=point.y+j;}
		}
	}
}
return local;
}

CvPoint Draw_extended_waypoint(IplImage *mpimg,IplImage *gauss_f32,IplImage *dt_f32){
static float lsm;
static CvPoint local1,local2;
int counter=0;

local1.x=100;local1.y=100;
//lsm=(float)cvGetReal2D(gauss_f32,local1.x,local1.y);

	//while(lsm<=900.f){
		local2=direct_search(gauss_f32,dt_f32,local1);
		//cvLine(mpimg,cvPoint((local1.x*(GRID_SIZE/DRAW_RESOLUTION)),(local1.y*(GRID_SIZE/DRAW_RESOLUTION))),cvPoint((local2.x*(GRID_SIZE/DRAW_RESOLUTION)),(local2.y*(GRID_SIZE/DRAW_RESOLUTION))),BLACK,1,8,0);
		cvCircle (mpimg, cvPoint((int)(local2.x*((float)GRID_SIZE/(float)DRAW_RESOLUTION)),(int)(local2.y*((float)GRID_SIZE/(float)DRAW_RESOLUTION))) , 3 , BLACK , 1);
		//local1=local2;
		//lsm=(float)cvGetReal2D(gauss_f32,local1.x,local1.y);
		//counter++;
	//}

	//printf("counter direct = %d \n",counter);
return local2;

}
