// ================================================================================== //
// Name of File				: tree_new.cpp
// Vendor					: AISL-TUT
// Version					: 2.0.3
// ================================================================================== //

#include "tree_new.h"
#include "obst_info.h"
#include <time.h>

using namespace std;

inline bool is_reach_goal(Position goal,Position point);
inline bool check_collision(Position pose, M_OBJE *obstacle[OBJECT_MAX_NUM],R_OBJE *robot ,DATA_LOGGER robot_data,float time,short NumOfMobje);
inline bool search_motion(unsigned char **LocalMap, IplImage *gauss_f32,IplImage *dt_f32,R_OBJE *robot,DATA_LOGGER robot_data,TREE *tree, MOTION_CONTROL *motion,M_OBJE *obstacle[OBJECT_MAX_NUM],TREE *temp_tree,TREE *temp_tree1,TREE *temp_tree2,Position target_node,int chosen_node,short NumOfMobje,Position goal);
inline void make_node(unsigned char **LocalMap, IplImage *gauss_f32,IplImage *dt_f32,R_OBJE *robot,DATA_LOGGER robot_data,TREE *tree, MOTION_CONTROL *motion,M_OBJE *obstacle[OBJECT_MAX_NUM],int *node,float *LSM,unsigned short *warning_flag,short NumOfMobje,float time_base,Position goal);
inline void convert_toVLVR(float r , float vel, float *vr, float *vl);

/********** Determine if the robot has arrived at the goal **********/
unsigned char Check_Arrive(TREE *path, int step, float *xg, float *distances, float threshold)
{
	float dx,dy,dst;
	dx = xg[1]-path[step].pose.x;//xc[1];
	dy = xg[2]-path[step].pose.y;//xc[2];
	dx *= dx; dy *= dy;
	dst = dx+dy;

	if(dst < threshold) {// && dt < PI_2) {
		printf("Goal Achieved!!!\n");
		return TRUE2;
	}

	*distances = sqrt(dst);
	return 0;
}

/********** Initialize tree **********/
int InitState(TREE *tree , int maxnode)
{
	for(int i=0;i < maxnode;i++){
		tree[i].parent = 0;
		tree[i].radius = tree[i].velocity = tree[i].time = tree[i].pose.x = tree[i].pose.y = tree[i].direction = 0.0;
		tree[i].density = 0.0;
		tree[i].vector = 0;
		tree[i].local_pose.x = 0;
		tree[i].local_pose.y = 0;
	}
	return 0;
}

inline void convert_toVLVR(float r , float vel, float *vr, float *vl)//Specifies the radius of gyration
{
	float inside_r , outside_r;
	float inside_v , outside_v;
	//Seek inner radius and outer radius
	inside_r  = (float)(r-(Wheel_Tread/2));
	outside_r = (float)(r+(Wheel_Tread/2));
	//Calculating the speed of the inner ring and outer ring
	if(ABS(r) >= Wheel_Tread/2){
		inside_v  = vel * (inside_r  / (float)r);
		outside_v = vel * (outside_r / (float)r);
	}else if(r == 0){
		inside_v = vel;
		outside_v = vel;
	}
	else{inside_v=-vel;outside_v=vel;}
	//inside_v = left, outside_v = right;
	*vr = inside_v;
	*vl = outside_v;
}


inline bool is_reach_goal(Position goal,Position point)
{
static float dx,dy,r;

	dx=point.x-goal.x;
	dy=point.y-goal.y;
	r=sqrt(POW(dx)+POW(dy));
	//if(r<MobjeR+50)return true;
	if(r<(MobjeR+RR))return true;
	else return false;
}

int build_path(TREE *tree,TREE *path,int closest_node,int step)
{
static vector<TREE>temp;
static int i,j;
static float vl_motion,vr_motion,vl,vr,delta_L,R,delta_theta,omega;
static bool refine;

refine=false;
i=closest_node;
j=0;

	temp.clear();
	//printf("inside build START\n");

	if(i==0){}
	else{
		while(i){
			temp.push_back(tree[i]);
			j=tree[i].parent;
			i=j;
		}
		temp.push_back(tree[0]);

		j=1;
		while(!temp.empty()){
			path[step+j]=temp.back();
			temp.pop_back();
			j++;
		}
	}
	
	// simple refine
	for(i=0;i<j;i++){
		if(path[step+i+1].velocity==path[step+i].velocity && path[step+i+1].radius+path[step+i].radius==0){path[step+i+1].radius=0;path[step+i].radius=0;refine=true;}
	}

	if(refine==true){
		for(i=0;i<j;i++){
			if(path[step+i+1].radius != 0.0){
			vl_motion = float(path[step+i+1].velocity * (1 + Wheel_Tread/(2*path[step+i+1].radius)));
			vr_motion = float(path[step+i+1].velocity * (1 - Wheel_Tread/(2*path[step+i+1].radius)));
			}else{
			vl_motion = path[step+i+1].velocity;
			vr_motion = path[step+i+1].velocity;
			}
			// Shape of the function will be replaced with the following speed settings
			if(path[step+i+1].radius !=0.0){
			vl_motion += (float)(path[step+i].velocity * (1 + Wheel_Tread/(2*path[step+i+1].radius)));
			vr_motion += (float)(path[step+i].velocity * (1 - Wheel_Tread/(2*path[step+i+1].radius)));
			}else{
			vl_motion += path[step+i].velocity;
			vr_motion += path[step+i].velocity;
			}
			vr_motion /= 2;
			vl_motion /= 2;
			
			omega = float((vl_motion - vr_motion) / Wheel_Tread);//Velocity calculation
			//delta_theta = float(omega*(LoopTime-((timeGetTime()-loopTimer)/1000)));//rad
			delta_theta = float(omega*(LoopTime));//rad

			if(vr_motion != vl_motion){
			R = float((Wheel_Tread/2.0) * (vr_motion + vl_motion)/(vl_motion - vr_motion));//Turning radius
			delta_L = 2.f*R*sin(delta_theta/2);
			}
			else{
			//delta_L_dash = float(vr*( LoopTime - ((timeGetTime()-loopTimer)/1000) ));
			delta_L = float(vr_motion*( LoopTime));
			}
			path[step+i+1].direction = path[step+i].direction + delta_theta;
			path[step+i+1].pose.x    = float(path[step+i].pose.x + delta_L * cos(path[step+i].direction + delta_theta/2.0));
			path[step+i+1].pose.y    = float(path[step+i].pose.y + delta_L * sin(path[step+i].direction + delta_theta/2.0));
		}
	}

printf("build path : %d node\n",j);
return j;

}


void save_path_for_next_use(TREE *path,TREE *tree,int step,int path_depth)
{
static int number;

	InitState(tree,MAXNODE);

	if(path_depth>REUSE_NUM_NODE)number=REUSE_NUM_NODE;else number=path_depth;
	if(number!=0){
		for(int i=0;i<=number;i++){
			tree[i].radius=path[step+2+i].radius;
			tree[i].velocity=path[step+2+i].velocity;
			if(i!=0)tree[i].parent=i-1;
			tree[i].time=i+1;
		}
	}

}

bool reuse_prevoius_path(unsigned char **LocalMap,IplImage *gauss_f32,TREE *tree,M_OBJE *obstacle[OBJECT_MAX_NUM],R_OBJE *robot ,DATA_LOGGER robot_data,int path_depth,short NumOfMobje)
{
static Position pose;
static CvPoint local_pose;
static float time;
static int number;
static bool flag,flag_reuse;
static float omega_motion,velo_motion;
static float vl_motion,vr_motion,vl,vr,delta_L,R,delta_theta,omega;


flag=false;
flag_reuse=true;

//printf("start inspecting REUSE PATH\n");

	tree[0].pose=robot->CPosition;
	tree[0].direction=robot->direction;
	tree[0].local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,tree[0].pose);

	if(path_depth>REUSE_NUM_NODE)number=REUSE_NUM_NODE;else number=path_depth;

	for(int i=0;i<=number;i++){
		flag=false;

		//old method
			if(tree[i+1].radius != 0.0){
			vl_motion = float(tree[i+1].velocity * (1 + Wheel_Tread/(2*tree[i+1].radius)));
			vr_motion = float(tree[i+1].velocity * (1 - Wheel_Tread/(2*tree[i+1].radius)));
			}else{
			vl_motion = tree[i+1].velocity;
			vr_motion = tree[i+1].velocity;
			}
			// Shape of the function will be replaced with the following speed settings
			if(tree[i+1].radius !=0.0){
			vl_motion += (float)(tree[i].velocity * (1 + Wheel_Tread/(2*tree[i+1].radius)));
			vr_motion += (float)(tree[i].velocity * (1 - Wheel_Tread/(2*tree[i+1].radius)));
			}else{
			vl_motion += tree[i].velocity;
			vr_motion += tree[i].velocity;
			}
			vr_motion /= 2;
			vl_motion /= 2;
			
			omega = float((vl_motion - vr_motion) / Wheel_Tread);//Velocity calculation
			//delta_theta = float(omega*(LoopTime-((timeGetTime()-loopTimer)/1000)));//rad
			delta_theta = float(omega*(LoopTime));//rad

			if(vr_motion != vl_motion){
			R = float((Wheel_Tread/2.0) * (vr_motion + vl_motion)/(vl_motion - vr_motion));//Turning radius
			delta_L = 2.f*R*sin(delta_theta/2);
			}
			else{
			//delta_L_dash = float(vr*( LoopTime - ((timeGetTime()-loopTimer)/1000) ));
			delta_L = float(vr_motion*( LoopTime));
			}
			tree[i+1].direction = tree[i].direction + delta_theta;
			tree[i+1].pose.x    = float(tree[i].pose.x + delta_L * cos(tree[i].direction + delta_theta/2.0));
			tree[i+1].pose.y    = float(tree[i].pose.y + delta_L * sin(tree[i].direction + delta_theta/2.0));

		// new method
		/*velo_motion=(tree[i+1].velocity+tree[i].velocity)/2;
		if(tree[i+1].radius != 0.0)omega_motion=velo_motion/((tree[i+1].radius));
		if(tree[i+1].radius != 0.0){
			tree[i+1].pose.x=tree[i].pose.x-(tree[i+1].radius*sin(tree[i].direction))+(tree[i+1].radius*sin(tree[i].direction+(omega_motion*(LoopTime))));
			tree[i+1].pose.y=tree[i].pose.y+(tree[i+1].radius*cos(tree[i].direction))-(tree[i+1].radius*cos(tree[i].direction+(omega_motion*(LoopTime))));
			tree[i+1].direction=tree[i].direction+omega_motion*(LoopTime);
		}
		else{
			tree[i+1].pose.x=tree[i].pose.x+velo_motion*cos(tree[i].direction)*(LoopTime);
			tree[i+1].pose.y=tree[i].pose.y+velo_motion*sin(tree[i].direction)*(LoopTime);
			tree[i+1].direction=tree[i].direction;
		}*/

		pose=tree[i+1].pose;
		time=tree[i+1].time;
		flag=check_collision(pose,obstacle,robot,robot_data,time,NumOfMobje);
		local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,pose);
		tree[i+1].local_pose=local_pose;
		if(local_pose.x>=0 && local_pose.y>=0){if((LocalMap[local_pose.x][local_pose.y]&R_AvailableFLAG)!=0 && (float)cvGetReal2D(gauss_f32,local_pose.x,local_pose.y)>0.f)flag|=false;else flag|=true;}
		if(flag==true){flag_reuse=false;break;}
	}

	if(flag_reuse==true){
	//printf("==============\n");
	//printf("REUSE PATH !!!\n");
	//printf("==============\n");
	}

return flag_reuse;

}



int search_close_node(IplImage *gauss_f32,IplImage *dt_f32,TREE *tree,DATA_LOGGER robot_data,Position goal,int num_node)
{
static int path_depth;
float temp=10000000.f;
static float dx,dy,r,criterion;
static CvPoint local_pose;

	for(int i=0;i<=num_node;i++){
		local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,tree[i].pose);
		dx=tree[i].pose.x-goal.x;
		dy=tree[i].pose.y-goal.y;
		r=sqrt(POW(dx)+POW(dy));
		if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE)criterion=0.1f*r-10.0f*(float)cvGetReal2D(gauss_f32,local_pose.x,local_pose.y);else criterion=0.1f*r;
		if(criterion<temp){temp=criterion;path_depth=i;}
	}

return path_depth;

}

inline bool check_collision(Position pose, M_OBJE *obstacle[OBJECT_MAX_NUM],R_OBJE *robot ,DATA_LOGGER robot_data,float time,short NumOfMobje)
{
static bool flag;
static Position pose_object;
static CvPoint local_pose;
float time1;

if(time>10)time1=10;else time1=time;


	for(int i=0;i<NumOfMobje;i++){
		local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,obstacle[i]->CPosition);
		//printf("start COLLISION checking\n");
		//if((local_pose.x!=-1 || local_pose.y!=-1) && obstacle[i]->velocity>15.0 ){	
		if((local_pose.x!=-1 || local_pose.y!=-1) && obstacle[i]->velocity>10.0 ){
			pose_object.x=obstacle[i]->CPosition.x+cos(obstacle[i]->direction)*obstacle[i]->velocity*LoopTime*time1;
			pose_object.y=obstacle[i]->CPosition.y+sin(obstacle[i]->direction)*obstacle[i]->velocity*LoopTime*time1;
			if((pose.x>=(pose_object.x-(MobjeR+RR)) && pose.x<=(pose_object.x+(MobjeR+RR))) && (pose.y>=(pose_object.y-(MobjeR+RR)) && pose.y<=(pose_object.y+(MobjeR+RR))))
			{
				flag=true;
				//printf("COLLIDE with Object !!!\n");
				break;
			}
			else flag=false;
		}
	}

return flag;

}

inline bool search_motion(unsigned char **LocalMap, IplImage *gauss_f32,IplImage *dt_f32,R_OBJE *robot,DATA_LOGGER robot_data,TREE *tree, MOTION_CONTROL *motion,M_OBJE *obstacle[OBJECT_MAX_NUM],TREE *temp_tree,TREE *temp_tree1,TREE *temp_tree2,Position target_node,int chosen_node,short NumOfMobje,Position goal)
{
static Position pose,pose_next;
static CvPoint local_pose;
static float radius_motion,velo_motion,velo,radi,omega,time;
float vl_motion,vr_motion,vl,vr,delta_L,R,delta_theta;
static float direction,direction_next,dist,dist1,delta_dir,delta_dir2;
float criterion,criterion1,criterion2;
float temp_dist,temp_dist1,temp_dist2;
bool flag=false,flag_collision=false;

temp_dist=1000000000.f;
temp_dist1=0.f;
temp_dist2=1000000000.f;

	for(int i=0;i<motion->data.motion_num;i++){
		//for(int j=0;j<MAX_OBS;j++){
		//if((obstacle[j].local_pose.x!=-1 || obstacle[j].local_pose.y!=-1) && (obstacle[j].vx!=0 && obstacle[j].vy!=0))
		//{	
		radius_motion=motion->data.r_v[i].radius;
		velo_motion=motion->data.r_v[i].velocity;
		velo=tree[chosen_node].velocity;
		radi=tree[chosen_node].radius;

		pose.x=tree[chosen_node].pose.x;
		pose.y=tree[chosen_node].pose.y;
		direction=tree[chosen_node].direction;

		float temp_vel;
		if((float)cvGetReal2D(dt_f32,tree[chosen_node].local_pose.x,tree[chosen_node].local_pose.y)<=6.f)temp_vel=300.f;else temp_vel=MAX_CONSTRAINT;

		//convert_toVLVR(radius_motion,velo_motion,&vr_motion,&vl_motion);
		//convert_toVLVR(radi,velo,&vr,&vl);

		if((ABS(velo_motion-velo))/(LoopTime)<=ACCEL_CONSTRAINT && velo_motion>=0 && velo_motion<=temp_vel && velo_motion>=100 /*&& ABS(vr_motion-vr)<=150 && ABS(vl_motion-vl)<=150*/){
		//if((ABS(velo_motion-velo))/(LoopTime)<ACCEL_CONSTRAINT){
			
			
			//old method
			if(radius_motion != 0.0){
			vl_motion = float(velo_motion * (1 + Wheel_Tread/(2*radius_motion)));
			vr_motion = float(velo_motion * (1 - Wheel_Tread/(2*radius_motion)));
			}else{
			vl_motion = velo_motion;
			vr_motion = velo_motion;
			}
			// Shape of the function will be replaced with the following speed settings
			if(radi !=0.0){
			vl_motion += (float)(velo * (1 + Wheel_Tread/(2*radi)));
			vr_motion += (float)(velo * (1 - Wheel_Tread/(2*radi)));
			}else{
			vl_motion += velo;
			vr_motion += velo;
			}
			vr_motion /= 2;
			vl_motion /= 2;
			
			omega = float((vl_motion - vr_motion) / Wheel_Tread);//Velocity calculation
			//delta_theta = float(omega*(LoopTime-((timeGetTime()-loopTimer)/1000)));//rad
			delta_theta = float(omega*(LoopTime));//rad

			if(vr_motion != vl_motion){
			R = float((Wheel_Tread/2.0) * (vr_motion + vl_motion)/(vl_motion - vr_motion));//Turning radius
			delta_L = 2.f*R*sin(delta_theta/2);
			}
			else{
			//delta_L_dash = float(vr*( LoopTime - ((timeGetTime()-loopTimer)/1000) ));
			delta_L = float(vr_motion*( LoopTime));
			}
			direction_next = direction + delta_theta;
			pose_next.x    = float(pose.x + delta_L * cos(direction + delta_theta/2.0));
			pose_next.y    = float(pose.y + delta_L * sin(direction + delta_theta/2.0));

			// new method
			/*if(radius_motion!=0)omega=((velo_motion+velo)/2)/(radius_motion);
			
			if(radius_motion!=0){
				pose_next.x=pose.x-(radius_motion*sin(direction))+(radius_motion*sin(direction+(omega*LoopTime)));
				pose_next.y=pose.y+(radius_motion*cos(direction))-(radius_motion*cos(direction+(omega*LoopTime)));
				direction_next=direction+omega*LoopTime;
			}
			else{
				pose_next.x=pose.x+((velo_motion+velo)/2)*cos(direction)*LoopTime;
				pose_next.y=pose.y+((velo_motion+velo)/2)*sin(direction)*LoopTime;
				direction_next=direction;
			}*/
#ifdef RRT_ONLY			

			dist=sqrt(POW(pose_next.x-target_node.x)+POW(pose_next.y-target_node.y));
			local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,pose_next);
			time=tree[chosen_node].time;
			if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE)criterion=dist;
			if(time>10)flag_collision=false;else flag_collision=check_collision(pose_next,obstacle,robot,robot_data,time,NumOfMobje);
			if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE &&( LocalMap[local_pose.x][local_pose.y]&R_AvailableFLAG)!=0 && criterion<=temp_dist && flag_collision==false){
					(*temp_tree).pose=pose_next;
					(*temp_tree).direction=direction_next;
					(*temp_tree).parent=chosen_node;
					(*temp_tree).velocity=velo_motion;
					(*temp_tree).time=time+1;
					(*temp_tree).radius=radius_motion;
					temp_dist=criterion;
					flag=true;
					}
#else
			dist=sqrt(POW(pose_next.x-target_node.x)+POW(pose_next.y-target_node.y));
			dist1=sqrt(POW(pose_next.x-goal.x)+POW(pose_next.y-goal.y));
			local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,pose_next);
			time=tree[chosen_node].time;
			delta_dir=ABS(direction_next-direction);
			delta_dir2=(direction_next-atan2(pose_next.x-goal.x,pose_next.y-goal.y))*R2D;
				while(delta_dir2 > 180){
							delta_dir2 -= 360;
						}
						while(delta_dir2 < -180){
							delta_dir2 += 360;
						}
			delta_dir2=ABS(delta_dir2);
			if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE)criterion=0.2f*dist-1.0f*(float)cvGetReal2D(gauss_f32,local_pose.x,local_pose.y)-20.0f*(float)cvGetReal2D(dt_f32,local_pose.x,local_pose.y)+5.0f*delta_dir+15.0f*delta_dir2;
			if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE)criterion1=(float)cvGetReal2D(gauss_f32,local_pose.x,local_pose.y);
			if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE)criterion2=0.25f*dist1+5.0f*delta_dir;


			if(time>10)flag_collision=false;else flag_collision=check_collision(pose_next,obstacle,robot,robot_data,time,NumOfMobje);
			if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE &&( LocalMap[local_pose.x][local_pose.y]&R_AvailableFLAG)!=0 && criterion<=temp_dist && flag_collision==false && (float)cvGetReal2D(gauss_f32,local_pose.x,local_pose.y)>=0.f){
					(*temp_tree).pose=pose_next;
					(*temp_tree).direction=direction_next;
					(*temp_tree).local_pose=local_pose;
					(*temp_tree).parent=chosen_node;
					(*temp_tree).velocity=velo_motion;
					(*temp_tree).time=time+1;
					(*temp_tree).radius=radius_motion;
					temp_dist=criterion;
					flag=true;
					}
			if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE &&( LocalMap[local_pose.x][local_pose.y]&R_AvailableFLAG)!=0 && criterion1>=temp_dist1 && flag_collision==false && (float)cvGetReal2D(gauss_f32,local_pose.x,local_pose.y)>=0.f){
					(*temp_tree1).pose=pose_next;
					(*temp_tree1).direction=direction_next;
					(*temp_tree1).local_pose=local_pose;
					(*temp_tree1).parent=chosen_node;
					(*temp_tree1).velocity=velo_motion;
					(*temp_tree1).time=time+1;
					(*temp_tree1).radius=radius_motion;
					temp_dist1=criterion1;
					flag=true;
					}
			if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE &&( LocalMap[local_pose.x][local_pose.y]&R_AvailableFLAG)!=0 && criterion2<=temp_dist2 && flag_collision==false && (float)cvGetReal2D(gauss_f32,local_pose.x,local_pose.y)>=0.f){
					(*temp_tree2).pose=pose_next;
					(*temp_tree2).direction=direction_next;
					(*temp_tree2).local_pose=local_pose;
					(*temp_tree2).parent=chosen_node;
					(*temp_tree2).velocity=velo_motion;
					(*temp_tree2).time=time+1;
					(*temp_tree2).radius=radius_motion;
					temp_dist2=criterion2;
					flag=true;
					}
#endif		
	//	}

	//}
		}
	}
	//printf("Local Pose [%d][%d]\n",local_pose.x,local_pose.y);
return flag;
}


inline void make_node(unsigned char **LocalMap, IplImage *gauss_f32,IplImage *dt_f32,R_OBJE *robot,DATA_LOGGER robot_data,TREE *tree, MOTION_CONTROL *motion,M_OBJE *obstacle[OBJECT_MAX_NUM],int *node,float *LSM,unsigned short *warning_flag,short NumOfMobje,float time_base,Position goal)
{
static CvPoint random,point;
static CvRNG rng = cvRNG(time(NULL));
float temp_LSM;
static float temp;
static int temp_node;
static Position target_node;
static bool flag;
static TREE temp_tree,temp_tree1,temp_tree2;

#ifdef RRT_ONLY
temp=10000000.f;
DWORD time1;
time1=timeGetTime();

	while(1){
		random.x=(int)(cvRandReal(&rng)*(G_SELL_SIZE / GRID_SIZE));
		random.y=(int)(cvRandReal(&rng)*(G_SELL_SIZE / GRID_SIZE));
		
		if(random.x>=0 && random.x<(G_SELL_SIZE / GRID_SIZE) && random.y>=0 && random.y<(G_SELL_SIZE / GRID_SIZE)){break;}
		if((double)(timeGetTime()-time1)>5.0f){
			/*printf("OVER TIME SEARCH RND !!!\n time elapse for RND = %d\n",timeGetTime()-time1);*/
			*warning_flag|=TIME_OVER;
			break;
		}
	}
	//printf("target point [%d][%d]\n",random.x,random.y);


target_node=Convert_Local2Global(random,robot_data.now.pose,robot_data.now.direction);
	for(int i=0;i<=(*node);i++){
		float dx=target_node.x-tree[i].pose.x;
		float dy=target_node.y-tree[i].pose.y;
		float r=sqrt(POW(dx)+POW(dy));
		if(r<temp){temp_node=i;temp=r;}
		
	}

	flag=search_motion(LocalMap,gauss_f32,robot,robot_data,tree,motion,obstacle,&temp_tree,&temp_tree1,&temp_tree2,target_node,temp_node,NumOfMobje,goal);
	if(flag==true){
		(*node)++;
		//(*LSM)=cvGetReal2D(gauss_f32,random.x,random.y);
		tree[(*node)]=temp_tree;
		tree[(*node)].local_pose=Convert_Global2Local(robot,tree[(*node)].pose);
	}

#else

temp_LSM=(*LSM);
temp=10000000.f;

DWORD time1;
time1=timeGetTime();

	while(1){
		random.x=(int)(cvRandReal(&rng)*(G_SELL_SIZE / GRID_SIZE));
		random.y=(int)(cvRandReal(&rng)*(G_SELL_SIZE / GRID_SIZE));
		float value=(float)cvGetReal2D(gauss_f32,random.x,random.y);
		if(random.x>=0 && random.x<(G_SELL_SIZE / GRID_SIZE) && random.y>=0 && random.y<(G_SELL_SIZE / GRID_SIZE)){
			//if((float)cvGetReal2D(gauss_f32,random.x,random.y)>(3*temp_LSM+995)/4){break;}
			//if(value>(2*temp_LSM+(float)cvGetReal2D(gauss_f32,100,100))/3 && value<(temp_LSM+995)/2){break;}
			if(value>temp_LSM){break;}
		}

		if((double)(timeGetTime()-time1)>10.0f){
			//printf("OVER TIME SEARCH RND !!!\n time elapse for RND = %d\n",timeGetTime()-time1);
			*warning_flag|=TIME_OVER;
			break;
		}
	}

	//printf("target LSM %f,boundary LSM %f, point [%d][%d]\n",(float)cvGetReal2D(gauss_f32,random.x,random.y),temp_LSM,random.x,random.y);

	target_node=Convert_Local2Global(random,robot_data.now.pose,robot_data.now.direction);
	for(int i=0;i<=(*node);i++){
		float dx=target_node.x-tree[i].pose.x;
		float dy=target_node.y-tree[i].pose.y;
		float r=sqrt(POW(dx)+POW(dy));
		if(time_base!=0){
			if(r<temp && tree[i].time<(time_base+20.f)){temp_node=i;temp=r;}
		}
		else {
			if(r<temp){temp_node=i;temp=r;}
		}
	}
	//temp_node=(int)(cvRandReal(&rng)*(*node));

	//printf("Chosen Node= %d\n",temp_node);

	//search motion
	flag=search_motion(LocalMap,gauss_f32,dt_f32,robot,robot_data,tree,motion,obstacle,&temp_tree,&temp_tree1,&temp_tree2,target_node,temp_node,NumOfMobje,goal);
	if(flag==true){
		(*node)++;
		//(*LSM)=cvGetReal2D(gauss_f32,random.x,random.y);
		tree[(*node)]=temp_tree;
		tree[(*node)].local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,tree[(*node)].pose);
		point=tree[(*node)].local_pose;
		if(point.x>=0 && point.x<(G_SELL_SIZE / GRID_SIZE) && point.y>=0 && point.y<(G_SELL_SIZE / GRID_SIZE)){
			temp_LSM=(float)cvGetReal2D(gauss_f32,point.x,point.y);
		}
		//if(temp_LSM<(*LSM))(*LSM)=temp_LSM;
		if(temp_LSM>(*LSM)){if(temp_LSM-(*LSM)>50)(*LSM)+=50;else (*LSM)=temp_LSM;}
		(*node)++;
		tree[(*node)]=temp_tree1; 
		tree[(*node)].local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,tree[(*node)].pose);
		(*node)++;
		tree[(*node)]=temp_tree2; 
		tree[(*node)].local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,tree[(*node)].pose);
	}
	else{
		(*LSM)=(float)cvGetReal2D(gauss_f32,100,100);
	}


#endif

}

int make_tree(unsigned char **LocalMap,IplImage *gauss_f32,IplImage *dt_f32,TREE *tree,Position goal,R_OBJE *robot,DATA_LOGGER robot_data,MOTION_CONTROL *motion,M_OBJE *obstacle[OBJECT_MAX_NUM],unsigned short *warning_flag,short NumOfMobje,bool flag_reuse,int path_depth)
{
int node=0,node_temp=0;
float time_base=0;
bool flag=false,flag_success=false;
float temp_LSM,temp_LSM1;
static CvPoint local_goal;



	if(flag_reuse==true && path_depth>1)
	{
		if(path_depth>REUSE_NUM_NODE)node=REUSE_NUM_NODE;else node=path_depth;
	}
	else{
		InitState(tree , MAXNODE);
		tree[0].pose=robot->CPosition;
		tree[0].local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,tree[0].pose);
		tree[0].direction=(float)robot->direction;
		tree[0].velocity=(float)robot->velocity;
		tree[0].radius=(float)robot->radius;
		tree[0].time=1;
		tree[0].parent=0;
	}

	temp_LSM=(float)cvGetReal2D(gauss_f32,tree[node].local_pose.x,tree[node].local_pose.y);
	temp_LSM1=temp_LSM;
	//node++;

DWORD time1;
time1=timeGetTime();

	local_goal=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,goal);
	//printf("Local Goal [%d][%d] with LSM %f\n",local_goal.x,local_goal.y,(float)cvGetReal2D(gauss_f32,local_goal.x,local_goal.y));

	//printf("1. starting RANDOM PATH\n");

	time_base=(float)sqrt(POW(goal.x-tree[0].pose.x)+POW(goal.y-tree[0].pose.y))/LoopTime;
	node_temp=node;

	while(1){
		for(int i=node_temp;i<node;i++){
			flag=is_reach_goal(goal,tree[i].pose);
			if(flag==true){flag_success=true;if(time_base==0)time_base=tree[i].time+10.f;temp_LSM=(temp_LSM1+900/2);/*break;*/}
		}
		if((double)(timeGetTime()-time1)/1000>(LoopTime/2)){break;}
		if(node>(MAXNODE-10)){printf("Number of Node exceed !!\n");break;}
		node_temp=node;
		make_node(LocalMap,gauss_f32,dt_f32,robot,robot_data,tree,motion,obstacle,&node,&temp_LSM,warning_flag,NumOfMobje,time_base,goal);
	}

	if(flag_success==false){
		printf("CAN'T MAKE TREE , end in node %d!!!! \n",node);
	}
	else{
		printf("SUCCESS MAKE TREE !!!! \n");
	}

	//printf("zero velocity %f, 1st velocity %f\n",tree[0].velocity,tree[1].velocity);
	printf("Make Full Tree Time %d ms\n",timeGetTime()-time1);


return node;

}

int make_tree_best(unsigned char **LocalMap,IplImage *gauss_f32,IplImage *dt_f32,TREE *tree,Position goal,R_OBJE *robot,DATA_LOGGER robot_data,MOTION_CONTROL *motion,M_OBJE *obstacle[OBJECT_MAX_NUM],unsigned short *warning_flag,short NumOfMobje,bool flag_reuse,int path_depth)
{
int node=0;
DWORD time1;
static float radius_motion,velo_motion,omega,velo,radi,direction,direction_next,dist,temp_dist;
float vl_motion,vr_motion,vl,vr,delta_L,R,delta_theta;
static float criterion,time,delta_dir,delta_dir2;
static Position pose, pose_next;
TREE temp_tree;
CvPoint local_pose;
bool flag=false,flag_collision=false,flag_node=false;

	if(flag_reuse==true && path_depth>1){
		if(path_depth>REUSE_NUM_NODE)node=REUSE_NUM_NODE;else node=path_depth;
		printf("BEST PATH and REUSE \n");
	}
	else{
		InitState(tree,MAXNODE);
		tree[0].pose=robot->CPosition;
		tree[0].local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,tree[0].pose);
		tree[0].direction=(float)robot->direction;
		tree[0].velocity=(float)robot->velocity;
		tree[0].radius=(float)robot->radius;
		tree[0].time=1;
		tree[0].parent=0;
	}

time1=timeGetTime();

	//printf("1. starting BEST PATH\n");

	while(1){
		flag=is_reach_goal(goal,tree[node].pose);
		if(flag==true){break;}
		if((double)(timeGetTime()-time1)/1000>0.020f){printf("Time exceed !!\n");(*warning_flag)|=TIME_OVER;break;}
		if(node>MAXSTEP){printf("Number of Node exceed !!\n");(*warning_flag)|=CANNOT_MAKE_NODE;break;}
		temp_dist=100000000000.f;
		//printf("2. starting BEST PATH\n");

		for(int i=0;i<motion->data.motion_num;i++)
		{
			radius_motion=motion->data.r_v[i].radius;
			velo_motion=motion->data.r_v[i].velocity;
			velo=tree[node].velocity;
			radi=tree[node].radius;
			pose.x=tree[node].pose.x;
			pose.y=tree[node].pose.y;
			direction=tree[node].direction;
			float temp_vel;
			if((float)cvGetReal2D(dt_f32,tree[node].local_pose.x,tree[node].local_pose.y)<=6.f)temp_vel=300.f;else temp_vel=MAX_CONSTRAINT;
			//convert_toVLVR(radius_motion,velo_motion,&vr_motion,&vl_motion);
			//convert_toVLVR(radi,velo,&vr,&vl);
			//printf("2. starting BEST PATH\n");
			if((ABS(velo_motion-velo))/(LoopTime)<=ACCEL_CONSTRAINT && velo_motion>=0 && velo_motion<=temp_vel && velo_motion>=100 /*&& ABS(vr_motion-vr)<=150 && ABS(vl_motion-vl)<=150*/){
			//if((ABS(velo_motion-velo))/(LoopTime)<ACCEL_CONSTRAINT ){
		
				pose_next.x=0;pose_next.y=0;
				
				
				//old method
				if(radius_motion != 0.0){
				vl_motion = float(velo_motion * (1 + Wheel_Tread/(2*radius_motion)));
				vr_motion = float(velo_motion * (1 - Wheel_Tread/(2*radius_motion)));
				}else{
				vl_motion = velo_motion;
				vr_motion = velo_motion;
				}
				// Shape of the function will be replaced with the following speed settings
				if(radi !=0.0){
				vl_motion += (float)(velo * (1 + Wheel_Tread/(2*radi)));
				vr_motion += (float)(velo * (1 - Wheel_Tread/(2*radi)));
				}else{
				vl_motion += velo;
				vr_motion += velo;
				}
				vr_motion /= 2;
				vl_motion /= 2;
			
				omega = float((vl_motion - vr_motion) / Wheel_Tread);//Velocity calculation
				//delta_theta = float(omega*(LoopTime-((timeGetTime()-loopTimer)/1000)));//rad
				delta_theta = float(omega*(LoopTime));//rad

				if(vr_motion != vl_motion){
				R = float((Wheel_Tread/2.0) * (vr_motion + vl_motion)/(vl_motion - vr_motion));//Turning radius
				delta_L = 2.f*R*sin(delta_theta/2);
				}
				else{
				//delta_L_dash = float(vr*( LoopTime - ((timeGetTime()-loopTimer)/1000) ));
				delta_L = float(vr_motion*( LoopTime));
				}
				direction_next = direction + delta_theta;
				pose_next.x    = float(pose.x + delta_L * cos(direction + delta_theta/2.0));
				pose_next.y    = float(pose.y + delta_L * sin(direction + delta_theta/2.0));
			
				//new method
				/*if(radius_motion!=0)omega=((velo_motion+velo)/2)/(radius_motion);

				if(radius_motion!=0){
					pose_next.x=pose.x-(radius_motion*sin(direction))+(radius_motion*sin(direction+(omega*LoopTime)));
					pose_next.y=pose.y+(radius_motion*cos(direction))-(radius_motion*cos(direction+(omega*LoopTime)));
					direction_next=direction+omega*LoopTime;
					if(direction_next>360.f)direction=360-direction_next;
				}
				else{
					pose_next.x=pose.x+((velo_motion+velo)/2)*cos(direction)*LoopTime;
					pose_next.y=pose.y+((velo_motion+velo)/2)*sin(direction)*LoopTime;
					direction_next=direction;
				}*/
			
				dist=sqrt(POW(pose_next.x-goal.x)+POW(pose_next.y-goal.y));
				local_pose=Convert_Global2Local(robot_data.now.pose,robot_data.now.direction,pose_next);
				delta_dir2=(direction_next-atan2(pose_next.x-goal.x,pose_next.y-goal.y))*R2D;
				while(delta_dir2 > 180){
							delta_dir2 -= 360;
						}
						while(delta_dir2 < -180){
							delta_dir2 += 360;
						}
				delta_dir2=ABS(delta_dir2);
				delta_dir=ABS(direction_next-direction);
				if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE)criterion=0.2f*dist-10.0f*(float)cvGetReal2D(gauss_f32,local_pose.x,local_pose.y)-0.0f*(float)cvGetReal2D(dt_f32,local_pose.x,local_pose.y)+5.0f*delta_dir+0.0f*delta_dir2;
				time=tree[node].time;
				if(time>10)flag_collision=false;else flag_collision=check_collision(pose_next,obstacle,robot,robot_data,time,NumOfMobje);
				if(local_pose.x>=0 && local_pose.y>=0 && local_pose.x<G_SELL_SIZE / GRID_SIZE && local_pose.y<G_SELL_SIZE / GRID_SIZE && (LocalMap[local_pose.x][local_pose.y] & R_AvailableFLAG)!=0 && criterion<temp_dist && flag_collision==false && (float)cvGetReal2D(gauss_f32,local_pose.x,local_pose.y)>0.f){
					temp_tree.pose=pose_next;
					temp_tree.direction=direction_next;
					temp_tree.local_pose=local_pose;
					temp_tree.parent=node;
					temp_tree.velocity=velo_motion;
					temp_tree.time=time+1;
					temp_tree.radius=radius_motion;
					temp_dist=criterion;
					flag_node=true;
					}
				}
		}
	
		if(flag_node==true){
			node++;
			tree[node]=temp_tree;
			flag_node=false;}
		}

	//printf("zero velocity %f, 1st velocity %f\n",tree[0].velocity,tree[1].velocity);
	printf("Time for making Best Tree = %d ms\n",timeGetTime()-time1);

return node;
}


unsigned char quadran(IplImage *gauss_f32){
	unsigned char quad=0;
	int num_element1=0,num_element2=0,num_element3=0,num_element4=0;
	double value1=0,value2=0,value3=0,value4=0,temp_value=0;

	
		for(int i=0;i<(int)((G_SELL_SIZE / GRID_SIZE)/8);i++){
			for(int j=0;j<(int)((G_SELL_SIZE / GRID_SIZE)/8);j++){
			temp_value=cvGetReal2D(gauss_f32,(int)((G_SELL_SIZE / GRID_SIZE)/2)+i,(int)((G_SELL_SIZE / GRID_SIZE)/2)-j);
			if(temp_value!=0){value1+=temp_value;num_element1++;}
			temp_value=cvGetReal2D(gauss_f32,(int)((G_SELL_SIZE / GRID_SIZE)/2)-i,(int)((G_SELL_SIZE / GRID_SIZE)/2)-j);
			if(temp_value!=0){value2+=temp_value;num_element2++;}
			temp_value=cvGetReal2D(gauss_f32,(int)((G_SELL_SIZE / GRID_SIZE)/2)-i,(int)((G_SELL_SIZE / GRID_SIZE)/2)+j);
			if(temp_value!=0){value3+=temp_value;num_element3++;}
			temp_value=cvGetReal2D(gauss_f32,(int)((G_SELL_SIZE / GRID_SIZE)/2)+i,(int)((G_SELL_SIZE / GRID_SIZE)/2)+j);
			if(temp_value!=0){value4+=temp_value;num_element4++;}
			}
		}
	

	if(num_element1!=0)value1=value1/(double)num_element1; else value1=0;
	if(num_element2!=0)value2=value2/(double)num_element2; else value2=0;
	if(num_element3!=0)value3=value3/(double)num_element3; else value3=0;
	if(num_element4!=0)value4=value4/(double)num_element4; else value4=0;

	if(value1>value2){quad=1;temp_value=value1;}else {quad=2;temp_value=value2;}
	if(temp_value<value3){quad=3;temp_value=value3;}
	if(temp_value<value4)quad=4;

	printf("kuadran 1 = %.2lf, kuadran 2 = %.2lf, kuadran 3 = %.2lf, kuadran 4 = %.2lf \n",value1,value2,value3,value4);

return quad;

}
