// -*- C++ -*-
/*!
 * @file  StateMachine.cpp
 * @brief State machine
 * @date $Date$
 *
 * $Id$
 */

#include "StateMachine.h"

// Module specification
// <rtc-template block="module_spec">
static const char* statemachine_spec[] =
  {
    "implementation_id", "StateMachine",
    "type_name",         "StateMachine",
    "description",       "State machine",
    "version",           "0.0.1",
    "vendor",            "AISL.TUT",
    "category",          "StateMachin",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.scenario_path", "scenario.xml",
    "conf.default.tcp_port", "8080",
    // Widget
    "conf.__widget__.scenario_path", "text",
    "conf.__widget__.tcp_port", "text",
    // Constraints
    "conf.__constraints__.tcp_port", "x>=1024,x<=65535",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
StateMachine::StateMachine(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inputIn("input", m_input),
    m_outputOut("output", m_output)

    // </rtc-template>
{
	sys::SocketBase::initializeWinSock();
}

/*!
 * @brief destructor
 */
StateMachine::~StateMachine()
{
}



RTC::ReturnCode_t StateMachine::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("input", m_inputIn);
  
  // Set OutPort buffer
  addOutPort("output", m_outputOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("scenario_path", m_scenario_path, "scenario.xml");
  bindParameter("tcp_port", m_tcp_port, "8080");
  
  // </rtc-template>
	return RTC::RTC_OK;
}


RTC::ReturnCode_t StateMachine::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t StateMachine::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t StateMachine::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t StateMachine::onActivated(RTC::UniqueId ec_id)
{
	if( initializeStateMachineAndHTTPC( m_scenario_path.c_str(), m_tcp_port ) )
		return RTC::RTC_ERROR;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t StateMachine::onDeactivated(RTC::UniqueId ec_id)
{
	if( finalizeStateMachineAndHTTPC() )
		return RTC::RTC_ERROR;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t StateMachine::onExecute(RTC::UniqueId ec_id)
{	
	
	process();

  return RTC::RTC_OK;
}


RTC::ReturnCode_t StateMachine::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t StateMachine::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t StateMachine::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t StateMachine::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t StateMachine::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void StateMachineInit(RTC::Manager* manager)
  {
    coil::Properties profile(statemachine_spec);
    manager->registerFactory(profile,
                             RTC::Create<StateMachine>,
                             RTC::Delete<StateMachine>);
  }
  
};


