/// DLL form ikfast header file
/// \author Junichi Sugiyama
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
///
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// ikfast version 41 generated on 2011-05-11 20:09:47.554272
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object:
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -shared -Wl,-soname,ik.so -o ik.so ik.cpp
#pragma once

#include <vector>

#ifndef IKFAST_ASSERT
#include <stdexcept>
#include <sstream>

#ifdef _MSC_VER
#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __FUNCDNAME__
#endif
#endif

#ifndef __PRETTY_FUNCTION__
#define __PRETTY_FUNCTION__ __func__
#endif

#define IKFAST_ASSERT(b) { if( !(b) ) { std::stringstream ss; ss << "ikfast exception: " << __FILE__ << ":" << __LINE__ << ": " <<__PRETTY_FUNCTION__ << ": Assertion '" << #b << "' failed"; throw std::runtime_error(ss.str()); } }
#endif

#ifdef IK2PI
#undef IK2PI
#endif
#ifdef IKPI
#undef IKPI
#endif
#ifdef IKPI_2
#undef IKPI_2
#endif

#define IK2PI  ((IKReal)6.28318530717959)
#define IKPI  ((IKReal)3.14159265358979)
#define IKPI_2  ((IKReal)1.57079632679490)

// defined when creating a shared object/dll
#ifdef IKFAST_CLIBRARY
#ifdef _MSC_VER
#define IKFAST_API extern "C" __declspec(dllexport)
#else
#define IKFAST_API extern "C"
#endif
#else
#define IKFAST_API extern "C" __declspec(dllimport)
#endif

using namespace std;			 // necessary to get std math routines

#ifndef IKFAST_NAMESPACE
#define IKFAST_NAMESPACE ikfast
#endif
#ifdef IKFAST_NAMESPACE
namespace IKFAST_NAMESPACE
{
	#endif

	#ifndef	IKFAST_REAL_DEFINED	//J.Sugiyama
	#define IKFAST_REAL_DEFINED	//J.Sugiyama
	#ifdef IKFAST_REAL
	typedef IKFAST_REAL IKReal;
	#else
	typedef double IKReal;
	#endif
	#endif	//J.Sugiyama

	#ifndef	IKFAST_IKSOLUTION_DEFINED	//J.Sugiyama
	#define IKFAST_IKSOLUTION_DEFINED	//J.Sugiyama
	class IKSolution
	{
	public:
		/// Gets a solution given its free parameters
		/// \param pfree The free parameters required, range is in [-pi,pi]
		void GetSolution(IKReal* psolution, const IKReal* pfree) const
		{
			for(std::size_t i = 0; i < basesol.size(); ++i)
			{
				if( basesol[i].freeind < 0 )
					psolution[i] = basesol[i].foffset;
				else
				{
					IKFAST_ASSERT(pfree != NULL);
					psolution[i] = pfree[basesol[i].freeind]*basesol[i].fmul + basesol[i].foffset;
					if( psolution[i] > IKPI )
					{
						psolution[i] -= IK2PI;
					}
					else if( psolution[i] < -IKPI )
					{
						psolution[i] += IK2PI;
					}
				}
			}
		}

		/// Gets the free parameters the solution requires to be set before a full solution can be returned
		/// \return vector of indices indicating the free parameters
		const std::vector<int>& GetFree() const { return vfree; }

		struct VARIABLE
		{
			VARIABLE() : freeind(-1), fmul(0), foffset(0) {}
			VARIABLE(int freeind, IKReal fmul, IKReal foffset) : freeind(freeind), fmul(fmul), foffset(foffset) {}
			int freeind;
								///< joint value is fmul*sol[freeind]+foffset
			IKReal fmul, foffset;
		};

								///< solution and their offsets if joints are mimiced
		std::vector<VARIABLE> basesol;
		std::vector<int> vfree;
	};
	#endif	//J.Sugiyama

	namespace hiro
	{
		/// solves the forward kinematics equations.
		/// \param pfree is an array specifying the free joints of the chain.
		IKFAST_API void fkL(const IKReal* j, IKReal* eetrans, IKReal* eerot);
		IKFAST_API int getNumFreeParametersL();
		IKFAST_API int* getFreeParametersL();
		IKFAST_API int getNumJointsL();
		IKFAST_API int getIKRealSizeL();
		IKFAST_API int getIKTypeL();

		/// solves the inverse kinematics equations.
		/// \param pfree is an array specifying the free joints of the chain.
		IKFAST_API bool ikL(const IKReal* eetrans, const IKReal* eerot, const IKReal* pfree, std::vector<IKSolution>& vsolutions);
		IKFAST_API const char* getKinematicsHashL();
	}

	#ifdef IKFAST_NAMESPACE
}								 // end namespace
#endif
