/**********************************************************/
/* OpenCV Wrapper                                         */
/*  @ Author        : J. Sugiyama @ AISL.TUT              */
/*  @ Last update   : 24.Mar.2012                         */
/*--------------------------------------------------------*/
/*	Comment added                             01.Feb.2012 */
/*  OpenCV2.3.1 available                     24.Mar.2012 */
/**********************************************************/
#pragma once


#include <opencv/cv.h>
#include <opencv/cxcore.h>
#include <opencv/highgui.h>
#include <vector>


#if defined( CVUTIL_LINK_LIBS_200 )
#	pragma comment( lib, "cv200.lib" )
#	pragma comment( lib, "cxcore200.lib" )
#	pragma comment( lib, "highgui200.lib" )
#elif defined( CVUTIL_LINK_LIBS_210 )
#	ifdef _DEBUG
#		pragma comment( lib, "cv210d.lib" )
#		pragma comment( lib, "cxcore210d.lib" )
#		pragma comment( lib, "highgui210d.lib" )
#	else
#		pragma comment( lib, "cv210.lib" )
#		pragma comment( lib, "cxcore210.lib" )
#		pragma comment( lib, "highgui210.lib" )
#	endif
#elif defined( CVUTIL_LINK_LIBS_220 )
#	ifdef _DEBUG
#		pragma comment( lib, "opencv_core220d.lib" )
#		pragma comment( lib, "opencv_imgproc220d.lib" )
#		pragma comment( lib, "opencv_highgui220d.lib" )
#		pragma comment( lib, "opencv_video220d.lib" )
#	else
#		pragma comment( lib, "opencv_core220.lib" )
#		pragma comment( lib, "opencv_imgproc220.lib" )
#		pragma comment( lib, "opencv_highgui220.lib" )
#		pragma comment( lib, "opencv_calib3d220.lib" )
#		pragma comment( lib, "opencv_contrib220.lib" )
#		pragma comment( lib, "opencv_features2d220.lib" )
#		pragma comment( lib, "opencv_ffmpeg220.lib" )
#		pragma comment( lib, "opencv_flann220.lib" )
#		pragma comment( lib, "opencv_gpu220.lib" )
#		pragma comment( lib, "opencv_legacy220.lib" )
#		pragma comment( lib, "opencv_ml220.lib" )
#		pragma comment( lib, "opencv_objdetect220.lib" )
#		pragma comment( lib, "opencv_ts220.lib" )
#		pragma comment( lib, "opencv_video220.lib" )
#	endif
#elif defined( CVUTIL_LINK_LIBS_230 )
#	ifdef _DEBUG
#		pragma comment( lib, "opencv_core230d.lib" )
#		pragma comment( lib, "opencv_imgproc230d.lib" )
#		pragma comment( lib, "opencv_highgui230d.lib" )
#	else
#		pragma comment( lib, "opencv_core230.lib" )
#		pragma comment( lib, "opencv_imgproc230.lib" )
#		pragma comment( lib, "opencv_highgui230.lib" )
#		pragma comment( lib, "opencv_calib3d230.lib" )
#		pragma comment( lib, "opencv_contrib230.lib" )
#		pragma comment( lib, "opencv_features2d230.lib" )
//#		pragma comment( lib, "opencv_flann230.lib" )
#		pragma comment( lib, "opencv_gpu230.lib" )
#		pragma comment( lib, "opencv_legacy230.lib" )
#		pragma comment( lib, "opencv_ml230.lib" )
#		pragma comment( lib, "opencv_objdetect230.lib" )
#		pragma comment( lib, "opencv_video230.lib" )
#	endif
#elif defined( CVUTIL_LINK_LIBS_231 )
#	ifdef _DEBUG
#		pragma comment( lib, "opencv_core231d.lib" )
#		pragma comment( lib, "opencv_imgproc231d.lib" )
#		pragma comment( lib, "opencv_highgui231d.lib" )
#	else
#		pragma comment( lib, "opencv_core231.lib" )
#		pragma comment( lib, "opencv_imgproc231.lib" )
#		pragma comment( lib, "opencv_highgui231.lib" )
#		pragma comment( lib, "opencv_calib3d231.lib" )
#		pragma comment( lib, "opencv_contrib231.lib" )
#		pragma comment( lib, "opencv_features2d231.lib" )
//#		pragma comment( lib, "opencv_flann231.lib" )
#		pragma comment( lib, "opencv_gpu231.lib" )
#		pragma comment( lib, "opencv_legacy231.lib" )
#		pragma comment( lib, "opencv_ml231.lib" )
#		pragma comment( lib, "opencv_objdetect231.lib" )
#		pragma comment( lib, "opencv_video231.lib" )
#	endif
#endif


namespace opencv
{
namespace cvutil
{
	inline unsigned char deg2hue( int deg ) { return deg >> 1; }

#if defined( WIN32 ) || defined( __WIN32__ )
	enum channel_t { BLUE = 0, GREEN, RED, GRAY = 0 };
#else
	enum channel_t { RED = 0, GREEN, BLUE, GRAY = 0 };
#endif
	
	namespace Pixel {
		template <int> struct CvImageElemTypeTraits;
		template <>    struct CvImageElemTypeTraits< IPL_DEPTH_1U >  { typedef bool           ImageElemType; };
		template <>    struct CvImageElemTypeTraits< IPL_DEPTH_8U >  { typedef unsigned char  ImageElemType; };
		template <>    struct CvImageElemTypeTraits< IPL_DEPTH_8S >  { typedef char           ImageElemType; };
		template <>    struct CvImageElemTypeTraits< IPL_DEPTH_16U > { typedef unsigned short ImageElemType; };
		template <>    struct CvImageElemTypeTraits< IPL_DEPTH_16S > { typedef short          ImageElemType; };
		template <>    struct CvImageElemTypeTraits< IPL_DEPTH_32S > { typedef int            ImageElemType; };
		template <>    struct CvImageElemTypeTraits< IPL_DEPTH_32F > { typedef float          ImageElemType; };
		template <>    struct CvImageElemTypeTraits< IPL_DEPTH_64F > { typedef double         ImageElemType; };
	}

	template < int IPL_DEPTH >
	class Image
	{
	private:
		// type of an element of this Image
		typedef typename Pixel::CvImageElemTypeTraits< IPL_DEPTH >::ImageElemType ImageElemType;

		// basic IplImage
		IplImage *ipl;
	
	public:
		// Default constructor
		Image() : ipl( NULL )
		{}
		
		// Constructor (cvCreateImage())
		Image( const char *file_name, int flag ) : ipl( NULL )
		{ load( file_name, flag ); }

		// Constructor (cvCreateImage())
		Image( int w, int h, int ch ) : ipl( NULL )
		{ create(w, h, ch); }

		// Constructor (cvCloneImage())
		// Same as copy constructor (pointer)
		Image( const Image *img ) : ipl( NULL )
		{ set( img ); }

		// Copy constructor (cvCloneImage())
		Image( const Image &img ) : ipl( NULL )
		{ set( img ); }


		// Destructor (cvReleaseImage())
		~Image()
		{ release(); }

	public:
		// cvSetZero()
		inline void clear()
		{ cvSetZero(ipl); }
		
		
		// cvLoadImage()
		inline void load( const char *file_name, int flag )
		{
			if( ipl != NULL )	release();
			ipl = cvLoadImage( file_name, flag );
		}

		
		// cvCreateImage()
		inline void create( int w, int h, int ch )
		{
			if( ipl != NULL )	release();
			ipl = cvCreateImage( cvSize( w, h ), IPL_DEPTH, ch );
		}
	

		// cvRelaseImage()
		inline void release()
		{
			if( ipl != NULL )	cvReleaseImage( &ipl );
			ipl = NULL;
		}


	public:
		// cvCloneImage() or cvCopy() [Pointer version]
		inline void set( const Image *img )
		{
			// different image size -> cvCloneImage() (size fitting to)
			if( ipl == NULL || getWidth() != img->getWidth()
				|| getHeight() != img->getHeight() )
			{
				release();
				ipl = cvCloneImage( *img );
			}
			// same image size -> cvCopyImage()
			else	copy( img );
		}
		// cvCloneImage() or cvCopy() [Reference version]
		inline void set( const Image &img )
		{
			if( ipl == NULL || getWidth() != img.getWidth()
				|| getHeight() != img.getHeight() ) {
				release();
				ipl = cvCloneImage( img );
			}
			else	copy( img );
		}


		// cvCopyImage()  [Pointer version]
		inline void copy( const Image *src )
		{ cvCopyImage( *src, ipl ); }


		// cvCopyImage()  [Reference version]
		inline void copy( const Image &src )
		{ cvCopyImage( src, ipl ); }


		// cvFlip() 
		inline void copyFlip( const Image &src )
		{ cvFlip(src, ipl); }


	public:
		inline void copyChannel(
			const Image &src, int src_u0, int src_v0, int src_ch,
			int dst_u0, int dst_v0, int dst_ch, int w, int h )
		{
			Image &dst = *this;
			const int SRC_CH_N = src.ipl->nChannels;
			const int DST_CH_N = ipl->nChannels;
			const ImageElemType *src_row_p;
			ImageElemType *dst_row_p;
			for( int v = 0; v < h; ++ v, ++ src_v0, ++ dst_v0 ) {
				src_row_p = &src( src_u0, src_v0, src_ch );
				dst_row_p = &dst( dst_u0, dst_v0, dst_ch );
				for( int u = 0; u < w; ++ u, src_row_p += SRC_CH_N, dst_row_p += DST_CH_N ) {
					*dst_row_p = *src_row_p;
				}
			}
		}
		inline void copyChannel( const Image &src, int src_ch, int dst_ch )
		{ copyChannel( src, 0, 0, src_ch, 0, 0, dst_ch, ipl->width, ipl->height ); }

	public:
		// cvCopyImage() [regional copy]
		inline void copy(
			const Image &src, int src_u0, int src_v0, int dst_u0, int dst_v0, int w, int h )
		{
			Image &dst = *this;
			const int byte = w * getNChannels() * sizeof( ImageElemType );
			const int dst_v1 = dst_v0 + h;
			for( dst_v0; dst_v0 < dst_v1; dst_v0 ++, src_v0 ++ )
				memcpy( &dst( dst_u0, dst_v0 ), &src( src_u0, src_v0), byte );
		}
		inline void copy( const Image &src, int src_u0, int src_v0 )
		{ copy( src, src_u0, src_v0, 0, 0, getWidth(), getHeight() ); }


	public:
		inline void setIplImage( IplImage *src )
		{ ipl = src; }


		inline void copyIplImage( const IplImage *src )
		{ cvCopyImage( src, ipl ); }

	public: // overloaded operators
		inline       IplImage *getIplImage()       { return ipl; }
		inline const IplImage *getIplImage() const { return ipl; }

		inline operator       IplImage *()       { return ipl; }
		inline operator const IplImage *() const { return ipl; }

		//inline operator IplImage **() { return &ipl; }
		//inline operator const IplImage **() { return &ipl; }

		inline operator       CvArr *()       { return ipl; }
		inline operator const CvArr *() const { return ipl; }
		

	public:
		inline const int getWidth()     const { return ipl->width; }
		inline const int getHeight()    const { return ipl->height; }
		inline const int getNChannels() const { return ipl->nChannels; }
		inline const int getWidthStep() const { return ipl->widthStep; }
		

		inline typename ImageElemType &operator()( int u, int v, int ch = 0 )
		{ return *(reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*v) + u*ipl->nChannels + ch); }
#if 0
		inline typename ImageElemType &W ( int u, int v, int ch = 0 ) {	return *( reinterpret_cast<ImageElemType*>( ipl->imageData + ipl->widthStep*v)     + (u-1)*ipl->nChannels + ch );	}
		inline typename ImageElemType &E ( int u, int v, int ch = 0 ) {	return *( reinterpret_cast<ImageElemType*>( ipl->imageData + ipl->widthStep*v)     + (u+1)*ipl->nChannels + ch );	}
		inline typename ImageElemType &N ( int u, int v, int ch = 0 ) {	return *( reinterpret_cast<ImageElemType*>( ipl->imageData + ipl->widthStep*(v-1)) +  u   *ipl->nChannels + ch );	}
		inline typename ImageElemType &S ( int u, int v, int ch = 0 ) {	return *( reinterpret_cast<ImageElemType*>( ipl->imageData + ipl->widthStep*(v+1)) +  u   *ipl->nChannels + ch );	}

		inline typename ImageElemType &NW( int u, int v, int ch = 0 ) { return *( reinterpret_cast<ImageElemType*>( ipl->imageData + ipl->widthStep*(v-1)) + (u-1)*ipl->nChannels + ch );	}
		inline typename ImageElemType &SW( int u, int v, int ch = 0 ) { return *( reinterpret_cast<ImageElemType*>( ipl->imageData + ipl->widthStep*(v+1)) + (u-1)*ipl->nChannels + ch );	}
		inline typename ImageElemType &NE( int u, int v, int ch = 0 ) { return *( reinterpret_cast<ImageElemType*>( ipl->imageData + ipl->widthStep*(v-1)) + (u+1)*ipl->nChannels + ch );	}
		inline typename ImageElemType &SE( int u, int v, int ch = 0 ) { return *( reinterpret_cast<ImageElemType*>( ipl->imageData + ipl->widthStep*(v+1)) + (u+1)*ipl->nChannels + ch );	}
#else
		inline typename ImageElemType &W ( int u, int v, int ch = 0 ) {	return (*this)( u-1, v, ch ); }
		inline typename ImageElemType &E ( int u, int v, int ch = 0 ) {	return (*this)( u+1, v, ch ); }
		inline typename ImageElemType &N ( int u, int v, int ch = 0 ) {	return (*this)( u, v-1, ch ); }
		inline typename ImageElemType &S ( int u, int v, int ch = 0 ) {	return (*this)( u, v+1, ch );	}

		inline typename ImageElemType &NW( int u, int v, int ch = 0 ) { return (*this)( u-1, v-1 ,ch ); }
		inline typename ImageElemType &SW( int u, int v, int ch = 0 ) { return (*this)( u-1, v+1 ,ch ); }
		inline typename ImageElemType &NE( int u, int v, int ch = 0 ) { return (*this)( u+1, v-1 ,ch ); }
		inline typename ImageElemType &SE( int u, int v, int ch = 0 ) { return (*this)( u+1, v+1 ,ch ); }
#endif
		inline const typename ImageElemType &operator()( int u, int v, int ch = 0 ) const
		{ return *( reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*v) + u*ipl->nChannels + ch); }
#if 0
		inline const typename ImageElemType &W ( int u, int v, int ch = 0 ) const {	return *(reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*v)     + (u-1)*ipl->nChannels + ch);	}
		inline const typename ImageElemType &E ( int u, int v, int ch = 0 ) const {	return *(reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*v)     + (u+1)*ipl->nChannels + ch);	}
		inline const typename ImageElemType &N ( int u, int v, int ch = 0 ) const {	return *(reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*(v-1)) +  u   *ipl->nChannels + ch);	}
		inline const typename ImageElemType &S ( int u, int v, int ch = 0 ) const {	return *(reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*(v+1)) +  u   *ipl->nChannels + ch);	}

		inline const typename ImageElemType &NW( int u, int v, int ch = 0 ) const { return *(reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*(v-1)) + (u-1)*ipl->nChannels + ch);	}
		inline const typename ImageElemType &SW( int u, int v, int ch = 0 ) const { return *(reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*(v+1)) + (u-1)*ipl->nChannels + ch);	}
		inline const typename ImageElemType &NE( int u, int v, int ch = 0 ) const { return *(reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*(v-1)) + (u+1)*ipl->nChannels + ch);	}
		inline const typename ImageElemType &SE( int u, int v, int ch = 0 ) const { return *(reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep*(v+1)) + (u+1)*ipl->nChannels + ch);	}
#else
		inline const typename ImageElemType &W ( int u, int v, int ch = 0 ) const {	return (*this)( u-1, v, ch ); }
		inline const typename ImageElemType &E ( int u, int v, int ch = 0 ) const {	return (*this)( u+1, v, ch ); }
		inline const typename ImageElemType &N ( int u, int v, int ch = 0 ) const {	return (*this)( u, v-1, ch ); }
		inline const typename ImageElemType &S ( int u, int v, int ch = 0 ) const {	return (*this)( u, v+1, ch );	}

		inline const typename ImageElemType &NW( int u, int v, int ch = 0 ) const { return (*this)( u-1, v-1 ,ch ); }
		inline const typename ImageElemType &SW( int u, int v, int ch = 0 ) const { return (*this)( u-1, v+1 ,ch ); }
		inline const typename ImageElemType &NE( int u, int v, int ch = 0 ) const { return (*this)( u+1, v-1 ,ch ); }
		inline const typename ImageElemType &SE( int u, int v, int ch = 0 ) const { return (*this)( u+1, v+1 ,ch ); }
#endif

		//inline typename ImageElemType *getImageData ( int u, int v, int ch = 0 )
		//{ return (reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep * v) + u * ipl->nChannels + ch); }
		//inline const typename ImageElemType *getImageDataAsConst ( int u, int v, int ch = 0 ) const
		//{ return (reinterpret_cast<ImageElemType*>(ipl->imageData + ipl->widthStep * v) + u * ipl->nChannels + ch); }
		
		//inline typename ImageElemType *getImageData () { return (ImageElemType *) ipl->imageData; }		
		//inline const typename ImageElemType *getImageDataAsConst () { return (ImageElemType *) ipl->imageData; }
		//inline const void *getImageDataAsConst () const { return ipl->imageData; }

		inline bool isIn( int u, int v ) const
		{ return 0 <= u && u < ipl->width && 0 <= v && v < ipl->height; }

		inline bool isNull() const
		{	return ( ipl == NULL ) ? true : false;	}

	public:
		// cvNamedWindow(), cvShowImage()
		inline void show( const char *title ) const
		{ cvNamedWindow(title), cvShowImage(title, ipl); }

		// cvNamedWindow(), cvMoveWindow(), cvShowImage()
		inline void show( const char *title, int screen_u, int screen_v ) const
		{ cvNamedWindow( title ), cvMoveWindow( title, screen_u, screen_v ), cvShowImage( title, ipl ); }

		// cvSaveImage()
		inline void save( const char *file_name ) const { cvSaveImage( file_name, ipl ); }

	public:
		// cvFlip()
		inline void flip( int flip_mode = 0 )
		{ cvFlip( ipl, NULL, flip_mode ); }

		// cvCvtColor()
		inline void convert( Image &dst, int code )
		{	cvCvtColor( ipl, dst, code );	}

		//inline void scale( Image &dst, double scale ) 
		//{ cvConvertScale( ipl, dst, scale ); }
	};


	class Window
	{
	protected:
		const char *name;
	
	public:
		Window () : name(NULL)
		{}


		Window ( const char *window_name ) : name(window_name)
		{}


		virtual ~Window ()
		{	if(isAlive())	close();	}

	public:
		
		inline void display ( const Image<IPL_DEPTH_8U> &img ) const
		{	cvShowImage(name, img);	}


		inline int wait ( int delay = 0 ) const
		{	return cvWaitKey(delay);	}


		inline bool isAlive () const
		{	return (cvGetWindowHandle(name) == NULL) ? false : true;	}


		inline void open () { cvNamedWindow(name); }

		inline void move ( int u, int v ) {	cvMoveWindow(name, u, v); }

		inline void close () { cvDestroyWindow(name); }


		inline const char *getName () const { return name; }

	};


	class Camera
	{
	private:
		bool is_initialized;
		CvCapture *cap;
		opencv::cvutil::Image<IPL_DEPTH_8U> frm;

	public:
		Camera () { cap = NULL; }


		~Camera() { finalize(); }


		bool initialize ( int w = 640, int h = 480, int class_idx = CV_CAP_ANY )
		{
			if ( cap != NULL ) return false;
		
			if ( (cap = cvCaptureFromCAM(class_idx)) == NULL ) return false;

			cvSetCaptureProperty(cap, CV_CAP_PROP_FRAME_WIDTH, w);
			cvSetCaptureProperty(cap, CV_CAP_PROP_FRAME_HEIGHT, h);
		
			frm.setIplImage(NULL);

			return true;
		}


		void finalize ()
		{
			if ( cap == NULL ) return;
			cvReleaseCapture(&cap);
			frm.setIplImage(NULL);	// Avoid to call the destructor.
			cap = NULL;
		}


	public:
		const Image<IPL_DEPTH_8U> &query ()
		{
			frm.setIplImage(cvQueryFrame(cap));
			return frm;
		}
		
		
		// Lv`OɎsłȂ
		//const int getWidth() const{	return frm.getWidth();	}
		//const int getHeight() const{	return frm.getHeight();	}
	};


	class OpticalFlowProcessor
	{
	private:
		bool is_initialized;
		int w, h;
		int corner_count_preset;
		CvTermCriteria criteria_preset;
		Image<IPL_DEPTH_8U> gray_curr, gray_prev;
		opencv::cvutil::Image<IPL_DEPTH_8U> pyramid_prev, pyramid_curr;
		CvPoint2D32f *corners_prev, *corners_curr;
		Image<IPL_DEPTH_32F> eig_img, temp_img;
		char *status;

	public:
		OpticalFlowProcessor () { is_initialized = false; }


		~OpticalFlowProcessor () { finalize(); }
	
		bool isInitialized() const { return is_initialized; }

	private:
		OpticalFlowProcessor ( const OpticalFlowProcessor &ofp );
		OpticalFlowProcessor &operator = ( const OpticalFlowProcessor &ofp );

	public:
		bool initialize ( const Image<IPL_DEPTH_8U> &first_rgb, const CvTermCriteria criteria, const int corner_count )
		{
			if ( is_initialized )	return false;

			w = first_rgb.getWidth(), h = first_rgb.getHeight();
			gray_prev.create(w, h, 1);
			gray_curr.create(w, h, 1);
			pyramid_prev.create(w + 8, h / 3, 1);
			pyramid_curr.create(w + 8, h / 3, 1);

			eig_img.create(w, h, 1);
			temp_img.create(w, h, 1);

			corner_count_preset = corner_count;
			criteria_preset = criteria;

			/* 1t[ */
			cvCvtColor(first_rgb, (IplImage *) gray_prev, CV_BGR2GRAY);	// O[XP[
			corners_prev = new CvPoint2D32f[corner_count_preset];
			corners_curr = new CvPoint2D32f[corner_count_preset];
			status = new char[corner_count_preset];
			memset(status, 0, corner_count_preset);
			is_initialized = true;

			return true;
		}


		void finalize ()
		{
			if ( is_initialized ) {
				delete [] corners_curr;
				delete [] corners_prev;
				delete [] status;
			}
			is_initialized = false;
		}

		
		int process ( const Image<IPL_DEPTH_8U> &frm_rgb )
		{
			// ݂̃t[̃O[XP[
			cvCvtColor(frm_rgb, gray_curr, CV_BGR2GRAY);

			// aȓ_o
			int corner_count = corner_count_preset;
			cvGoodFeaturesToTrack(gray_prev, eig_img, temp_img, corners_prev, &corner_count, 0.01, 10, NULL, 3, 1);

			// IveBJt[vZ
			memset(status, 0, corner_count_preset);
			cvCalcOpticalFlowPyrLK(
				gray_prev, gray_curr,
				pyramid_prev, pyramid_curr,
				corners_prev, corners_curr, corner_count, cvSize(20, 20), 1, status, NULL, criteria_preset, 0);

			return corner_count;
		}

		
		void updateFrame ()
		{
			// XV
			IplImage *swap = gray_prev;
			gray_prev.setIplImage(gray_curr);
			gray_curr.setIplImage(swap);
		}
		

		inline char getStatus ( const int index ) { return status[index]; }


		inline CvPoint getCurrentPoint ( const int index ) { return cvPointFrom32f(corners_curr[index]); }


		inline CvPoint getPreviousPoint ( const int index ) { return cvPointFrom32f(corners_prev[index]); }


	};


}
}