#include "state-machine.h"
#include <iostream>


using namespace state_machine;
using namespace std;


/**********************************************************
	StateMachine::State::transition_t
===========================================================
	ftHgRXgN^
**********************************************************/
StateMachine::State::transition_t::transition_t()
	: on_manual( false )
{}


/**********************************************************
	Rs[RXgN^
**********************************************************/
StateMachine::State::transition_t::transition_t( 
	const transition_t &src )
	: input( src.input ), output( src.output ), 
	next( src.next ), on_manual( src.on_manual )
{}


/**********************************************************
	fXgN^
**********************************************************/
StateMachine::State::transition_t::~transition_t()
{}


/**********************************************************
	\
**********************************************************/
void StateMachine::State::transition_t::print(
	const char *white_spaces ) const
{
	cout << white_spaces << "<next in=\"" << input << "\" out=\"" << output << "\" timing=\"" << ( on_manual ? "manual>" : "auto>" ) << endl; 
	cout << white_spaces << "  " << next->getName() << endl;
	cout << white_spaces << "</next>" << endl; 
}


/**********************************************************
	StateMachine::State
===========================================================
	ftHgRXgN^
**********************************************************/
StateMachine::State::State()
	: id( -1 ), error( NULL )
{}


/**********************************************************
	Rs[RXgN^
**********************************************************/
StateMachine::State::State( const State &src )
	: id( src.id ), name( src.name ), 
	trans_tbl( src.trans_tbl ), 
	killed_on_error( src.killed_on_error ), error( NULL )
{}


/**********************************************************
	fXgN^
**********************************************************/
StateMachine::State::~State()
{
	finalize();
}


/**********************************************************
	
**********************************************************/
bool StateMachine::State::initialize(
	int id, const std::string &name )
{
	this->id = id;
	this->name = name;
	error = NULL;
	return true;
}
bool StateMachine::State::setTransitionTable( 
	const std::vector< transition_t > &trans_tbl )
{
	this->trans_tbl = trans_tbl;
	return true;
}
bool StateMachine::State::setRTCsKilledOnError( 
	const std::vector< std::string > &kill_list )
{
	killed_on_error = kill_list;
	return true;
}


/**********************************************************
	I
**********************************************************/
bool StateMachine::State::finalize()
{
	if( id != -1 ) {
		id = -1;
		name = "";
		trans_tbl.clear();
		error = NULL;
	}
	return true;
}


/**********************************************************
	\
**********************************************************/
void StateMachine::State::print() const
{
	cout << "<state id=\"" << id << "\" >" << endl;
	cout << "  <name>" << name << "</name>" << endl;
	for( size_t i = 0; i < trans_tbl.size(); ++ i )
		trans_tbl.at(i).print( "  " );
	//printf( "error = %p\n", error );
	if( error )
		cout << "  <error>" << error->getName() << "</error>" << endl;

	if( killed_on_error.size() ) {
		cout << "  <kill>" << endl;
		for( vector< string >::const_iterator cit = killed_on_error.begin();
			cit != killed_on_error.end(); ++ cit )
			cout << "    <rtc>" << *cit << "</rtc>" << endl;
		cout << "  </kill>" << endl;
	}

	cout << "<state>" << endl;
}


/**********************************************************
	ԑJڂɕKvȓ͂T
		݂̎́CPȓ͕̈vɂT
		͂ɑꍇɂ́CnbVȂǂōKv邩
**********************************************************/
int StateMachine::State::getRequiredInput(
	string &required_input ) const
{
	int i, n = (int) trans_tbl.size();
	for( i = 0; i < n; ++ i ) {
		printf( "[%2d]: %s\n", i, trans_tbl.at(i).next->getName().c_str() );
	}
	char buf[256] = {};
	fgets( buf, 255, stdin );
	int idx = atoi( buf );

	if( 0 <= idx && idx < n ) {
		required_input = trans_tbl.at(idx).input;
		return idx;
	}
	return -1;
}
int StateMachine::State::getRequiredInput(
	const string &next_state, string &required_input ) const
{
	int idx, n = (int) trans_tbl.size();
	//cout << getName() << endl;
	for( idx = 0; idx < n; ++ idx ) {
		//cout << trans_tbl.at(idx).next->getName() << "  " << next_state << endl;
		if( trans_tbl.at(idx).next->getName() == next_state ) {
			required_input = trans_tbl.at(idx).input;
			cout << required_input << endl;
			return idx;
		}
	}
	return -1;
}


#if 0
/**********************************************************
	~RTCXgRTC~
**********************************************************/
bool StateMachine::State::killRTCs()
{
	if( !killed_on_error.size() ) 
		return false;

	sys::
	bool rc = true;
	for( vector< string >::const_iterator cit = killed_on_error.begin();
		cit != killed_on_error.end(); ++ cit )
	{
		
	}

}
#endif