/*********************************************************/
/* Event (WinAPI)                                        */
/* @ Author: J.Sugiyama@AISL.TUT                         */
/* @ Date  : Mar.14.2012                                 */
/* @ Release note:                                       */
/*   Git test                                Mar.14.2012 */
/*********************************************************/
#pragma once


#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

#include <windows.h>
#include "../../win32api/win32api-strconv.h"


namespace sys
{

	//=============================================
	//
	//	Event class (NOT INHERITABLE)
	//
	//=============================================
	class Event
	{
	private:
		HANDLE event;

	public:	// Constructor & destructor
		/**
			Default constructor
		*/
		Event()
		{ event = INVALID_HANDLE_VALUE; }


		/**
			Destructor
		*/
		~Event()
		{
			std::cerr << ">> ~Event()" << std::endl;
			finalize();
			std::cerr << "<< ~Event()" << std::endl;
		}

	private: // Forbidden
		Event( const Event &e );
		Event &operator =( const Event &e );

	public:	// Functions
		/**
			Initialize 
			@	<I>	manual_reset	: true; event is resetted manually/ false; done automatically.
			@	<I>	init_stat			:	initial event state (set or reset).
		*/
		bool initialize( bool manual_reset, bool init_stat, const char *name = NULL )
		{
			if( event != INVALID_HANDLE_VALUE ) return false;
#ifdef UNICODE
			wchar_t *name_wcs = NULL;
			if( name ) {
				name_wcs = new wchar_t [ strlen(name) + 1 ];
				win32api::strconv::char2wchar( name, name_wcs );
			}
			event = CreateEvent( NULL, manual_reset ? TRUE : FALSE, init_stat ? TRUE : FALSE, name_wcs );
			if( name_wcs )	delete [] name_wcs;
#else
			event = CreateEvent( NULL, manual_reset ? TRUE : FALSE, init_stat ? TRUE : FALSE, name );
#endif

			return event == INVALID_HANDLE_VALUE ? false : true;
		}


		/**
			Finalize
		*/
		bool finalize()
		{
			if( event == INVALID_HANDLE_VALUE ) return false;
			CloseHandle( event );
			event = INVALID_HANDLE_VALUE;
			return true;
		}


		/**
			Post this event.
		*/
		void post()
		{ SetEvent( event ); }


		/**
			Reset this event.
		*/
		void reset()
		{ ResetEvent( event ); }


		/**
			Wait for this event.
			@	<I> time_ms	: time to wait in milli-seconds.
			@	return			: status code
		*/
		DWORD wait( DWORD time_ms = INFINITE )
		{ return WaitForSingleObject( event, time_ms ); }

	};


}