#pragma once


#include <cstdlib>
#include <cstdio>
#include <conio.h>
#include <windows.h>


namespace sys
{
	inline void exit( const char *msg = NULL )
	{
		if( msg )
			fprintf( stderr, "%s\n", msg );

		fprintf( stderr, "Press 'return' key to exit..." );
		getchar();
		::exit( EXIT_FAILURE );
	}

	inline void exitWithPerror( const char *msg )
	{
		::perror( msg );

		fprintf( stderr, "Press 'return' key to exit..." );
		getchar();
		::exit( EXIT_FAILURE );
	}

	template< typename T >
	inline T returnByError( T rc, const char *msg = NULL, bool wait_for_type = false )
	{
		if( msg )
			fprintf( stderr, "%s\n", msg );

		if( wait_for_type ) {
			fprintf( stderr, "Press 'return' key to exit..." );
			getchar();
		}
		return rc;
	}

	
	/**
		R\[
		http://www.masukawa.co.jp/sdk/10.html
	*/
	class ConsoleController
	{
	private:
		HANDLE h;
		COORD p;
		CONSOLE_CURSOR_INFO i;

	public:
		enum color_t {
			BLACK = 0, GRAY_P = 7, GRAY_D, BLUE, GREEN, BLUE_P, RED, PINK, YELLOW, WHITE
		};

	public:
		ConsoleController() : h( GetStdHandle( STD_OUTPUT_HANDLE ) )
		{
			setColor();
			//setPosition( 0, 0 );
			showCursor();
		}
		~ConsoleController() {}

	public:
		/******************************************************************************
			Fݒ(F, wiF)
		******************************************************************************/
		void setColor( color_t fg = GRAY_P, color_t bg = BLACK )
		{
			SetConsoleTextAttribute( h, fg | (bg << 4) );
		}

		/******************************************************************************
			ʒuݒ(w, x) X=0`79, Y=0`
		******************************************************************************/
		void setPosition( int x = 0, int y = 0 )
		{ 
			p.X = x; p.Y = y;
			SetConsoleCursorPosition( h, p );
		}

		/******************************************************************************
			J[\\(^U) 1:\  0:\
		******************************************************************************/
		void showCursor()
		{
			GetConsoleCursorInfo( h, &i );
			i.bVisible = 0;
			SetConsoleCursorInfo( h, &i );
		}

		/******************************************************************************
			J[\\(^U) 1:\  0:\
		******************************************************************************/
		void hideCursor()
		{
			GetConsoleCursorInfo( h, &i );
			i.bVisible = 1;
			SetConsoleCursorInfo( h, &i );
		}

		/******************************************************************************
			ʏ(wiF)
		******************************************************************************/
		void clear( color_t bg, int row_top = 0, int row_bottom = -1 )
		{
			DWORD w;
			COORD pi = { 0, row_top }, pt = { 0, row_bottom };

			CONSOLE_SCREEN_BUFFER_INFO i;
			GetConsoleScreenBufferInfo( h, &i );

			if( row_bottom == -1 )
				pt.Y = i.dwSize.Y;
			pt.X = i.dwSize.X;

			// pt.X - pi.X ͕K pt.X(=i.dwSize.X) ɂȂipi.X=0j
			const DWORD n = (pt.X-pi.X) * (pt.Y-pi.Y);
			FillConsoleOutputAttribute( h, bg << 4, n, pi, &w );
			FillConsoleOutputCharacter( h, ' ', n, pi, &w );
		}

	};

}