/*********************************************************/
/* Socket (TCP & UDP)(WinAPI)                            */
/* @ Author: J.Sugiyama@AISL.TUT                         */
/* @ Date  : Mar.15.2012                                 */
/* @ Release note:                                       */
/*   modified type of socket (int -> SOCKET) Jan.01.2012 */
/*   CRLF conversion                         Mar.15.2012 */
/*********************************************************/
#pragma once


#include <winsock2.h>
#include <ws2tcpip.h>
#include "../misc/misc.h"
#include <iostream>
#include <vector>
#include <string>


#pragma comment( lib, "ws2_32.lib" )


namespace sys
{
	/**
		addr_t addr ̎g
		<TCP> 
			[Active open / Client]
				ڑT[o̎w
			[Passive open / Server]
				bindɂڑ󂯓NCAg/Ҏ󂯃|[g̎w
				(IN_ADDR_ANYȂ)
		<UDP>
			[Active open / Client]
				recvfrom/sendtȏM/̎w
			[Passive open / Server]
				bindɂڑ󂯓NCAg/Ҏ󂯃|[g̎wC
				̌recvfrom/sendtȏM/̎w
				bind sockaddr_in  const |C^ĝŁC
				bindɕύXĂȂ͂
	*/
	class SocketBase
	{
	protected:
		struct addr_t
		{
			IN_ADDR name;
			SOCKADDR_IN addr_in;
		};

	protected:
		SOCKET sock;
		addr_t addr;

	public:
		/**
			Default Constructor
		*/
		SocketBase()
		{
			initialize();
		}


		/**
			Copy constructor
		*/
		SocketBase( const SocketBase &src )
		{
			std::cerr << ">> SocketBase()" << std::endl;
			initialize();

			sock = src.sock;
			addr = src.addr;
			std::cerr << "<< SocketBase()" << std::endl;
		}

		
		/**
			Destructor
		*/
		virtual ~SocketBase()
		{
			//std::cerr << ">> ~SocketBase()" << std::endl;
			// fXgN^s_Ń\PbgȂLł΃\Pbg
			if( sock != INVALID_SOCKET )
				close();
			sock = INVALID_SOCKET;
			//std::cerr << "<< ~SocketBase()" << std::endl;
		}


		/**
			Initialization (unique to WinSock of Windows OS)
		*/
		static void initializeWinSock()
		{
			WSADATA wsa_data;
			if( WSAStartup( MAKEWORD( 2, 2 ), &wsa_data ) != 0 )
				sys::exit( "WSAStartup() : failed." );

			if( atexit( (void (*)(void))(WSACleanup) ) )
				sys::exit( "atexit() : failed." );
		}


		/**
			Common initialization
		*/
		void initialize()
		{
			sock = INVALID_SOCKET;	// \Pbg
		}


	public:
		/**
			Open the socket
			@	return :	0;	successful / non 0; failure
		*/
		virtual int open() = 0;


		/**
			Close the socket
			@	return :	0;	successful / non 0; failure
		*/
		int close()
		{
			//cout << "close :" << sock << endl;
			if( sock != INVALID_SOCKET ) {
				::shutdown( sock, SD_BOTH );	// Mؒf
				closesocket( sock );
				// WSACleanup();						// atexit()ŒS
				sock = INVALID_SOCKET;	// \Pbg
			}
			else
				return sys::returnByError( -1, "SocketBase::close():  invalid socket." );
			
			return 0;
		}


	public: // For client side
		/**
			Establish the connection
			@	return :	0;	successful / non 0; failure
		*/
		int connect()
		{
			if( ::connect( sock, (SOCKADDR *) &addr.addr_in, sizeof(addr.addr_in) ) == SOCKET_ERROR )
				return sys::returnByError( -1, "SocketBase::connect() : failed.", true );
			return 0;
		}


	public: // For server side
		/**
			Bind the socket
			@	return :	0;	successful / non 0; failure
		*/
		int bind()
		{
			if( ::bind( sock, (SOCKADDR *) &addr.addr_in, sizeof(addr.addr_in) ) == SOCKET_ERROR )
				return sys::returnByError( -1, "SocketBase::bind() : failed.", true );
			return 0;
		}


	public: // Communication / For both sides
		/**
			Configure sockaddr_in
			NCAgł̃T[o̎w / T[oł̑҂󂯃AhX̐ݒ
			@	host_name	: zXg
			@	port			:	TCP|[gԍ
			@	return		:	0;	successful / non 0; failure
		*/
		int setAddressINet( const char *host_name, unsigned short port )
		{
			if( getIPAddress( host_name, &addr.name ) )
				return sys::returnByError( -1, "SocketBase::getSockAddressINet(): failed.", true );
			return setAddressINet( &addr.addr_in, addr.name.s_addr, htons(port) );
		}
		/**
			Configure sockaddr_in 
			NCAgł̃T[o̎w / T[oł̑҂󂯃AhX̐ݒ
			@	host_addr_inet	:	IPAhX
			@	short port			:	TCP|[gԍ
			@	return					:	0;	successful / non 0; failure
		*/
		int setAddressINet( unsigned long host_addr_inet, unsigned short port )
		{
			return setAddressINet( &addr.addr_in, htonl(host_addr_inet), htons(port) );
		}

		/**
			Send the message (primitive ver.)
			@ msg			:	the pointer to the message to be sent
			@	n_byte	:	the number of byte of msg
			@	return	:	the number of actually sent byte
		*/
		virtual int send( const char *msg, size_t n_byte ) = 0;


		/**
			Receive the message (primitive ver.)
			@	msg					:	the reference of the message to be stored the received message
			@	max_n_byte	:	the maximum number of byte of msg
			@	return			:	the number of actually received byte
		*/
		virtual int recv( char *msg, size_t max_n_byte ) = 0;


	public:
		/**
			\PbgLqq̎擾
		*/
		inline SOCKET getSocket()
		{
			return sock;
		}


		/**
			
		*/
		inline int setAddressINet(
			SOCKADDR_IN *sock_addr_in,
			unsigned int addr_nl, unsigned short port_ns )
		{
			memset( sock_addr_in, 0, sizeof(*sock_addr_in) );
			sock_addr_in->sin_family = AF_INET;				/* C^[lbgw */
			sock_addr_in->sin_addr.s_addr = addr_nl;	/* IPAhX */
			sock_addr_in->sin_port = port_ns;					/* TCP|[gԍ */

			return 0;
		}


		/**
			Solve the name
			@	host_name :
			@	return	:	0;	successful / non 0; failure
		*/
		inline int getIPAddress( const char *name, IN_ADDR *addr )
		{
			// host_name Ƀhbgŋ؂10iIPAhXĂꍇA
			// s_addr 32bitIPAhXԂ
			addr->s_addr = inet_addr( name );

			if( addr->s_addr == INADDR_NONE ) {
				// '.' ؂IPAhXłȂꍇ
				LPHOSTENT host;
				host = gethostbyname( name );
					// T[o(host_name)T[õzXg擾܂
				if( host == NULL )
					return sys::returnByError( -1, "SocketBase::getIPAddress(): cannot get host name.", true );
				else
					*addr = *( (IN_ADDR *) host->h_addr_list[0] );
						// T[õzXg񂩂IPAhXserveraddrɃRs[
			}
			return 0;
		}


		/**
			AhX擾
		*/
		inline std::string getIPAddress() const
		{
			return std::string( inet_ntoa(addr.addr_in.sin_addr) );
		}


		/**
			AhX\
		*/
		inline void printINetAddr() const
		{
			std::cout << inet_ntoa( addr.addr_in.sin_addr ) << std::endl;
		}

	};


	/**
		TCP\Pbg
	*/
	class TCPSocket : public SocketBase
	{
	public:
		/**
			Default Constructor
		*/
		TCPSocket()
		{
			initialize();
		}


		/**
			Copy constructor
		*/
		TCPSocket( const TCPSocket &src )
		{
			initialize();

			sock = src.sock;
			addr = src.addr;
		}

		
		/**
			Destructor
		*/
		~TCPSocket()
		{
			//std::cerr << ">> ~TCPSocket()" << std::endl;
			// fXgN^s_Ń\PbgȂLł΃\Pbg
			/* ~SocketBase() S
			if( sock != INVALID_SOCKET )
				close();
			*/
			//std::cerr << "<< ~TCPSocket()" << std::endl;
		}

	public:
		int open()
		{
			sock = (int) ::socket( AF_INET, SOCK_STREAM, 0 );
			if( sock == INVALID_SOCKET )
				return sys::returnByError( -1, "SocketBase::open(): socket() : failed.", true );
			//cout << "open :" << sock << endl;
			return 0;
		}
		

	public: // For client side


	public: // For server side
		/**
			Listen throuth the socket
			@	return :	0;	successful / non 0; failure
		*/
		int listen( int backlog )
		{
			if( ::listen( sock, backlog ) == SOCKET_ERROR )
				return sys::returnByError( -1, "TCPSocket::listen() : failed.", true );
			return 0;
		}


		/**
			Accept the connection through the socket
			@	Socket *sock_dst : pointer of the socket to be connected
			@	return :	0;	successful / non 0; failure
		*/
		int accept( TCPSocket *sock_dst )
		{
			socklen_t addr_size = sizeof( sock_dst->addr.addr_in );

			sock_dst->sock = ::accept( sock,
				(SOCKADDR *) &sock_dst->addr.addr_in, &addr_size );
			if( sock_dst->sock == INVALID_SOCKET )
				return returnByError( -1, "TCPSocket::accept() : failed." );
			
			/*
			std::cout
				<< "accept() : accepted from "
				<< inet_ntoa(sock_dst->sock_addr_in.sin_addr)
				<< " using socket[" << sock_dst->sock << "]"
				<< std::endl;
			*/
			return 0;
		}
		int accept( TCPSocket &sock_dst )
		{
			socklen_t addr_size = sizeof( sock_dst.addr.addr_in );

			sock_dst.sock = ::accept( sock, (SOCKADDR *) &sock_dst.addr.addr_in, &addr_size );
			if(	sock_dst.sock == INVALID_SOCKET )
				return returnByError( -1, "TCPSocket::accept() : failed." );
			/*
			std::cout
				<< "accept() : accepted from "
				<< sock_dst.getIPAddress()
				<< " using socket["	<< sock_dst.sock << "]"
				<< std::endl;
			*/
			return 0;
		}


	public: // Communication / For both sides
		/**
			Send the message (primitive ver.)
			@ msg			:	the pointer to the message to be sent
			@	n_byte	:	the number of byte of msg
			@	return	:	the number of actually sent byte
		*/
		int send( const char *msg, size_t n_byte )
		{
			//cout << "send1 : " << sock << endl;
			int byte =  ::send( sock, msg, (int)n_byte, 0 );
			if( byte == SOCKET_ERROR )
				return returnByError( -1, "TCPSocket::send() : failed." );
			//cout << "send2 : " << sock << ", " << strlen(msg) << endl;
			return byte;
		}
		/**
			Send the message (std::string ver.)
			@	msg			:	the reference of the message to be sent
			@	return	:	the byte number sent actually
		*/
		int send( const std::string &msg )
		{
			return send( msg.c_str(), msg.size() );
		}
		/**
			Send the message (std::vector ver.)
			@	msg			:	the reference of the message to be sent
			@	return	:	the number of actually sent byte
		*/
		template < typename T >
		int send( const std::vector< T > &msg, size_t byte_of_one_element = sizeof( T ) )
		{
			return send( (char *) &msg[0], msg.size() * byte_of_one_element );
		}


		/**
			Receive the message (primitive ver.)
			@	msg					:	the reference of the message to be stored the received message
			@	max_n_byte	:	the maximum number of byte of msg
			@	return			:	the number of actually received byte
		*/
		int recv( char *msg, size_t max_n_byte )
		{
			int recv_len = ::recv( sock, msg, (int) max_n_byte, 0 );
			if( recv_len == SOCKET_ERROR )
				return sys::returnByError( -1, "TCPSocket::recv() : failed." );
			return recv_len;
		}
		/**
			Receive the message (std::string ver.)
			@	msg			:	the reference of the message to be sent
			@	return	:	the number of actually sent byte
		*/
		//int recv(std::string &msg){	return recv(&msg[0], msg.size());	}
		/**
			Receive the message
			@	recv_msg	:	the reference of the message to be stored the received message
				note: it is necessary to allocate recv_msg. using resize() etc.
			@	return	:	0;	successful / non 0; failure
		*/
		template < typename T >
		int recv( std::vector<T> &msg, size_t byte_of_one_element = sizeof(T) )
		{
			return recv((char *) &msg[0], msg.size() * byte_of_one_element);
		}
	};


	/**
		UDP\Pbg
	*/
	class UDPSocket : public SocketBase
	{
	public:
		/**
			Default Constructor
		*/
		UDPSocket()
		{
			initialize();
		}


		/**
			Copy constructor
		*/
		UDPSocket( const UDPSocket &src )
		{
			initialize();
			sock = src.sock;
			addr = src.addr;
		}

		
		/**
			Destructor
		*/
		~UDPSocket()
		{
			// fXgN^s_Ń\PbgȂLł΃\Pbg
			/* ~SocketBase() S
			if( sock != INVALID_SOCKET )
				close();
			*/
		}


	public:
		int open()
		{
			if( (sock = (int)::socket(AF_INET, SOCK_DGRAM, 0)) < 0 )
				return sys::returnByError( -1, "SocketBase::open(): socket() : failed.", true );
			//cout << "open :" << sock << endl;
			return 0;
		}


	public: // For client side
		/**
			Configure sockaddr_in (NCAgł̃T[o̎w)
			@	host_name	: zXg
			@	port			:	TCP|[gԍ
			@	return		:	0;	successful / non 0; failure
		*/
		int setAddressToSend( const char *remote_name, unsigned short port )
		{
			if( getIPAddress( remote_name, &addr.name ) )
				return sys::returnByError( -1, "UDPSocket::getSockAddressINet():  failed.", true );
			return setAddressINet( &addr.addr_in, addr.name.s_addr, htons(port) );
		}


	public: // For server side


	public: // Communication / For both sides
		/**
			Send the message (primitive ver.)
			@ msg			:	the pointer to the message to be sent
			@	n_byte	:	the number of byte of msg
			@	return	:	the number of actually sent byte
		*/
		int send( const char *msg, size_t n_byte )
		{
			socklen_t addr_size = sizeof( addr.addr_in );
			//cout << "send1 : " << sock << endl;
			int byte = ::sendto( sock, msg, (int) n_byte, 0, (SOCKADDR *) &addr.addr_in, addr_size );
			if( byte == SOCKET_ERROR )
				return returnByError( -1, "UDPSocket::send() : failed." );
			//cout << "send2 : " << sock << ", " << strlen(msg) << endl;
			return byte;
		}
		/**
			Send the message (std::string ver.)
			@	msg			:	the reference of the message to be sent
			@	return	:	the number of actually sent byte
		*/
		//int send(const std::string &msg){	return send(msg.c_str(), msg.size());	}
		/**
			Send the message (std::vector ver.)
			@	msg			:	the reference of the message to be sent
			@	return	:	the number of actually sent byte
		*/
		template <typename T>
		int send( const std::vector<T> &msg, size_t byte_of_one_element = sizeof(T) )
		{
			return send( (char *) &msg[0], msg.size() * byte_of_one_element );
		}


		/**
			Receive the message (primitive ver.)
			@	msg					:	the reference of the message to be stored the received message
			@	max_n_byte	:	the maximum number of byte of msg
			@	return			:	the number of actually received byte
		*/
		int recv( char *msg, size_t max_n_byte )
		{
			socklen_t addr_size = sizeof( addr.addr_in );
			int recv_len = ::recvfrom( sock, msg, (int) max_n_byte, 0, (SOCKADDR *) &addr.addr_in, &addr_size );
			if( recv_len == SOCKET_ERROR )
				return sys::returnByError( -1, "UDPSocket::recv() : failed." );
			return recv_len;
		}
		/**
			Receive the message (std::string ver.)
			@	msg			:	the reference of the message to be sent
			@	return	:	the number of actually sent byte
		*/
		//int recv(std::string &msg){	return recv(&msg[0], msg.size());	}
		/**
			Receive the message
			@	recv_msg	:	the reference of the message to be stored the received message
				note: it is necessary to allocate recv_msg. using resize() etc.
			@	return	:	0;	successful / non 0; failure
		*/
		template <typename T>
		int recv( std::vector<T> &msg, size_t byte_of_one_element = sizeof(T) )
		{
			return recv( (char *) &msg[0], msg.size() * byte_of_one_element );
		}
	};


}