/*********************************************************/
/* Thread (WinAPI)                                       */
/* @ Author: J.Sugiyama@AISL.TUT                         */
/* @ Date  : Jan.18.2012                                 */
/* @ Release note:                                       */
/*   modified setPriority()                  Jan.18.2012 */
/*********************************************************/
#pragma once


#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif


#include <windows.h>
#include <process.h>
#include <iostream>


namespace sys
{


	//=============================================
	//
	//	Thread class (NOT INHERITABLE)
	//
	//=============================================
	class Thread
	{
	private:	// fields
		HANDLE handle;		// Thread handle
		unsigned int id;	// Thread ID

	public:	// Constructor & destructor
		/**
			default constructor
		*/
		Thread() : handle( INVALID_HANDLE_VALUE )
		{}


		/**
			Destructor
		*/
		~Thread()
		{
			std::cerr << ">> ~Thread()" << std::endl;
			if( handle != INVALID_HANDLE_VALUE ) stop();
			std::cerr << "<< ~Thread()" << std::endl;
		}

	private:	// Forbidden constructor
		Thread( const Thread &obj );
		Thread &operator =( const Thread &obj );

	public:	// Functions
		/**
			Create thread if handle is free.
			@	<I>		func	:
			@	<IO>	list	:
		*/
		bool start( unsigned int (__stdcall *func)(void *), void *arg_list )
		{
			if( handle != INVALID_HANDLE_VALUE ) return false;
			handle = (HANDLE) _beginthreadex(NULL, 0, func, arg_list, 0, &id);
			return true;
		}


		/**
			Close handle
		*/
		bool stop ()
		{
			if( handle == INVALID_HANDLE_VALUE ) return false;
			wait();
			CloseHandle(handle);
			handle = INVALID_HANDLE_VALUE;
			return true;
		}


		/**
			Check the signal state.
		*/
		bool isSignaled()
		{ return (WaitForSingleObject( handle, 0 ) == WAIT_OBJECT_0 ) ? true : false; }


		/**
			Test the handle state.
		*/
		bool isBusy()
		{ return (handle != INVALID_HANDLE_VALUE); }


		/**
			Wait for finishing thread.
		*/
		DWORD wait( DWORD time_ms = INFINITE )
		{ return WaitForSingleObject( handle, time_ms ); }

		/**
		*/
		unsigned int getId() const { return id; }
		
		
		/**
		*/
		void setPriority( int level )
		{ SetThreadPriority( handle, level ); }


		/**
		*/
		inline operator unsigned int () { return id; }
		
		void postEvent( int event_id ) const
		{	PostThreadMessage( id, event_id, 0, 0 );	}

	};


}